#include "curve_manager.hpp"
#include "main_window.hpp"


CurveManager :: CurveManager( CurrentImageType * ptr_ima )
{
  ptr_image = ptr_ima;

  ptr_perfil = NULL;

  is_linear = false;
  is_simpson = false;
  is_spline = false;
}


CurveManager :: ~CurveManager()
{
  if( ptr_image->IsDexelCreated() )
    {ptr_image->GetDexel()->GetCurveOpen( 0 )->UndoUpdateImageImplementation();
    ptr_image->GetDexel()->DestroyCurves();
    ptr_image->DestroyDexel();
    for( int i = 0; i < ptr_image->GetActor()->CountProgenies(); i ++ )
      if( !strcmp( ptr_image->GetActor()->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	((Cruz*)(ptr_image->GetActor()->GetProgeny( i )))->RemoveListenerGeometry( this );}

  if( ptr_perfil != NULL )
    delete ptr_perfil;
}


bool CurveManager :: IsLinearPossible( CurrentImageType * ptr_image )
{
  int count = 0;
  CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();
  
  for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
    if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
      count++;

  return( count >= 2 );
}


bool CurveManager :: IsSimpsonPossible( CurrentImageType * ptr_image )
{
  int count = 0;
  CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();

  for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
    if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
      count++;
  
  return( count >= 3 );
}


bool CurveManager :: IsSplinePossible( CurrentImageType * ptr_image )
{
  int count = 0;
  CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();

  for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
    if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
      count++;

  return( count >= 4 );
}


void CurveManager :: ActorResized( bicotiActor * ptr_actor, const bicotiEventGeometry& event )
{}


void CurveManager :: ActorMoved( bicotiActor * ptr_actor, const bicotiEventGeometry& evento )
{
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "cruz" ) )
    RefreshCurve();
}


bool CurveManager :: IsCurrentPossible()
{
  if( is_linear )
    return( IsLinearPossible( ptr_image ) );

  if( is_simpson )
    return( IsSimpsonPossible( ptr_image ) );

  if( is_spline )
    return( IsSplinePossible( ptr_image ) );

  return( false );
}


void CurveManager :: CreateLinear()
{
  if( !is_linear )
    {CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();
    
    vector< CurrentImageType :: RealCoordinate > vect;
    
    for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
      if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	{vect.push_back( CurrentImageType :: RealCoordinate( ((Cruz*)ptr_actor->GetProgeny( i ))->CentroX(), ((Cruz*)ptr_actor->GetProgeny( i ))->CentroY() ) );
	((Cruz*)(ptr_actor->GetProgeny( i )))->AddListenerGeometry( this );}
    
    if( ptr_image->IsDexelCreated() )
      {UnPaintCurve();
      ptr_image->GetDexel()->DestroyCurves();}
    else
      ptr_image->BuildDexel();
    
    ptr_image->GetDexel()->BuildNewCurveOpenLinear( vect );

    PaintCurve();
    PaintPerfil();

    is_linear = true;
    is_simpson = false;
    is_spline = false;}
}


void CurveManager :: CreateSimpson()
{
  if( !is_simpson )
    {CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();

    vector< CurrentImageType :: RealCoordinate > vect;
    
    for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
      if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	{vect.push_back( CurrentImageType :: RealCoordinate( ((Cruz*)ptr_actor->GetProgeny( i ))->CentroX(), ((Cruz*)ptr_actor->GetProgeny( i ))->CentroY() ) );
	((Cruz*)(ptr_actor->GetProgeny( i )))->AddListenerGeometry( this );}
    
    if( ptr_image->IsDexelCreated() )
      {UnPaintCurve();
      ptr_image->GetDexel()->DestroyCurves();}
    else
      ptr_image->BuildDexel();
    
    ptr_image->GetDexel()->BuildNewCurveOpenSimpson( vect );

    PaintCurve();
    PaintPerfil();

    is_linear = false;
    is_simpson = true;
    is_spline = false;}
}


void CurveManager :: CreateSpline()
{
  if( !is_spline )
    {CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();

    vector< CurrentImageType :: RealCoordinate > vect;
    
    for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
      if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	{vect.push_back( CurrentImageType :: RealCoordinate( ((Cruz*)ptr_actor->GetProgeny( i ))->CentroX(), ((Cruz*)ptr_actor->GetProgeny( i ))->CentroY() ) );
	((Cruz*)(ptr_actor->GetProgeny( i )))->AddListenerGeometry( this );}

    
    if( ptr_image->IsDexelCreated() )
      {UnPaintCurve();
      ptr_image->GetDexel()->DestroyCurves();}
    else
      ptr_image->BuildDexel();
    
    ptr_image->GetDexel()->BuildNewCurveOpenSpline( vect );

    PaintCurve();
    PaintPerfil();

    is_linear = false;
    is_simpson = false;
    is_spline = true;}
}


void CurveManager :: RefreshCurve()
{
  CurrentImageType :: Actor * ptr_actor = ptr_image->GetActor();

  vector< CurrentImageType :: RealCoordinate > vect;
  
  for( int i = 0; i < ptr_actor->CountProgenies(); i ++ )
    if( !strcmp( ptr_actor->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
      vect.push_back( CurrentImageType :: RealCoordinate( ((Cruz*)ptr_actor->GetProgeny( i ))->CentroX(), ((Cruz*)ptr_actor->GetProgeny( i ))->CentroY() ) );

  UnPaintCurve();
  
  ptr_image->GetDexel()->GetCurveOpen( 0 )->ListCoordinates() = vect;
  ptr_image->GetDexel()->GetCurveOpen( 0 )->UpdateDexel();
  
  PaintCurve();
}


void CurveManager :: RefreshPerfil()
{
  if( ptr_perfil != NULL )
    PaintPerfil();
}


void CurveManager :: PaintCurve()
{
  bicotiPixelRGBA local_pixel = bicotiRGBTools :: CreateRGBA( 255, 255, 0 );
  double curve_length = ptr_image->GetDexel()->GetCurveOpen( 0 )->GetLength();
  
  CurrentImageType :: RealCoordinate aux_coord;
  
  for( double index = 0; index < curve_length; index+=2 )
    {aux_coord = ptr_image->GetDexel()->GetCurveOpen( 0 )->GetCoordinate( index / curve_length );
    ((bicotiHomologousGraphic2DImage< CurrentImageType :: Pixel >*)(ptr_image->GetActor()->GetHomologous()))->DirectSetPixel( ((int)aux_coord.X0()), ((int)aux_coord.X1()), local_pixel );}
  
  ((bicotiHomologousGraphic2DImage< CurrentImageType :: Pixel >*)(ptr_image->GetActor()->GetHomologous()))->DirectRefresh();
}


void CurveManager :: UnPaintCurve()
{
  ((bicotiHomologousGraphic2DImage< CurrentImageType :: Pixel >*)(ptr_image->GetActor()->GetHomologous()))->ClearSettedPixels();
}


void CurveManager :: PaintPerfil()
{
  double curve_length = ptr_image->GetDexel()->GetCurveOpen( 0 )->GetLength();
  
  if( ptr_perfil == NULL )
    {ptr_perfil = new CurrentImageType( ((int)curve_length), 256, "perfil" );
    CurrentImageType :: HomologousFactory factory;
    factory.CreateHomologous( ptr_perfil->GetActor() );}
  else
    ptr_perfil->GetImplementation()->Resize( ((int)curve_length), 256 );
  
  for( double index = 0; index < curve_length - 1; index++ )
    ptr_perfil->GetImplementation()->SetPixel( ((int)index), 255 - ((int)(ptr_image->GetDexel()->GetCurveOpen( 0 )->GetPixel( index / curve_length ))), 255 );
  
  ptr_perfil->GetActor()->SetSize( ((int)curve_length), 256 );
  ptr_perfil->GetActor()->SetVisible( true );
  ptr_perfil->GetActor()->RefreshImage();
}


