#include "menu_manager.hpp"
#include "main_window.hpp"


MenuManager :: MenuManager( MainWindow * ptr_mw )
{
  ptr_main_window = ptr_mw;

  ptr_menu_interpolator = new bicotiActorGraphicMenuPopup( "menu_interpolator" );
  ptr_menu_interpolator->AddItem( "&Linear" );
  ptr_menu_interpolator->AddItem( "S&impson" );
  ptr_menu_interpolator->AddItem( "&Spline" );
  ptr_menu_interpolator->AddListenerMenu( this, "&Linear" );
  ptr_menu_interpolator->AddListenerMenu( this, "S&impson" );
  ptr_menu_interpolator->AddListenerMenu( this, "&Spline" );
  ptr_menu_interpolator->SetVisible( false );
  factory.CreateHomologous( ptr_menu_interpolator );

  ptr_menu_edit_points = new bicotiActorGraphicMenuPopup( "menu_edit_points" );
  ptr_menu_edit_points->AddItem( "&Agregar" );
  ptr_menu_edit_points->AddItem( "&Mover" );
  ptr_menu_edit_points->AddItem( "&Quitar" );
  ptr_menu_edit_points->AddListenerMenu( this, "&Agregar" );
  ptr_menu_edit_points->AddListenerMenu( this, "&Mover" );
  ptr_menu_edit_points->AddListenerMenu( this, "&Quitar" );
  ptr_menu_edit_points->SetVisible( false );
  factory.CreateHomologous( ptr_menu_edit_points );

  ptr_menu_perfil = new bicotiActorGraphicMenuPopup( "menu_perfil" );
  ptr_menu_perfil->AddItem( "&Editar Puntos", ptr_menu_edit_points );
  ptr_menu_perfil->AddItem( "&Crear", ptr_menu_interpolator );
  ptr_menu_perfil->AddSeparator();
  ptr_menu_perfil->AddItem( "C&errar" );
  ptr_menu_perfil->AddListenerMenu( this, "&Editar Puntos" );
  ptr_menu_perfil->AddListenerMenu( this, "&Crear" );
  ptr_menu_perfil->AddListenerMenu( this, "C&errar" );
  ptr_menu_perfil->SetVisible( false );
  factory.CreateHomologous( ptr_menu_perfil );

  ptr_menu_file = new bicotiActorGraphicMenuPopup( "menu_file" );
  ptr_menu_file->AddItem( "&Abrir" );
  ptr_menu_file->AddItem( "&Cerrar" );
  ptr_menu_file->AddSeparator();
  ptr_menu_file->AddItem( "&Salir" );
  ptr_menu_file->AddListenerMenu( this, "&Abrir" );
  ptr_menu_file->AddListenerMenu( this, "&Cerrar" );
  ptr_menu_file->AddListenerMenu( this, "&Salir" );
  ptr_menu_file->SetVisible( false );
  factory.CreateHomologous( ptr_menu_file );

  ptr_menu_main = new bicotiActorGraphicMenuBar( "menu_main" );
  ptr_menu_main->AddItem( "&Archivo", ptr_menu_file );
  ptr_menu_main->AddItem( "&Perfil", ptr_menu_perfil );
  ptr_menu_main->AddSeparator();
  ptr_menu_main->AddItem( "&Help" );
  ptr_menu_main->AddListenerMenu( this, "&Archivo" );
  ptr_menu_main->AddListenerMenu( this, "&Perfil" );
  ptr_menu_main->AddListenerMenu( this, "&Help" );
  ptr_menu_main->SetVisible( true );
  factory.CreateHomologous( ptr_menu_main );

  ptr_file_dialog = NULL;
}


MenuManager :: ~MenuManager()
{
  delete ptr_menu_interpolator;
  delete ptr_menu_edit_points;
  delete ptr_menu_perfil;
  delete ptr_menu_file;
  delete ptr_menu_main;

  if( ptr_file_dialog != NULL )
    delete ptr_file_dialog;
}


void MenuManager :: ItemSelected( bicotiActor * ptr_menu, char * item )
{
  if( !strcmp( ptr_menu->GetActorDescriptor().GetActorName(), "menu_interpolator" ) )
    {
      if( !strcmp( item, "&Linear" ) )
	ptr_main_window->CreateCurveLinear();
      
      if( !strcmp( item, "S&impson" ) )
	ptr_main_window->CreateCurveSimpson();

      if( !strcmp( item, "&Spline" ) )
	ptr_main_window->CreateCurveSpline();
    }

  if( !strcmp( ptr_menu->GetActorDescriptor().GetActorName(), "menu_edit_points" ) )
    {
      if( !strcmp( item, "&Agregar" ) )
	ptr_main_window->AddPoints();

      if( !strcmp( item, "&Mover" ) )
	ptr_main_window->MovePoints();

      if( !strcmp( item, "&Quitar" ) )
	ptr_main_window->RemovePoints();
    }

  if( !strcmp( ptr_menu->GetActorDescriptor().GetActorName(), "menu_perfil" ) )
    {
      if( !strcmp( item, "C&errar" ) )
	cout << "Es el Cerrar de menu_perfil" << endl;
    }

  if( !strcmp( ptr_menu->GetActorDescriptor().GetActorName(), "menu_file" ) )
    {
      if( !strcmp( item, "&Abrir" ) )
	{if( ptr_file_dialog == NULL )
	  {ptr_file_dialog = new bicotiActorGraphic2DFileDialog;
	  ptr_file_dialog->SetSize( file_dialog_width, file_dialog_height );
	  ptr_file_dialog->SetFilter( "*.jpg" );
	  ptr_file_dialog->SetExistingFile( true );
	  factory.CreateHomologous( ptr_file_dialog );
	  ptr_file_dialog->AddListenerFileDialog( this );}
	else
	  ptr_file_dialog->SetVisible( true );}
      
      if( !strcmp( item, "&Cerrar" ) )
	ptr_main_window->Recreate();
      
      if( !strcmp( item, "&Salir" ) )
	ptr_main_window->Stop();
    }
  
  if( !strcmp( ptr_menu->GetActorDescriptor().GetActorName(), "menu_main" ) )
    {
      if( !strcmp( item, "&Help" ) )
	cout << "Es el Help de menu_main" << endl;
    }
}


bicotiActorGraphicMenuBar * MenuManager :: GetMainMenu()
{
  return ptr_menu_main;
}


void MenuManager :: FileDialogFileSelected( bicotiActor * ptr_actor, const char * file )
{
  ptr_main_window->LoadImage( file );
}


void MenuManager :: FileDialogFileHighlighted( bicotiActor * ptr_actor, const char * file )
{}


void MenuManager :: FileDialogDirectoryChanged( bicotiActor * ptr_actor, const char * dir )
{}


