#include <image_2D_array_hsv_int.hpp>
#include <actor_image_2D_hsv_int_qt.hpp>

class RegionData
{
private:
  REAL hue_value;
  REAL saturation_value;
  REAL value_value;
  REAL area_value;
  REAL perimeter_value;
  bicotiCoordinate2D< REAL > baricenter;
  
public:
  RegionData() {};
  REAL& Hue() { return hue_value; };
  REAL& Saturation() { return saturation_value; };
  REAL& Value() { return value_value; };
  REAL& Area() { return area_value; };
  REAL& Perimeter() { return perimeter_value; };
  bicotiCoordinate2D< REAL >& Baricenter() { return baricenter; };
};


class PixelRange
{
private:
  bool direct;
  bicotiHSV< unsigned int, REAL > lower;
  bicotiHSV< unsigned int, REAL > upper;

public:
  PixelRange( bicotiHSV< unsigned int, REAL > low, bicotiHSV< unsigned int, REAL > up, bool val_dir = true ) 
    {lower = low;
    upper = up;
    direct = val_dir;};
  PixelRange() {};
  bicotiHSV< unsigned int, REAL >& Lower() { return lower; };
  bicotiHSV< unsigned int, REAL >& Upper() { return upper; };
  bool& Direct() { return direct; };
};



void SubstractBackground( bicotiImageImplementation2DArray< bicotiHSV< unsigned int, REAL > > * ptr_ima_hsv, unsigned int lower_hue = 210, unsigned int upper_hue = 270 )
{
  vector< bicotiPixelAttributeSelector< unsigned int, bicotiHSV< unsigned int, REAL > > * > list_col_sel;
  list_col_sel.push_back( new bicotiPixelAttributeSelectorColorHue< unsigned int, REAL > );
  
  bicotiPropertyHistogramExtended< bicotiCoordinate2D< INTEGER >, unsigned int, bicotiHSV< unsigned int, REAL > > histogram( ptr_ima_hsv, list_col_sel );
  
  vector< bicotiPixelOperatorUnary< unsigned int > * > operators_list;
  operators_list.push_back( new bicotiPixelOperatorUnarySegmentationRange< unsigned int >( 361, lower_hue, upper_hue ) );
  
  bicotiHistogramEqualizator< bicotiCoordinate2D< INTEGER >, unsigned int, bicotiHSV< unsigned int, REAL > > equalizator( &histogram , operators_list );
  
  equalizator();
  
  delete operators_list[ 0 ];
  delete list_col_sel[ 0 ];
};


void ClasiffyComponents( bicotiActorGraphic2DImage< bicotiHSV< unsigned int, REAL > > * ptr_ima_hsv, vector< PixelRange >& pixel_ranges, vector< RegionData >& region_data )
{

  const unsigned int min_interest_area = 20;
  const unsigned int min_interest_adjacents = 10;
  const unsigned int adjacents_step = 5; // OJO hay que asegurar suficientes puntos para el interpolador
  const REAL adjacents_tol = 8;

  RegionData aux_data;
  REAL mean_hue;
  REAL mean_saturation;
  REAL mean_value;

  unsigned reg_index, index, adjacents_index, greater_adjacents_index, greater_adjacents_number;  

  vector< bicotiPixelLabelingCriterion< bicotiHSV< unsigned int, REAL > > * > list_criterions;
  for( index = 0; index < pixel_ranges.size(); index++ )
    list_criterions.push_back( new bicotiPixelLabelingCriterionVectorialRange< bicotiHSV< unsigned int, REAL > >( pixel_ranges[ index ].Lower(), pixel_ranges[ index ].Upper(), pixel_ranges[ index ].Direct() ) );
  bicotiPixelLabelingCriterionMultipleOr< bicotiHSV< unsigned int, REAL > > pixel_criterion( list_criterions );
  bicotiConnectivityLabelingStrategy2D4N connectivity_strategy;
  bicotiDexelLabeling< bicotiCoordinate2D< INTEGER >, bicotiHSV< unsigned int, REAL > > labeling( ptr_ima_hsv, &pixel_criterion, &connectivity_strategy );

  region_data.clear();

  bicotiCoordinate< INTEGER > aux_baricenter;
  bicotiHSV< unsigned int, REAL > white_pix( 0, 0, 255 );
  bicotiHSV< unsigned int, REAL > black_pix( 0, 0, 0 );
  bicotiPixelOperatorUnaryConstant< bicotiHSV< unsigned int, REAL > > pix_mod( black_pix );

  vector< bicotiCoordinate2D< REAL > > aux_list_coords;
  bicotiRegionProperty< bicotiCoordinate2D< INTEGER >, bicotiHSV< unsigned int, REAL > > property;
  bicotiDexelCurveClosed< bicotiCoordinate2D< REAL >, bicotiHSV< unsigned int, REAL > > curve;
  bicotiCoordinateInterpolator1DClosedLinear< bicotiCoordinate2D< REAL > > coord_interpolator;
  bicotiImagePixelInterpolator2DLinear< bicotiHSV< unsigned int, REAL >, bicotiHSV< REAL, unsigned int > > pixel_interpolator( ptr_ima_hsv );

  for( reg_index = 0; reg_index < labeling.GetRegionsNumber(); reg_index++ )
    {if( (aux_data.Area() = labeling.GetRegion( reg_index ).GetPixelsNumber()) >= min_interest_area )
      {for( index = 0, mean_hue = 0, mean_saturation = 0, mean_value = 0; index < labeling.GetRegion( reg_index ).GetPixelsNumber(); index++ )
	{mean_hue += labeling.GetRegion( reg_index ).GetPixel( index ).Hue();
	mean_saturation += labeling.GetRegion( reg_index ).GetPixel( index ).Saturation();
	mean_value += labeling.GetRegion( reg_index ).GetPixel( index ).Value();}
      mean_hue /= labeling.GetRegion( reg_index ).GetPixelsNumber();
      mean_saturation /= labeling.GetRegion( reg_index ).GetPixelsNumber();
      mean_value /= labeling.GetRegion( reg_index ).GetPixelsNumber();
      aux_data.Hue() = mean_hue;
      aux_data.Saturation() = mean_saturation;
      aux_data.Value() = mean_value;
      
      property.SetRegion( &(labeling.GetRegion( reg_index )) );
      property.FixInfo();
      property.ComputeBorder();
      property.ClassifyAdjacents( adjacents_tol );

      for( adjacents_index = 0, greater_adjacents_index = 0, greater_adjacents_number = 0; adjacents_index < property.GetDistinctsAdjacentsSetsNumber(); adjacents_index++ )
	{if( greater_adjacents_number < property.GetAdjacents( adjacents_index ).size() )
	  {greater_adjacents_number = property.GetAdjacents( adjacents_index ).size();
	  greater_adjacents_index = adjacents_index;}}

      if( greater_adjacents_number >= min_interest_adjacents )
	{aux_list_coords.clear();
	for( index = 0; index < greater_adjacents_number; index += adjacents_step )
	  aux_list_coords.push_back( (bicotiCoordinate2D< REAL >)(property.GetAdjacents( greater_adjacents_index )[ index ]) );
	curve.SetInfo( ptr_ima_hsv, aux_list_coords, &coord_interpolator, &pixel_interpolator );
	aux_data.Perimeter() = (REAL)(curve.GetLength());
	aux_data.Baricenter() = (bicotiCoordinate2D< REAL >)(property.GetBaricenter());
	region_data.push_back( aux_data );
	labeling.GetRegion( reg_index ).ModifyPixels( &pix_mod );
	labeling.GetRegion( reg_index ).UpdateImageImplementation();
	aux_baricenter = (bicotiCoordinate< INTEGER >)(property.GetBaricenter());
	ptr_ima_hsv->SetPixel( aux_baricenter, white_pix );
	for( index = 0; index < 1000; index++ )
	  curve.SetPixel( ((double)index / 1000), white_pix );
	curve.UpdateImageImplementation();
	}}}

  for( index = 0; index < pixel_ranges.size(); index++ )
    delete list_criterions[ index ];

};


void main( int argc, char ** argv )
{ 
  const PixelRange yellow_range( bicotiHSV< unsigned int, REAL >( 25, 0, 100 ), bicotiHSV< unsigned int, REAL >( 75, 255, 255 ) );
  const PixelRange red_range_1( bicotiHSV< unsigned int, REAL >( 0, 0, 100 ), bicotiHSV< unsigned int, REAL >( 20, 255, 255 ) );
  const PixelRange red_range_2( bicotiHSV< unsigned int, REAL >( 340, 0, 100 ), bicotiHSV< unsigned int, REAL >( 360, 255, 255 ) );
  const PixelRange green_range( bicotiHSV< unsigned int, REAL >( 110, 0, 50 ), bicotiHSV< unsigned int, REAL >( 170, 255, 255 ) );
  const PixelRange blue_range( bicotiHSV< unsigned int, REAL >( 210, 0, 0 ), bicotiHSV< unsigned int, REAL >( 285, 255, 255 ), false ); 
 
  bicotiActorImage2DHSVIntQT ima;  
  vector< PixelRange > pixel_ranges;
  vector< RegionData > region_data;

  ima.BuildMapper();
  ima.GetMapper()->BuildDeviceFile( "../Pictures/lego.jpg" );
  ima.GetMapper()->BuildMapperJPEGIJG();
  ima.GetMapper()->Read();
  ima.GetActor()->SetSize( ima.GetImplementation()->GetX0Size(), ima.GetImplementation()->GetX1Size() );
  ima.DestroyMapper();
  
  bicotiActorImage2DHSVIntQT :: HomologousFactory factory;
  bicotiActorImage2DHSVIntQT :: EventLoopManager event_loop( argc, argv );
  ima.GetActor()->SetMainActor( true );
  factory.CreateHomologous( ima.GetActor() );
  event_loop.Start();
  
  pixel_ranges.push_back( blue_range );
  ClasiffyComponents( ima.GetImplementation(), pixel_ranges, region_data );
  
  for( unsigned index = 0; index < region_data.size(); index++ )
    cout << "La region " << index << ", tiene centro=(" << region_data[ index ].Baricenter().X0() << "," << region_data[ index ].Baricenter().X1() << "), hue=" << region_data[ index ].Hue() << ", saturacion=" << region_data[ index ].Saturation() << ", valor=" << region_data[ index ].Value() << endl << "    area=" << region_data[ index ].Area() << ", perimetro=" << region_data[ index ].Perimeter() << endl;
  
  ima.GetActor()->RefreshImage();
  ima.GetActor()->SetVisible( true );
  ima.GetActor()->SetPosition( 50, 50 );
  event_loop.Start();
};
