//************************************************************************
//* Este archivo contiene algunos ejemplos de las diferentes estrategias *
//* de extrapolacin de imagenes                                         *
//************************************************************************  
#include <actor_image_2D_char_qt.hpp>


void main( int argc, char ** argv )
{
  bicotiActorImage2DCharQT imagen;
  imagen.BuildMapper();
  imagen.GetMapper()->BuildDeviceFile( "extrapol3.jpg" );
  imagen.GetMapper()->BuildMapperJPEGIJG();
  imagen.GetMapper()->Read();
  imagen.GetActor()->SetSize( imagen.GetImplementation()->GetX0Size(), imagen.GetImplementation()->GetX1Size() );

  bicotiActorImage2DCharQT :: HomologousFactory factory;
  bicotiActorImage2DCharQT :: EventLoopManager event_loop( argc, argv );
  
  imagen.GetActor()->SetMainActor( true );
  factory.CreateHomologous( imagen.GetActor() );
  event_loop.Start();
  
  unsigned borde_der, borde_izq, borde_sup, borde_inf, filas_des, columnas_des;
  int ext_pix = 0;
  int option;

  cout << "================================================================" << endl;
  cout << "BORDE DERECHO : ";
  cin >> borde_der;
  cout << "BORDE IZQUIERDO : ";
  cin >> borde_izq;
  cout << "BORDE SUPERIOR : ";
  cin >> borde_sup;
  cout << "BORDE INFERIOR : ";
  cin >> borde_inf;

  cout << "================================================================" << endl;
  cout << " Estrategia de extrapolacin " << endl;
  cout << " 1 - Constant" << endl;
  cout << " 2 - Equal" << endl;
  cout << " 3 - Simetric" << endl;
  cout << " 4 - Ciclic" << endl;
  cout << "Ingrese su opcion : ";
  cin >> option;

  filas_des = imagen.GetImplementation()->GetX1Size() + borde_sup + borde_inf;
  columnas_des = imagen.GetImplementation()->GetX0Size() + borde_der + borde_izq;

  // Ahora creo una imagen 2D Array de enteros ms grande, con borde
  bicotiImageImplementation2DArray< unsigned char > * ptr_ima_des;
  ptr_ima_des = new bicotiImageImplementation2DArray< unsigned char >(0, columnas_des , filas_des );
  

  // Creo la estrategia de acuerdo a la opcin
  bicotiImageExtrapolationStrategy2D< unsigned char > * ptr_ext_strat;

  switch ( option ){
  case 1 :
    cout<<"Ingrese el valor del pixel del borde : ";
    cin>>ext_pix;
    ptr_ext_strat = new bicotiImageExtrapolationStrategy2DConstant< unsigned char >(ext_pix);
    break;
  case 2:
    ptr_ext_strat = new bicotiImageExtrapolationStrategy2DEqual< unsigned char >;
    break;
  case 3:
    ptr_ext_strat = new bicotiImageExtrapolationStrategy2DSimetric< unsigned char >;
    break;
  case 4:
    ptr_ext_strat = new bicotiImageExtrapolationStrategy2DCiclic< unsigned char >;
  };
  
  ptr_ext_strat->SetImplementation(imagen.GetImplementation());
  

  cout<<"================================================================"<<endl;

  // Cargo la nueva imagen a travs de la estrategia y la muestro.  
  unsigned i,j;
  for ( i = 0 ; i < filas_des ; i++ )
    for ( j = 0 ; j < columnas_des ; j++ )
      ptr_ima_des->SetPixel( j, i, (*ptr_ext_strat)( j - borde_izq, i - borde_sup ) );
  
  imagen.GetImplementation()->CopyMeFrom( ptr_ima_des );
  imagen.GetActor()->SetVisible( true );
  imagen.GetActor()->SetSize( imagen.GetImplementation()->GetX0Size(), imagen.GetImplementation()->GetX1Size() );
  factory.RecursiveCreateHomologous( imagen.GetActor() );
  event_loop.Start();
  
  delete ptr_ext_strat;
  delete ptr_ima_des;
};

