#include <actor_image_2D_double_qt.hpp> 
 

typedef bicotiActorImage2DDoubleQT ImaT; 
typedef ImaT :: Pixel PixT; 
typedef bicotiPixelComplex< PixT > ComT; 
typedef bicotiImageImplementation2DArray< ComT > ImpComT;


void main( int argc, char ** argv ) 
{ 
  ComT zero(0, 0); 
 
  ImaT imagen;
  imagen.BuildMapper();
  imagen.GetMapper()->BuildDeviceFile( "fourier3.jpg" );
  imagen.GetMapper()->BuildMapperJPEGIJG();
  imagen.GetMapper()->Read();
  imagen.GetActor()->SetSize( imagen.GetImplementation()->GetX0Size(), imagen.GetImplementation()->GetX1Size() );

  ImaT :: HomologousFactory factory;
  ImaT :: EventLoopManager event_loop( argc, argv );
  imagen.GetActor()->SetMainActor( true );
  factory.CreateHomologous( imagen.GetActor() );
  event_loop.Start();
  
  ImpComT * ptr_trans = new ImpComT( zero, 1, 1 );
  
  bicotiTransformationFourier2D< PixT > fourier;
  fourier.SetImplementations( ptr_trans, imagen.GetImplementation() );

  bicotiImageExtrapolationStrategy< PixT > * ptr_dir_ext;
  ptr_dir_ext = new bicotiImageExtrapolationStrategy2DCiclic< PixT >( );
  bicotiImageExtrapolationStrategy< ComT > * ptr_inv_ext;
  ptr_inv_ext = new bicotiImageExtrapolationStrategy2DConstant< ComT >( zero );
  fourier.SetDirectExtrapolationStrategy( ptr_dir_ext );
  fourier.SetInverseExtrapolationStrategy( ptr_inv_ext );

  cout << "Transformando" << endl;
  fourier.Transform();
  cout << "Transformacion terminada" << endl;
  
  ImaT modulo;
  modulo.SetImplementation( (ImaT::Implementation *)(fourier.GetTransformedModule()) );
  
  cout << "Adaptando transformada a visualizacion" << endl;
  modulo.BuildOperatorUnary();
  modulo.GetOperatorUnary()->Addition( 1 );
  modulo.GetOperatorUnary()->Logarithm();
  modulo.BuildProperty();
  modulo.GetProperty()->BuildStatistical();
  PixT maximo = modulo.GetProperty()->GetStatistical()->MaximumValue();
  modulo.DestroyProperty();
  modulo.GetOperatorUnary()->Product( 255 / maximo );
  modulo.DestroyOperatorUnary();

  modulo.GetActor()->SetMainActor( true );
  modulo.GetActor()->SetSize( modulo.GetImplementation()->GetX0Size(), modulo.GetImplementation()->GetX1Size() );
  factory.CreateHomologous( modulo.GetActor() );
  event_loop.Start();
  
  //----------------
  cout << "Filtrando" << endl;
  ImpComT * ptr_filtro = new ImpComT( zero, ptr_trans->GetX0Size(), ptr_trans->GetX1Size() );

  unsigned columnas = ptr_trans->GetX0Size();
  unsigned filas = ptr_trans->GetX1Size();
  unsigned ancho;
  
  cout << "columnas = " << columnas << ", filas = " << filas << endl;

  cout << "Ingrese el ancho: ";
  cin >> ancho;
  
  unsigned i, j; 
  unsigned radio;
  float angulo,paso;
  ComT uno( 1, 0 );
  
  paso = pi/200;
  
  for( radio = 0 ; radio < ancho ; radio++ ) 
    for( angulo = 0 ; angulo < (pi/2) ; angulo += paso ) 
      {j = (unsigned)( radio*sin(angulo) );
      i = (unsigned)( radio*cos(angulo) );
      ptr_filtro->SetPixel( i, j, uno );}

  for( radio = 0 ; radio < ancho ; radio++ ) 
    for( angulo = 0 ; angulo < (pi/2) ; angulo += paso ) 
      {j = (unsigned)((filas-1)-radio*sin(angulo));
      i = (unsigned)(radio*cos(angulo));
      ptr_filtro->SetPixel( i, j, uno );}
  
  for( radio = 0 ; radio < ancho ; radio++ ) 
    for( angulo = 0 ; angulo < (pi/2) ; angulo += paso ) 
      {i = (unsigned)(columnas-1-radio*cos(angulo));
      j = (unsigned)(radio*sin(angulo));
      ptr_filtro->SetPixel( i, j, uno );}
  
  for( radio = 0 ; radio < ancho ; radio++ ) 
    for( angulo = 0 ; angulo < (pi/2) ; angulo += paso ) 
      {i = (unsigned)(columnas-1-radio*cos(angulo));
      j = (unsigned)(filas-1-radio*sin(angulo));				 
      ptr_filtro->SetPixel( i, j, uno );}

  bicotiOperatorMultipleFacade< ComT > oper( ptr_trans );
  oper.Append( ptr_filtro );
  oper.Product();
  //----------------
  
  cout << "Antitransformando" << endl;
  fourier.Antitransform();
  cout << "Antitransformacion terminada" << endl;
  
  imagen.GetActor()->SetMainActor( true );
  imagen.GetActor()->RefreshImage();
  imagen.GetActor()->SetVisible( true );

  event_loop.Start();
    
  delete ptr_dir_ext;
  delete ptr_inv_ext;
  delete ptr_trans;
  delete ptr_filtro;
};
