//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _bicoti_basic
#define _bicoti_basic



#include <fstream.h>
#include <math.h>
#include <stl.h>



//**********************DECLARATIONS***************************
//-------------------BASIC DECLARATIONS------------------------
//-------------Definiciones de pixeles estandar----------------

/** Clases de pixel ( ver bicoti I ) */
enum pixel_cases { unknown, single ,vectorial };

/** Clases de pixel (ver bicoti I ) */
typedef enum pixel_cases PixelCases;

//-----------------------Default real type---------------------
#define REAL float

//---------------------Default integer type--------------------
#define INTEGER unsigned int

//--------------------Epsilon de la mquina--------------------
#define EPSILON_MACHINE 1e-6 // HACER MACRO PARA MEJORAR EPS CON DOUBLE

//-----------------------------PI------------------------------
#define pi 3.1415926535 

//--------------------------TOTAL_CHECK------------------------
#define TOTAL_CHECK

//--------------------------VISUAL_60--------------------------
//#define VISUAL_60

//----------------------FACADES_INCLUDES-----------------------
//#define IMAGE_2D_ARRAY_CHAR
//#define IMAGE_2D_ARRAY_INT
//#define IMAGE_2D_ARRAY_FLOAT
//#define IMAGE_2D_ARRAY_DOUBLE
//#define IMAGE_2D_ARRAY_RGB_CHAR
//#define IMAGE_2D_ARRAY_RGB_FLOAT
//#define IMAGE_2D_ARRAY_HSV_INT
//#define IMAGE_2D_SPARSE_CHAR
//#define IMAGE_2D_SPARSE_FLOAT
//#define IMAGE_2D_SPARSE_RGB_CHAR
//#define IMAGE_2D_SPARSE_RGB_FLOAT
//#define IMAGE_3D_ARRAY_CHAR
//#define IMAGE_3D_ARRAY_FLOAT
//#define IMAGE_3D_ARRAY_RGB_CHAR
//#define IMAGE_3D_ARRAY_RGB_FLOAT
//#define IMAGE_3D_LINEAR_CHAR
//#define IMAGE_3D_LINEAR_FLOAT
//#define IMAGE_3D_LINEAR_RGB_CHAR
//#define IMAGE_3D_LINEAR_RGB_FLOAT


//------------NUMERICAL ALGORITHMS DECLARATIONS----------------
//-----------------------Bipartition---------------------------
#define BIPART_ITER_MAX 500
#define BIPART_DOM_ERR_TOL .000001
#define BIPART_COD_ERR_TOL .000001

//-------------------------Secant------------------------------
#define SECANT_ITER_MAX 20
#define SECANT_DOM_ERR_TOL .0001
#define SECANT_COD_ERR_TOL .0001

//-------------------------Newton------------------------------
#define NEWTON_ITER_MAX 15
#define NEWTON_DOM_ERR_TOL .0001
#define NEWTON_COD_ERR_TOL .0001

//------------------------Quadrature---------------------------
#define QUAD_ERR_TOL .00001

//-------------------------Derivate----------------------------
#define DERIVATE_ERR_TOL .00001

//------------------------Derivate2----------------------------
#define DERIVATE2_ERR_TOL .00001


//--------------FRAME ITERATOR 3D DECLARATIONS-----------------
#define FRM_SIZE_TMS_CPY 2


//----------------INTERPOLATOR DECLARATIONS--------------------
//----------Parmetros para el interpolador de simpson---------
#define MAX_ITER_SIMP1 20
#define IT_ERROR_SIMP1 .001

//----------Parmetros para el interpolador de splines---------
#define MAX_ITER_SPLINE1 20
#define IT_ERROR_SPLINE1 .001
#define INT_SEGMENTS_SPLINE1 20


//-------------BICOTI EXCEPTION  DECLARATIONS------------------
//---------------------bicotiException-------------------------

/** ubicacin del archivo con cdigos de error */
const char _error_codes_path[70] = "/home/crisso/working/bicoti/source/error_codes";

/** 
 * Excepciones de bicoti
 *
 * Almacena informacin de utilidad cuando se produce una excepcin
 */
class bicotiException
{
private:
  void ConvertToString( vector< char >&, int );
  
protected:

  char * local_class_name;
  char * local_function_name;
  int local_error_code;
  
public:

  /** 
   * Constructor
   *
   *@param class_name nombre de la clase donde se produce la excepcin
   *@param function_name nombre del mtodo donde se produce la excepcin
   *@param error_code cdigo de error ( ver archivo error_codes )
   */ 
  bicotiException( char * class_name, char * function_name, int error_code );
  
  /**
   * Constructor de copia
   *
   *@param except_src excepcin original
   */
  bicotiException( const bicotiException& except_src );
  virtual ~bicotiException();
  
  /**
   * Asignacin
   *
   * Idem operador de copia
   */
  void operator=( const bicotiException& );
  
  /** 
   * Devuelve el nombre de la clase 
   *
   *@returns nombre de la clase donde se produce la excepcin
   *@see #bicotiException
   */
  const char * GetClassName();
  
  /** 
   * Devuelve el nombre del mtodo 
   *
   *@returns nombre del mtodo donde se produce la excepcin
   *@see #bicotiException
   */
  const char * GetFunctionName();

  /** 
   * Devuelve el nombre del mtodo 
   *
   *@returns cdigo de error
   *@see #bicotiException
   */
  int GetErrorCode();
  
  /**
   * Imprime una descripcin del error
   *
   * No implementada.
   */
  virtual void What();
};




//***********************DECLARATIONS***************************


#endif         
