//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _coordinate_2D
#define _coordinate_2D

#include <source/coordinate/coordinate_basic.hpp>


//----------------------Declarations-----------------------
template< class CoordComponentType >
class bicotiCoordinate2D : public bicotiCoordinate< CoordComponentType >
{
private:
  CoordComponentType x0;
  CoordComponentType x1;
  
public:
  bicotiCoordinate2D( CoordComponentType = 0 , CoordComponentType = 0 );
  bicotiCoordinate2D( const bicotiCoordinate2D<CoordComponentType>& );
  
  void operator=( const bicotiCoordinate2D< CoordComponentType >& );
  
  bool operator==( const bicotiCoordinate2D< CoordComponentType >& );
  
  bicotiCoordinate2D< CoordComponentType > operator+( double );
  bicotiCoordinate2D< CoordComponentType > operator-( double );
  bicotiCoordinate2D< CoordComponentType > operator*( double );
  bicotiCoordinate2D< CoordComponentType > operator/( double );
  
  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );
  
  bicotiCoordinate2D< CoordComponentType > operator+( const bicotiCoordinate2D< CoordComponentType >& );
  bicotiCoordinate2D< CoordComponentType > operator-( const bicotiCoordinate2D< CoordComponentType >& );
  bicotiCoordinate2D< CoordComponentType > operator*( const bicotiCoordinate2D< CoordComponentType >& ); 
  bicotiCoordinate2D< CoordComponentType > operator/( const bicotiCoordinate2D< CoordComponentType >& ); 
  
  void operator+=( const bicotiCoordinate2D< CoordComponentType >& );
  void operator-=( const bicotiCoordinate2D< CoordComponentType >& );
  void operator*=( const bicotiCoordinate2D< CoordComponentType >& );
  void operator/=( const bicotiCoordinate2D< CoordComponentType >& );
  
  REAL ScalarProduct( const bicotiCoordinate2D< CoordComponentType >& );
  REAL Norm();
  
  bicotiCoordinate2D< CoordComponentType > operator^( int );
  
  CoordComponentType& operator() ( int );
  
  unsigned GetDimension();
  
  CoordComponentType& X0();
  CoordComponentType& X1();
  
  operator bicotiCoordinate2D< INTEGER >();
  operator bicotiCoordinate2D< REAL >();
};



//---------------------Implementations---------------------
template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > :: bicotiCoordinate2D( CoordComponentType x_0, CoordComponentType x_1 )
{
  x0 = x_0;
  x1 = x_1;
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > :: bicotiCoordinate2D( const bicotiCoordinate2D< CoordComponentType >& coord_src )
{
  x0 = coord_src.x0;
  x1 = coord_src.x1;
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator=( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  x0 = coord.x0;
  x1 = coord.x1;
};


template< class CoordComponentType >
bool bicotiCoordinate2D< CoordComponentType > :: operator==( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  return ( ( x0 == coord.x0 ) && ( x1 == coord.x1 ) );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator+( double scalar )
{
  bicotiCoordinate2D< CoordComponentType > temp;
  
  temp.x0 = ( CoordComponentType ) ( x0 + scalar );
  temp.x1 = ( CoordComponentType ) ( x1 + scalar );
  
  return( temp );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator-( double scalar )
{
  bicotiCoordinate2D< CoordComponentType > temp;
  
  temp.x0 = ( CoordComponentType ) ( x0 - scalar );
  temp.x1 = ( CoordComponentType ) ( x1 - scalar );

  return( temp );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator*( double scalar )
{
  bicotiCoordinate2D< CoordComponentType > temp;

  temp.x0 = ( CoordComponentType ) ( x0 * scalar );
  temp.x1 = ( CoordComponentType ) ( x1 * scalar );
  
  return( temp );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator/( double scalar )
{
  bicotiCoordinate2D< CoordComponentType > temp;
  
  temp.x0 = ( CoordComponentType ) ( x0 / scalar );
  temp.x1 = ( CoordComponentType ) ( x1 / scalar );
  
  return( temp );
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator+=( double scalar )
{
  x0 = ( CoordComponentType ) ( x0 + scalar );
  x1 = ( CoordComponentType ) ( x1 + scalar );
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator-=( double scalar )
{
  x0 = ( CoordComponentType ) ( x0 - scalar );
  x1 = ( CoordComponentType ) ( x1 - scalar );
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator*=( double scalar )
{
  x0 = ( CoordComponentType ) ( x0 * scalar );
  x1 = ( CoordComponentType ) ( x1 * scalar );
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator/=( double scalar )
{
  x0 = ( CoordComponentType ) ( x0 / scalar );
  x1 = ( CoordComponentType ) ( x1 / scalar );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator+( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  bicotiCoordinate2D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 + coord.x0;
  coord_temp.x1 = x1 + coord.x1;
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator-( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  bicotiCoordinate2D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 - coord.x0;
  coord_temp.x1 = x1 - coord.x1;
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator*( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  bicotiCoordinate2D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 * coord.x0;
  coord_temp.x1 = x1 * coord.x1;
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator/( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  bicotiCoordinate2D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 / coord.x0;
  coord_temp.x1 = x1 / coord.x1;
  
  return( coord_temp );
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator+=( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  x0 = x0 + coord.x0;
  x1 = x1 + coord.x1;
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator-=( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  x0 = x0 - coord.x0;
  x1 = x1 - coord.x1;
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator*=( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  x0 = x0 * coord.x0;
  x1 = x1 * coord.x1;
};


template< class CoordComponentType >
void bicotiCoordinate2D< CoordComponentType > :: operator/=( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  x0 = x0 / coord.x0;
  x1 = x1 / coord.x1;
};


template< class CoordComponentType >
REAL bicotiCoordinate2D< CoordComponentType > :: ScalarProduct( const bicotiCoordinate2D< CoordComponentType >& coord )
{
  return( ( REAL ) ( ( x0 * coord.x0 ) + ( x1 * coord.x1 ) ) );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > bicotiCoordinate2D< CoordComponentType > :: operator^( int exponent )
{
  bicotiCoordinate2D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = ( CoordComponentType ) pow( x0, exponent );
  coord_temp.x1 = ( CoordComponentType ) pow( x1, exponent );
  
  return( coord_temp );
};


template< class CoordComponentType >
REAL bicotiCoordinate2D< CoordComponentType > :: Norm()
{
  return ( (REAL) sqrt( x0*x0 + x1*x1 ) );
};


template< class CoordComponentType >
inline CoordComponentType& bicotiCoordinate2D< CoordComponentType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( 1 < index) )
    throw bicotiException( "bicotiCoordinate2D", "operator()", 1 );
#endif
  
  switch( index )
    {case 0:
      {return ( x0 );
      break;}
    case 1:
      {return ( x1 );
      break;}}
};


template< class CoordComponentType >
unsigned bicotiCoordinate2D< CoordComponentType > :: GetDimension()
{
  return 2;
};


template< class CoordComponentType >
CoordComponentType& bicotiCoordinate2D< CoordComponentType > :: X0()
{
  return x0;
};


template< class CoordComponentType >
CoordComponentType& bicotiCoordinate2D< CoordComponentType > :: X1()
{
  return x1;
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > :: operator bicotiCoordinate2D< INTEGER >()
{
  bicotiCoordinate2D< INTEGER > coord_aux( ( INTEGER ) x0, ( INTEGER ) x1 );
  return ( coord_aux );
};


template< class CoordComponentType >
bicotiCoordinate2D< CoordComponentType > :: operator bicotiCoordinate2D< REAL >()
{
  bicotiCoordinate2D< REAL > coord_aux( ( REAL ) x0, ( REAL ) x1 );
  return ( coord_aux );
};               

#endif
