//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _coordinate_vectorial_static
#define _coordinate_vectorial_static

#include <source/coordinate/coordinate_basic.hpp>



//----------------------Declarations-----------------------
template< class CoordComponentType, int dim = 0 >
class bicotiCoordinateVectorialStatic  : public bicotiCoordinate< CoordComponentType > 
{
private:
  CoordComponentType components[ dim ];
  
public:
  bicotiCoordinateVectorialStatic(){};
  
  bicotiCoordinateVectorialStatic( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  
  void operator=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  
  bool operator==( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > operator+( double );
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > operator-( double );
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > operator*( double );
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > operator/( double );
  
  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );
  
  bicotiCoordinateVectorialStatic< CoordComponentType, dim >  operator+( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  bicotiCoordinateVectorialStatic< CoordComponentType, dim >  operator-( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  bicotiCoordinateVectorialStatic< CoordComponentType, dim >  operator*( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  bicotiCoordinateVectorialStatic< CoordComponentType, dim >  operator/( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  
  void operator+=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  void operator-=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  void operator*=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  void operator/=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  
  REAL ScalarProduct( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& );
  REAL Norm();
  
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > operator^( int );
  
  virtual CoordComponentType& operator() ( int );
  virtual unsigned GetDimension();
};



//----------------------Implementation-----------------------
template< class CoordComponentType , int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: bicotiCoordinateVectorialStatic( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  for( unsigned index = 0; index < dim; index++ )
    components[ index ] = coord_src.components[ index ];
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src)
{
  unsigned index;
  
  for( index = 0; index < dim; index++ )
    components[ index ] = coord_src.components[ index ];
};


template< class CoordComponentType, int dim >
bool bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator==( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src)
{
  bool is_equal = true;
  
  for( unsigned index = 0; ( (index < dim ) && is_equal ); index++ )
    is_equal = ( components[ index ] == coord_src.components[ index ] );
  
  return is_equal;
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator+( double scalar )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for ( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = (CoordComponentType) ( components[ index ] + scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator-( double scalar )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for ( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = (CoordComponentType) ( components[ index ] - scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator*( double scalar )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for ( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = (CoordComponentType) ( components[ index ] * scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator/( double scalar )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for ( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = (CoordComponentType) ( components[ index ] / scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator+=( double scalar )
{
  for ( unsigned index = 0; index < dim; index++ )
    components[ index ] = (CoordComponentType) ( components[ index ] + scalar );
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator-=( double scalar )
{
  for ( unsigned index = 0; index < dim; index++ )
    components[ index ] = (CoordComponentType) ( components[ index ] - scalar );
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator*=( double scalar )
{
  for ( unsigned index = 0; index < dim; index++ )
    components[ index ] = (CoordComponentType) ( components[ index ] * scalar );
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator/=( double scalar )
{
  for ( unsigned index = 0; index < dim; index++ )
    components[ index ] = (CoordComponentType) ( components[ index ] / scalar );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator+( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = components[ index ] + coord_src.components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator-( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = components[ index ] - coord_src.components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator*( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = components[ index ] * coord_src.components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator/( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = components[ index ] / coord_src.components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator+=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  for( unsigned index = 0; index < dim; index++ )
    components[ index ] = components[ index ] + coord_src.components[ index ];
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator-=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  for( unsigned index = 0; index < dim; index++ )
    components[ index ] = components[ index ] - coord_src.components[ index ];
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator*=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  for( unsigned index = 0; index < dim; index++ )
    components[ index ] = components[ index ] * coord_src.components[ index ];
};


template< class CoordComponentType, int dim >
void bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator/=( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  for( unsigned index = 0; index < dim; index++ )
    components[ index ] = components[ index ] / coord_src.components[ index ];
};


template< class CoordComponentType, int dim >
REAL bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: ScalarProduct( const bicotiCoordinateVectorialStatic< CoordComponentType, dim >& coord_src )
{
  REAL result = 0;
  
  for( unsigned index = 0; index < dim; index++ )
    result += components[ index ] * coord_src.components[ index ];
  
  return ( result );
};


template< class CoordComponentType, int dim >
bicotiCoordinateVectorialStatic< CoordComponentType, dim > bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator^( int exponent )
{
  bicotiCoordinateVectorialStatic< CoordComponentType, dim > coord_temp;
  
  for( unsigned index = 0; index < dim; index++ )
    coord_temp.components[ index ] = ( CoordComponentType ) pow( components[ index ], exponent );
  
  return( coord_temp );
};


template< class CoordComponentType, int dim >
REAL bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: Norm()
{
  REAL result = 0;
  
  for( unsigned index = 0; index < dim; index++ )
    result += components[ index ] * components[ index ];
  
  return ( (REAL) sqrt( result ) );
};


template< class CoordComponentType, int dim >
inline CoordComponentType& bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( dim <= (unsigned) index) )
    throw bicotiException( "bicotiCoordinateVectorialStatic", "operator()", 1 );
#endif
  
  return ( components[ index ] );
};


template< class CoordComponentType, int dim >
unsigned bicotiCoordinateVectorialStatic< CoordComponentType, dim > :: GetDimension()
{
  return( dim );
};

#endif
