#ifndef _bicoti_device_basic
#define _bicoti_device_basic

#include <stdio.h>
#include <stdlib.h>

#define MAX_FILE_NAME_SIZE 80

//--------------------DECLARATIONS---------------------
//--------------------bicotiDevice---------------------

/**
 * Clase base para device
 *
 * clase base para device. Contiene la interfase comn para todos los devices
 *@see bicotiMapper
 */
class bicotiDevice
{
protected:
  long block_size;
  
public:
  
  /**
   * Abre un device para entrada o salida
   *
   *@param mode modo de escritura/lectura (rb/wb) 
   */
  virtual void Open( const char * mode )=0;

  /** 
   * Cierra un device
   *
   */
  virtual void Close()=0;

  /**
   * Lee bytes secuencialmente del device
   *
   * Lee BlockSize() o los bytes que queden del device
   *@param blob contiene los bytes leidos
   *@returns la cantidad de bytes leidos
   */
  virtual int Get( char * blob)=0;

  
  /**
   * Escribe secuencialmente bytes en el device
   *
   * Lee  BlockSize() bytes o los que queden en el device
   *@param blob contiene los bytes a escribir
   */
   virtual void Put( const char * blob)=0;


  /**
   * Devuelve el tamao de la imagen
   *
   *@returns el tamao de la imagen en bytes
   */
  virtual long GetImageSize()=0;

  
  /**
   * Fija o devuelve el tamo del bloque de lectura/escritura
   *
   *@returns el tamo del bloque de lectura/escritura en bytes
   */
  long& BlockSize();
};


//------------------IMPLEMENTATIONS--------------------
//--------------------bicotiDevice---------------------
inline long& bicotiDevice :: BlockSize()
{return block_size;}



#endif
