//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _dexel
#define _dexel



#include <source/dexel/dexel_histogram/dexel_histogram.hpp>
#include <source/dexel/dexel_statistical/dexel_statistical.hpp>
#include <source/dexel/dexel_curves/dexel_curves.hpp>
#include <source/dexel/dexel_labeling/dexel_labeling.hpp>



//***************************DECLARATIONS****************************
//-----------------------bicotiPropertyFacade------------------------
template< class CoordTypeInt, class CoordTypeReal , class PixelType >
class bicotiDexelFacade
{
protected:
  bicotiImageImplementation< PixelType > * ptr_image_imp;
 
  vector< bicotiDexelRegion< CoordTypeInt, PixelType > * > list_regions;
  vector< bicotiRegionProperty< CoordTypeInt, PixelType > * > list_regions_properties;

  bicotiDexelLabeling< CoordTypeInt , PixelType > * ptr_labeling;
  bicotiPixelLabelingCriterion< PixelType > * ptr_labeling_criterion;
  bicotiConnectivityLabelingStrategy * ptr_labeling_strategy; 

  vector< bicotiDexelCurveOpen< CoordTypeReal , PixelType > * > list_curves_open;
  vector< bicotiDexelCurveClosed< CoordTypeReal , PixelType > * > list_curves_closed;
  vector< bicotiCoordinateInterpolator1DOpen< CoordTypeReal > * > list_curves_interpolator_open;
  vector< bicotiCoordinateInterpolator1DClosed< CoordTypeReal > * > list_curves_interpolator_closed;
  bicotiImagePixelInterpolator< PixelType > * ptr_image_pixel_interpolator;
  vector< CoordTypeReal > temp_coord_vector;
  virtual void SetPixelInterpolator()=0;

  void ResetPointers();
  void DeleteMe();

public:

  typedef CoordTypeInt IntegerCoordinate;
  typedef CoordTypeReal RealCoordinate;

  virtual ~bicotiDexelFacade(){};
  

  void BuildNewRegion();
  bicotiDexelRegion< CoordTypeInt, PixelType > * GetRegion( unsigned );
  unsigned GetRegionsNumber();
  void DestroyRegions(); 
  
  void BuildRegionsProperties();
  bicotiRegionProperty< CoordTypeInt, PixelType > * GetRegionProperty( unsigned );
  void DestroyRegionsProperties(); 


  void BuildLabeling();
  virtual void UsePixelLabelingCriterionRange( PixelType, PixelType ) = 0;
  void LoadRegionLabelingResult();
  bicotiDexelLabeling< CoordTypeInt, PixelType > * GetLabeling();
  void DestroyLabeling();

  vector< CoordTypeReal > & CurveVector();
  void BuildNewCurveOpenLinear( vector< CoordTypeReal > & );
  void BuildNewCurveClosedLinear( vector< CoordTypeReal > & );
  void BuildNewCurveOpenSimpson( vector< CoordTypeReal > & );
  void BuildNewCurveClosedSimpson( vector< CoordTypeReal > & );
  void BuildNewCurveOpenSpline( vector< CoordTypeReal > & );
  void BuildNewCurveClosedSpline( vector< CoordTypeReal > & );
  bicotiDexelCurveOpen< CoordTypeReal, PixelType > * GetCurveOpen( unsigned );
  bicotiDexelCurveClosed< CoordTypeReal, PixelType > * GetCurveClosed( unsigned );
  unsigned GetOpenCurvesNumber();
  unsigned GetClosedCurvesNumber();
  void DestroyCurves();

  void BuildCurvesProperties();
  void DestroyCurvesProperties();
};



//---------------------bicotiPropertyFacade2D----------------------

template< class PixelType >
class bicotiDexelFacade2D : public bicotiDexelFacade< bicotiCoordinate2D< INTEGER >,bicotiCoordinate2D< REAL >, PixelType >
{
  vector< bicotiCurveProperties2D< PixelType > * > list_open_curve_properties;
  vector< bicotiCurveProperties2DClosed< PixelType > * > list_closed_curve_properties;

  void SetPixelInterpolator();

public:
  virtual ~bicotiDexelFacade2D(){};


  void UseConnectivityLabelingStrategy4N();
  virtual void UsePixelLabelingCriterionRange( PixelType, PixelType ) = 0;

  bicotiCurveProperties2D< PixelType > * GetCurveOpenProperty( unsigned );
  bicotiCurveProperties2DClosed< PixelType > * GetCurveClosedProperty( unsigned );

};



//---------------------bicotiPropertyFacade3D----------------------
template< class PixelType >
class bicotiDexelFacade3D : public bicotiDexelFacade< bicotiCoordinate3D< INTEGER >,bicotiCoordinate3D< REAL >, PixelType >
{
  vector< bicotiCurveProperties3D< PixelType > * > list_open_curve_properties;
  vector< bicotiCurveProperties3D< PixelType > * > list_closed_curve_properties;

  void SetPixelInterpolator();

public:
  virtual ~bicotiDexelFacade3D(){};

  void UseConnectivityLabelingStrategy6N();
  virtual void UsePixelLabelingCriterionRange( PixelType, PixelType ) = 0;

  bicotiCurveProperties3D< PixelType > * GetCurveOpenProperty( unsigned );
  bicotiCurveProperties3D< PixelType > * GetCurveClosedProperty( unsigned );

};



//---------------------bicotiPropertyFacade2DSingle----------------------
template< class PixelType >
class bicotiDexelFacade2DSingle : public bicotiDexelFacade2D< PixelType >
{
public:
  bicotiDexelFacade2DSingle( bicotiImageImplementation2D< PixelType > * );
  ~bicotiDexelFacade2DSingle();

  void UsePixelLabelingCriterionRange( PixelType, PixelType );
};



//---------------------bicotiPropertyFacade2DVectorial----------------------
template< class PixelType >
class bicotiDexelFacade2DVectorial : public bicotiDexelFacade2D< PixelType >
{
public:
  bicotiDexelFacade2DVectorial( bicotiImageImplementation2D< PixelType > * );
  ~bicotiDexelFacade2DVectorial();

  void UsePixelLabelingCriterionRange( PixelType, PixelType );
};



//---------------------bicotiPropertyFacade3DSingle----------------------
template< class PixelType >
class bicotiDexelFacade3DSingle : public bicotiDexelFacade3D< PixelType >
{
public:
  bicotiDexelFacade3DSingle( bicotiImageImplementation3D< PixelType > * );
  ~bicotiDexelFacade3DSingle();

  void UsePixelLabelingCriterionRange( PixelType, PixelType );
};



//---------------------bicotiPropertyFacade3DVectorial----------------------
template< class PixelType >
class bicotiDexelFacade3DVectorial : public bicotiDexelFacade3D< PixelType >
{
public:
  bicotiDexelFacade3DVectorial( bicotiImageImplementation3D< PixelType > * );
  ~bicotiDexelFacade3DVectorial();

  void UsePixelLabelingCriterionRange( PixelType, PixelType );
};




//******************************IMPLEMENTATIONS**********************************
//-----------------------bicotiPropertyFacade------------------------
template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: ResetPointers()
{
  ptr_labeling = NULL;
  ptr_labeling_criterion = NULL;
  ptr_labeling_strategy = NULL; 
  ptr_image_pixel_interpolator = NULL;
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: DeleteMe()
{
  while ( list_regions.size() != 0 )
    { delete list_regions.back();
    list_regions.pop_back();}

  while ( list_regions_properties.size() != 0 )
    { delete list_regions_properties.back();
    list_regions_properties.pop_back();}

  if ( ptr_labeling != NULL )
    delete ptr_labeling;

  if ( ptr_labeling_criterion != NULL )
    delete ptr_labeling_criterion;

  if ( ptr_labeling_strategy != NULL )
    delete ptr_labeling_strategy;

  bicotiDexel< PixelType > * ptr_aux_dexel;
  while ( list_curves_open.size() != 0 )
    {ptr_aux_dexel = list_curves_open.back();
    delete ptr_aux_dexel;
    list_curves_open.pop_back();}
  
  while ( list_curves_closed.size() != 0 )
    {ptr_aux_dexel = list_curves_closed.back();
    delete ptr_aux_dexel;
    list_curves_closed.pop_back();}


  while ( list_curves_interpolator_open.size() != 0 )
    {delete list_curves_interpolator_open.back();
    list_curves_interpolator_open.pop_back();};
  
  while ( list_curves_interpolator_closed.size() != 0 )
    {delete list_curves_interpolator_closed.back();
    list_curves_interpolator_closed.pop_back();};
  
  while ( temp_coord_vector.size() != 0 )
    temp_coord_vector.pop_back();
  
  if ( ptr_image_pixel_interpolator != NULL )
    delete ptr_image_pixel_interpolator;

  ResetPointers();
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewRegion()
{
  bicotiDexelRegion< CoordTypeInt, PixelType >* ptr_temp;
  ptr_temp = new bicotiDexelRegion< CoordTypeInt, PixelType>( ptr_image_imp );
  list_regions.push_back( ptr_temp );

  if ( list_regions_properties.size() != 0 )
    {bicotiRegionProperty< CoordTypeInt, PixelType >* ptr_temp_proper;
    ptr_temp_proper = new bicotiRegionProperty< CoordTypeInt, PixelType >( ptr_temp );
    list_region_properties.push_back( ptr_temp_proper );}
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
bicotiDexelRegion< CoordTypeInt, PixelType > *  bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType >:: GetRegion( unsigned index )
{
  if  ( index < list_regions.size() )
    return list_regions[ index ];
  else
    throw bicotiException( "bicotiDexelFacade", "GetRegion", 1 );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
unsigned bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetRegionsNumber()
{
  return list_regions.size();
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: DestroyRegions()
{
  DestroyRegionsProperties();
  while ( list_regions.size() != 0 )
    { delete list_regions.back();
    list_regions.pop_back();}
};
 

template< class CoordTypeInt, class CoordTypeReal , class PixelType >
bicotiRegionProperty< CoordTypeInt, PixelType > * bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetRegionProperty( unsigned  index )
{
   if ( index < list_regions_properties.size() )
    return list_regions_properties[ index ];
  else
    throw bicotiException( "bicotiDexelFacade", "GetRegionProperty", 1 );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildRegionsProperties()
{
  if ( list_regions_properties.size() != 0 )
    DestroyRegionsProperties();
  
    bicotiRegionProperty< CoordTypeInt, PixelType >* ptr_temp_proper;  
    unsigned i;
    for( i = 0; i < list_regions.size(); i++)
      {ptr_temp_proper = new bicotiRegionProperty< CoordTypeInt, PixelType >( list_regions[ i ] );
      list_regions_properties.push_back( ptr_temp_proper );}
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: DestroyRegionsProperties()
{
    while ( list_regions_properties.size() != 0 )
    { delete list_regions_properties.back();
    list_regions_properties.pop_back();}
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildLabeling()
{
  if( ptr_labeling != NULL )
    delete ptr_labeling;

   ptr_labeling = new bicotiDexelLabeling< CoordTypeInt, PixelType >(ptr_image_imp, ptr_labeling_criterion, ptr_labeling_strategy); 
};


template< class CoordTypeInt, class CoordTypeReal, class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: LoadRegionLabelingResult()
{
  if( ptr_labeling == NULL )
    throw bicotiException( "bicotiDexelFacade", "LoadRegionLabelingResult", 6 );
  else
    {if( ptr_labeling->GetRegionsNumber() > 0 )
      {DestroyRegions();
       unsigned i,j;
       bicotiDexelRegion< CoordTypeInt, PixelType >* ptr_temp;
       for( i = 0; i < ptr_labeling->GetRegionsNumber() ; i++)
	 {ptr_temp = new bicotiDexelRegion< CoordTypeInt, PixelType>( ptr_image_imp );
	  list_regions.push_back( ptr_temp );
	  for( j = 0; j < ptr_labeling->GetRegion( i ).GetPixelsNumber(); j++  )
	    list_regions[ i ]->Append( ptr_labeling->GetRegion( i ).GetCoordinate( j ) );}}}
}


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
bicotiDexelLabeling< CoordTypeInt, PixelType > * bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetLabeling()
{
  return( ptr_labeling );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: DestroyLabeling()
{
  if( ptr_labeling != NULL )
    {delete ptr_labeling;
    ptr_labeling = NULL;}
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
vector< CoordTypeReal > & bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: CurveVector()
{
  return( temp_coord_vector );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewCurveOpenLinear( vector< CoordTypeReal > & points_vect )
{
  SetPixelInterpolator();
  bicotiCoordinateInterpolator1DOpen< CoordTypeReal > * ptr_temp_coord_int;
  ptr_temp_coord_int = new bicotiCoordinateInterpolator1DOpenLinear< CoordTypeReal >;

  list_curves_interpolator_open.push_back( ptr_temp_coord_int );

  bicotiDexelCurveOpen< CoordTypeReal , PixelType > * ptr_temp_curve;
  ptr_temp_curve = new bicotiDexelCurveOpen< CoordTypeReal , PixelType >( ptr_image_imp, points_vect, ptr_temp_coord_int, ptr_image_pixel_interpolator );

  list_curves_open.push_back( ptr_temp_curve );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewCurveClosedLinear( vector< CoordTypeReal > & points_vect )
{
  SetPixelInterpolator();
  bicotiCoordinateInterpolator1DClosed< CoordTypeReal > * ptr_temp_coord_int;
  ptr_temp_coord_int = new bicotiCoordinateInterpolator1DClosedLinear< CoordTypeReal >;

  list_curves_interpolator_closed.push_back( ptr_temp_coord_int );

  bicotiDexelCurveClosed< CoordTypeReal , PixelType > * ptr_temp_curve;
  ptr_temp_curve = new bicotiDexelCurveClosed< CoordTypeReal , PixelType >( ptr_image_imp, points_vect, ptr_temp_coord_int, ptr_image_pixel_interpolator );

  list_curves_closed.push_back( ptr_temp_curve );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewCurveOpenSimpson( vector< CoordTypeReal > & points_vect )
{
  SetPixelInterpolator();
  bicotiCoordinateInterpolator1DOpen< CoordTypeReal > * ptr_temp_coord_int;
  ptr_temp_coord_int = new bicotiCoordinateInterpolator1DOpenSimpson< CoordTypeReal >;

  list_curves_interpolator_open.push_back( ptr_temp_coord_int );

  bicotiDexelCurveOpen< CoordTypeReal , PixelType > * ptr_temp_curve;
  ptr_temp_curve = new bicotiDexelCurveOpen< CoordTypeReal , PixelType >( ptr_image_imp, points_vect, ptr_temp_coord_int, ptr_image_pixel_interpolator );

  list_curves_open.push_back( ptr_temp_curve );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewCurveClosedSimpson( vector< CoordTypeReal > & points_vector )
{
  SetPixelInterpolator();
  bicotiCoordinateInterpolator1DClosed< CoordTypeReal > * ptr_temp_coord_int;
  ptr_temp_coord_int = new bicotiCoordinateInterpolator1DClosedSimpson< CoordTypeReal >;

  list_curves_interpolator_closed.push_back( ptr_temp_coord_int );

  bicotiDexelCurveClosed< CoordTypeReal , PixelType > * ptr_temp_curve;
  ptr_temp_curve = new bicotiDexelCurveClosed< CoordTypeReal , PixelType >( ptr_image_imp, points_vector, ptr_temp_coord_int, ptr_image_pixel_interpolator );

  list_curves_closed.push_back( ptr_temp_curve );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewCurveOpenSpline( vector< CoordTypeReal > & points_vector )
{
  SetPixelInterpolator();
  bicotiCoordinateInterpolator1DOpen< CoordTypeReal > * ptr_temp_coord_int;
  ptr_temp_coord_int = new bicotiCoordinateInterpolator1DOpenSpline< CoordTypeReal >;

  list_curves_interpolator_open.push_back( ptr_temp_coord_int );

  bicotiDexelCurveOpen< CoordTypeReal , PixelType > * ptr_temp_curve;
  ptr_temp_curve = new bicotiDexelCurveOpen< CoordTypeReal , PixelType >( ptr_image_imp, points_vector, ptr_temp_coord_int, ptr_image_pixel_interpolator );

  list_curves_open.push_back( ptr_temp_curve );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildNewCurveClosedSpline( vector< CoordTypeReal > & points_vector )
{
  SetPixelInterpolator();
  bicotiCoordinateInterpolator1DClosed< CoordTypeReal > * ptr_temp_coord_int;
  ptr_temp_coord_int = new bicotiCoordinateInterpolator1DClosedSpline< CoordTypeReal >;

  list_curves_interpolator_closed.push_back( ptr_temp_coord_int );

  bicotiDexelCurveClosed< CoordTypeReal , PixelType > * ptr_temp_curve;
  ptr_temp_curve = new bicotiDexelCurveClosed< CoordTypeReal , PixelType >( ptr_image_imp, points_vector, ptr_temp_coord_int, ptr_image_pixel_interpolator );

  list_curves_closed.push_back( ptr_temp_curve );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
bicotiDexelCurveOpen< CoordTypeReal, PixelType > * bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetCurveOpen( unsigned index )
{
  if ( index >= list_curves_open.size() ) 
    throw bicotiException( "bicotiDexelFacade", "GetCurveOpen", 1 );

  return ( list_curves_open[ index ] );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
bicotiDexelCurveClosed< CoordTypeReal, PixelType > * bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetCurveClosed( unsigned index )
{
  if ( index >= list_curves_closed.size() ) 
    throw bicotiException( "bicotiDexelFacade", "GetCurveClosed", 1 );

  return ( list_curves_closed[ index ] );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
unsigned bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetOpenCurvesNumber()
{
  return( list_curves_open.size() );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
unsigned bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: GetClosedCurvesNumber()
{
  return( list_curves_closed.size() );
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: DestroyCurves()
{
  while ( list_curves_open.size() != 0 )
    { delete list_curves_open.back();
    list_curves_open.pop_back();};
  
  while ( list_curves_closed.size() != 0 )
    { delete list_curves_closed.back();
    list_curves_closed.pop_back();};
  
  while ( list_curves_interpolator_open.size() != 0 )
    { delete list_curves_interpolator_open.back();
    list_curves_interpolator_open.pop_back();};
  
  while ( list_curves_interpolator_closed.size() != 0 )
    { delete list_curves_interpolator_closed.back();
    list_curves_interpolator_closed.pop_back();};
  
  while ( temp_coord_vector.size() != 0 )
    temp_coord_vector.pop_back();
  
  if ( ptr_image_pixel_interpolator != NULL )
    delete ptr_image_pixel_interpolator;
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: BuildCurvesProperties()
{
};


template< class CoordTypeInt, class CoordTypeReal , class PixelType >
void bicotiDexelFacade< CoordTypeInt, CoordTypeReal, PixelType > :: DestroyCurvesProperties()
{
};



//---------------------bicotiPropertyFacade2D----------------------
template< class PixelType >
void bicotiDexelFacade2D< PixelType > :: SetPixelInterpolator()
{
  ptr_image_pixel_interpolator = new bicotiImagePixelInterpolator2DLinear< PixelType >( ( bicotiImageImplementation2D< PixelType > * ) ptr_image_imp );
};


template< class PixelType >
void bicotiDexelFacade2D< PixelType > :: UseConnectivityLabelingStrategy4N()
{
  if( ptr_labeling_strategy != NULL )
    delete ptr_labeling_strategy;

 ptr_labeling_strategy = new bicotiConnectivityLabelingStrategy2D4N;
};


template< class PixelType >
bicotiCurveProperties2D< PixelType > * bicotiDexelFacade2D< PixelType> :: GetCurveOpenProperty(unsigned index )
{
  return( NULL );
};


template< class PixelType >
bicotiCurveProperties2DClosed< PixelType > * bicotiDexelFacade2D< PixelType> :: GetCurveClosedProperty( unsigned index )
{
  return( NULL );
};



//---------------------bicotiPropertyFacade3D----------------------
template< class PixelType >
void bicotiDexelFacade3D< PixelType > :: SetPixelInterpolator()
{
  ptr_image_pixel_interpolator = new bicotiImagePixelInterpolator3DLinear< PixelType >( ( bicotiImageImplementation3D< PixelType > * ) ptr_image_imp );
};


template< class PixelType >
void bicotiDexelFacade3D< PixelType > :: UseConnectivityLabelingStrategy6N()
{
  if( ptr_labeling_strategy != NULL )
    delete ptr_labeling_strategy;

  ptr_labeling_strategy = new bicotiConnectivityLabelingStrategy3D6N;
};


template< class PixelType >
bicotiCurveProperties3D< PixelType > * bicotiDexelFacade3D< PixelType > :: GetCurveOpenProperty( unsigned index )
{
  return( NULL );
};


template< class PixelType >
bicotiCurveProperties3D< PixelType > * bicotiDexelFacade3D< PixelType > :: GetCurveClosedProperty( unsigned index )
{
  return( NULL );
};



//---------------------bicotiPropertyFacade2DSingle----------------------
template< class PixelType >
bicotiDexelFacade2DSingle< PixelType > :: bicotiDexelFacade2DSingle( bicotiImageImplementation2D< PixelType > * ptr_imp )
{
  ptr_image_imp = ptr_imp;
  ResetPointers();
};


template< class PixelType >
bicotiDexelFacade2DSingle< PixelType > :: ~bicotiDexelFacade2DSingle()
{
  DeleteMe();
};


template< class PixelType >
void bicotiDexelFacade2DSingle< PixelType > :: UsePixelLabelingCriterionRange( PixelType pixel_start, PixelType pixel_end )
{
  if( ptr_labeling_criterion != NULL )
    delete ptr_labeling_criterion;

  ptr_labeling_criterion = new bicotiPixelLabelingCriterionSingleRange< PixelType >( pixel_start, pixel_end );
};



//---------------------bicotiPropertyFacade2DVectorial----------------------
template< class PixelType >
bicotiDexelFacade2DVectorial< PixelType > :: bicotiDexelFacade2DVectorial( bicotiImageImplementation2D< PixelType > * ptr_imp )
{
  ptr_image_imp = ptr_imp;
  ResetPointers();
};


template< class PixelType >
bicotiDexelFacade2DVectorial< PixelType > :: ~bicotiDexelFacade2DVectorial()
{
   DeleteMe();
};


template< class PixelType >
void bicotiDexelFacade2DVectorial< PixelType > :: UsePixelLabelingCriterionRange( PixelType pixel_start, PixelType pixel_end)
{
  if( ptr_labeling_criterion != NULL )
    delete ptr_labeling_criterion;

  ptr_labeling_criterion = new bicotiPixelLabelingCriterionVectorialRange< PixelType >( pixel_start, pixel_end );
};



//---------------------bicotiPropertyFacade3DSingle----------------------
template< class PixelType >
bicotiDexelFacade3DSingle< PixelType > :: bicotiDexelFacade3DSingle( bicotiImageImplementation3D< PixelType > * ptr_imp )
{
  ptr_image_imp = ptr_imp;
  ResetPointers();
};


template< class PixelType >
bicotiDexelFacade3DSingle< PixelType > :: ~bicotiDexelFacade3DSingle()
{
  DeleteMe();
};


template< class PixelType >
void bicotiDexelFacade3DSingle< PixelType > :: UsePixelLabelingCriterionRange( PixelType pixel_start, PixelType pixel_end)
{
  if( ptr_labeling_criterion != NULL )
    delete ptr_labeling_criterion;

  ptr_labeling_criterion = new bicotiPixelLabelingCriterionSingleRange< PixelType >( pixel_start, pixel_end );
};



//---------------------bicotiPropertyFacade3DVectorial----------------------
template< class PixelType >
bicotiDexelFacade3DVectorial< PixelType > :: bicotiDexelFacade3DVectorial( bicotiImageImplementation3D< PixelType > * ptr_imp )
{
  ptr_image_imp = ptr_imp;
  ResetPointers();
};


template< class PixelType >
bicotiDexelFacade3DVectorial< PixelType > :: ~bicotiDexelFacade3DVectorial()
{
  DeleteMe();
};


template< class PixelType >
void bicotiDexelFacade3DVectorial< PixelType > :: UsePixelLabelingCriterionRange( PixelType pixel_start, PixelType pixel_end )
{
  if( ptr_labeling_criterion != NULL )
    delete ptr_labeling_criterion;

  ptr_labeling_criterion = new bicotiPixelLabelingCriterionSingleRange< PixelType >( pixel_start, pixel_end );
};


#endif
