//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _coordinate_interpolator_1D
#define _coordinate_interpolator_1D



#include <source/dexel/coordinate_interpolator_basic.hpp>



//*******************************DECLARATIONS***********************************
//------------------------ListLengthsSearchCriterion---------------------------
class ListLengthsSearchCriterion
{
private:
  double value;
  
public:
  ListLengthsSearchCriterion( double );
  
  bool operator() ( double );
};


//----------------------------bicotiCoordinateInterpolator1D---------------------------------
template< class CoordType >
class bicotiCoordinateInterpolator1D
{
protected:
  vector< CoordType > list_coords;

  vector< double > list_relative_lengths;
  double length;

  virtual int FindIndex( double ) = 0;

public:
  virtual ~bicotiCoordinateInterpolator1D(){};

  virtual void Initiate( const vector< CoordType >& ) = 0;

  virtual CoordType operator() ( double ) = 0;
  virtual CoordType GetDerivate( double ) = 0;
  virtual CoordType GetDerivate2( double ) = 0;
  virtual CoordType GetDerivate3( double ) = 0;

  double GetLength();

  virtual unsigned GetPointsNumber();
  virtual CoordType GetPoint( int );
  virtual double GetOffset( int );
};



//----------------------------bicotiCoordinateInterpolator1DOpen---------------------------------
template< class CoordType >
class bicotiCoordinateInterpolator1DOpen : public bicotiCoordinateInterpolator1D< CoordType >
{
protected:
  int FindIndex( double );

public:
  virtual ~bicotiCoordinateInterpolator1DOpen(){};
};



//----------------------------bicotiCoordinateInterpolator1DClosed---------------------------------
template< class CoordType >
class bicotiCoordinateInterpolator1DClosed : public bicotiCoordinateInterpolator1D< CoordType >
{
protected:
  int FindIndex( double );

public:
  virtual ~bicotiCoordinateInterpolator1DClosed(){};
};




//******************************IMPLEMENTATIONS*********************************
//----------------------------bicotiCoordinateInterpolator1D---------------------------------
template< class CoordType >
double bicotiCoordinateInterpolator1D< CoordType > :: GetLength()
{
  return ( length );
};


template< class CoordType >
unsigned bicotiCoordinateInterpolator1D< CoordType > :: GetPointsNumber()
{
  return ( list_coords.size() );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1D< CoordType > :: GetPoint( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( index >= list_coords.size() ) ) 
    throw bicotiException( "bicotiCoordinateInterpolator1D", "GetPoint()", 1 );
#endif
  
  return ( list_coords[ index ] );
};


template< class CoordType >
double bicotiCoordinateInterpolator1D< CoordType > :: GetOffset( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( index >= list_coords.size() ) ) 
    throw bicotiException( "bicotiCoordinateInterpolator1D", "GetOffset()", 1 );
#endif

  return ( list_relative_lengths[ index ] );
};



//--------------------------bicotiCoordinateInterpolator1DOpen-------------------------------
template< class CoordType >
int bicotiCoordinateInterpolator1DOpen< CoordType > :: FindIndex( double relative_offset )
{
#ifdef TOTAL_CHECK
  if ( ( relative_offset < 0 ) || ( relative_offset > 1 ) )
    throw bicotiException( "bicotiCoordinateInterpolator1DOpen", "FindIndex()", 1 );
#endif

  int index;
  
  vector< double > :: iterator first = list_relative_lengths.begin();
  vector< double > :: iterator last = list_relative_lengths.end();
  
  index = 0;
  
  ListLengthsSearchCriterion criterioum( relative_offset );
  
  count_if( first, last, criterioum, index );
  index--;
  
  if ( relative_offset == 1 )
    index--;
  
  return( index );
};



//-------------------------bicotiCoordinateInterpolator1DClosed-------------------------------
template< class CoordType >
int bicotiCoordinateInterpolator1DClosed< CoordType > :: FindIndex( double relative_offset )
{
#ifdef TOTAL_CHECK
  if ( ( relative_offset < 0 ) || ( relative_offset > 1 ) )
    throw bicotiException( "bicotiCoordinateInterpolator1DClosed","FindIndex()", 1 );
#endif
  
  int index;
  
  if ( relative_offset == 1 )
    { index = 0; }
  else
    {  vector< double > :: iterator first = list_relative_lengths.begin();
    vector< double > :: iterator last = list_relative_lengths.end();
    
    index = 0;
    
    ListLengthsSearchCriterion criterioum( relative_offset );

#ifdef VISUAL_60
   index  = count_if( first, last, criterioum );
#else
	count_if( first, last, criterioum, index );
#endif
    index--;};
  
  return( index );
};



#endif       
