//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _coordinate_interpolator_1D_open_linear
#define _coordinate_interpolator_1D_open_linear



#include <source/dexel/dexel_curves/coordinate_interpolator_1D.hpp>



//*******************************DECLARATIONS***********************************
//-----------------bicotiCoordinateInterpolator1DOpenLinear---------------------
template< class CoordType >
class bicotiCoordinateInterpolator1DOpenLinear : public bicotiCoordinateInterpolator1DOpen< CoordType >
{
private:
  vector< CoordType > list_directions;

public:
  void Initiate( const vector< CoordType >& );

  CoordType operator()( double );
  CoordType GetDerivate( double );
  CoordType GetDerivate2( double );
  CoordType GetDerivate3( double );
};




//******************************IMPLEMENTATIONS*********************************
//-----------------bicotiCoordinateInterpolator1DOpenLinear---------------------
template< class CoordType >
void bicotiCoordinateInterpolator1DOpenLinear< CoordType > :: Initiate( const vector< CoordType >& coords )
{
  CoordType coord_aux;
  unsigned index;

  list_coords = coords;

  if ( list_coords.size() <= 1 )
    throw bicotiException( "bicotiCoordinateInterpolator1DOpenLinear", "Initiate()", 4 );

  length = 0;
  while( list_directions.size() != 0 )
    list_directions.pop_back();

  while( list_relative_lengths.size() != 0 )
    list_relative_lengths.pop_back();

  list_relative_lengths.push_back( 0 );
  for( index = 0; index < list_coords.size() - 1; index++ )
    {coord_aux = list_coords[ index + 1 ] - list_coords[ index ];
    list_directions.push_back( coord_aux );
    length += coord_aux.Norm();
    list_relative_lengths.push_back( length );}

  for( index = 1; index < list_relative_lengths.size(); index++ )
    list_relative_lengths[ index ] /= length;

  for( index = 0; index < list_directions.size(); index++ )
    list_directions[ index ] = list_directions[ index ] * ( 1/( list_relative_lengths[ index + 1 ] - list_relative_lengths[ index ] ) );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenLinear< CoordType > :: operator() ( double relative_offset )
{
  int index;

  index = FindIndex( relative_offset );
  
  return( list_coords[ index ] + list_directions[ index ] * ( relative_offset - list_relative_lengths[ index ] ) );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenLinear< CoordType > :: GetDerivate( double relative_offset )
{
  int index;

  index = FindIndex( relative_offset );

  return( list_directions[ index ] );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenLinear< CoordType > :: GetDerivate2( double relative_offset )
{
  CoordType aux_coord;

  aux_coord = aux_coord * 0;

  return( aux_coord );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenLinear< CoordType > :: GetDerivate3( double relative_offset )
{
  CoordType aux_coord;

  aux_coord = aux_coord * 0;

  return( aux_coord );
};



#endif
