//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _coordinate_interpolator_function_simpson
#define _coordinate_interpolator_function_simpson



#include <source/dexel/dexel_curves/coordinate_interpolator_function_basic.hpp>



//*******************************DECLARATIONS***********************************
//--------------------------FunctionClassSimpson--------------------------------
template< class CoordType >
class bicotiCoordinateInterpolatorFunctionSimpson : public bicotiCoordinateInterpolatorFunction< CoordType >
{
public:
  bicotiCoordinateInterpolatorFunctionSimpson(){};
  bicotiCoordinateInterpolatorFunctionSimpson( CoordType, CoordType, CoordType );

  double operator()( double );
  CoordType Evaluate( double );
  CoordType EvaluateDer( double );
  double EvaluateDerNorm( double );
  CoordType EvaluateDer2( double );
  CoordType EvaluateDer3( double );
  
  void ComputeConsts();

  CoordType x_2_coeff, x_1_coeff, x_0_coeff;
  double const_1, const_2, const_3;
};



//-----------------------FunctionClassSplineDistance----------------------------
template< class CoordType >
class bicotiCoordinateInterpolatorFunctionDistanceSimpson : public bicotiCoordinateInterpolatorFunctionDistance< CoordType >
{
public:
  bicotiCoordinateInterpolatorFunctionDistanceSimpson(){};

  double Distance( double );
  double Distance( double, double );
};


//******************************IMPLEMENTATIONS*********************************
//-----------------bicotiCoordinateInterpolatorFunctionSimpson------------------
template< class CoordType >
bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: bicotiCoordinateInterpolatorFunctionSimpson( CoordType x_2, CoordType x_1, CoordType x_0 )
{
  x_2_coeff = x_2;
  x_1_coeff = x_1;
  x_0_coeff = x_0;
};


template< class CoordType >
double bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: operator()( double t )
{
  return( EvaluateDerNorm( t ) );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: Evaluate( double t )
{
  return( x_2_coeff * t * t + x_1_coeff * t + x_0_coeff );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: EvaluateDer( double t )
{
  return( x_2_coeff * t * 2 + x_1_coeff );
};


template< class CoordType >
double bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: EvaluateDerNorm( double t )
{
  CoordType aux_coord;
  
  aux_coord = EvaluateDer( t );
  
  return(  aux_coord.Norm() );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: EvaluateDer2( double t )
{
  return( x_2_coeff * 2 );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: EvaluateDer3( double t )
{
  CoordType aux_coord;

  aux_coord = aux_coord * 0;

  return( aux_coord );
};


template< class CoordType >
void bicotiCoordinateInterpolatorFunctionSimpson< CoordType > :: ComputeConsts()
{
  double A, B, C;

  const_1 = 2 * ( x_2_coeff.Norm() );
  
  if( const_1 != 0 )
    {A = const_1 * const_1;
    B = 4 * x_2_coeff.ScalarProduct( x_1_coeff );
    C = x_1_coeff.Norm();
    C *= C;
    const_2 = B / A;
    const_3 = C / A;}
};


//-----------------bicotiCoordinateInterpolatorFunctionSimpsonDistance-----------------------
template< class CoordType >
double bicotiCoordinateInterpolatorFunctionDistanceSimpson< CoordType > :: Distance( double t )
{
  bicotiCoordinateInterpolatorFunctionSimpson< CoordType > * ptr_fun;
  ptr_fun = (bicotiCoordinateInterpolatorFunctionSimpson< CoordType >*) ptr_function;
 
  double d = ptr_fun->const_2;
  double f = ptr_fun->const_3;
  double disc = 4 * f - d * d;
  double aux;
  
  if ( ptr_fun->const_1 != 0 )
    {if( disc != 0 )
      {aux = (double) ( ( .5 * t + .25 * d ) * sqrt( t * t + d * t + f ) );
#ifdef VISUAL_60
  aux -= (double) ( ( .5 * f - .125 * d * d ) * log( (d / sqrt( disc )) + sqrt( 1 + (d / sqrt( disc )) * (d / sqrt( disc ))) ) );
  aux += (double) ( ( .5 * f - .125 * d * d ) * log( (( 2 * t + d ) / sqrt( disc )) + sqrt( 1 + (( 2 * t + d ) / sqrt( disc ))*(( 2 * t + d ) / sqrt( disc ))) ) );
#else
  aux -= (double) ( ( .5 * f - .125 * d * d ) * asinh( d / sqrt( disc ) ) );
  aux += (double) ( ( .5 * f - .125 * d * d ) * asinh( ( 2 * t + d ) / sqrt( disc ) ) );
#endif
      aux -= (double) ( ( .25 * d ) * sqrt( f ) );}
    else
      {if ( d > 0 )
	aux = 0.5 * t * ( t + d );
      else
	{if ( -0.5 * d < t )
	  aux = 0.5 * t * ( t + d ) + 0.25 * d * d;
	else
	  aux = -0.5 * t * ( t + d );}}
    aux *= ptr_fun->const_1;}
  else
    aux = ptr_fun->x_1_coeff.Norm() * t;
  
  return( aux );
};


template< class CoordType >
double bicotiCoordinateInterpolatorFunctionDistanceSimpson< CoordType > :: Distance( double first_t, double last_t )
{
  return( Distance( last_t ) - Distance( first_t ) );
};



#endif     
