//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _dexel_curves_basic
#define _dexel_curves_basic



#include <source/dexel/dexel_basic.hpp>
#include <source/dexel/coordinate_interpolator.hpp>


//*******************************DECLARATIONS***********************************
//-----------------------------bicotiDexelCurve---------------------------------
template< class CoordType, class PixelType >
class PixelCoord
{
public:
  PixelCoord( PixelType, CoordType );

  PixelType pixel;
  CoordType coordinate;
};


template< class CoordType, class PixelType > // OJO NO QUEDO VIRTUAL
class bicotiDexelCurve : public bicotiDexel< PixelType > , public bicotiFunction< CoordType >
{
protected:
  vector< CoordType > list_coordinates;
  vector< PixelCoord< CoordType, PixelType > > list_pix_coord;
  vector< PixelCoord< CoordType, PixelType > > list_pix_coord_back;
  vector< bicotiPixelOperatorUnary< PixelType > * > operations;
  bicotiCoordinateInterpolator1D< CoordType > * ptr_coord_interpolator;
  bicotiImagePixelInterpolator< PixelType > * ptr_image_pixel_interp;

public:
  void UpdateImageImplementation();
  void UpdateDexel();
  void UndoUpdateImageImplementation();

  CoordType operator() ( double );
  CoordType GetCoordinate( double );
  CoordType GetDerivate( double );
  CoordType GetDerivate2( double );
  CoordType GetDerivate3( double );

  PixelType GetPixel( double );
  vector< CoordType >& ListCoordinates();
  void SetPixel( double, PixelType );

  double GetLength();

  unsigned GetPointsNumber();
  CoordType GetPoint( unsigned );
  double GetOffset( unsigned );

  void ModifyPixels( bicotiPixelOperatorUnary< PixelType > * );
  void DrawCurve( PixelType );
};



//-----------------------------bicotiDexelCurveOpen-----------------------------------
template< class CoordType, class PixelType >
class bicotiDexelCurveOpen : public bicotiDexelCurve< CoordType, PixelType >
{
public:
  bicotiDexelCurveOpen( bicotiImageImplementation< PixelType > *, const vector< CoordType >& , bicotiCoordinateInterpolator1DOpen< CoordType > * , bicotiImagePixelInterpolator< PixelType > * );
  bicotiDexelCurveOpen(){};

  void SetInfo( bicotiImageImplementation< PixelType > *, const vector< CoordType >& , bicotiCoordinateInterpolator1DOpen< CoordType > * , bicotiImagePixelInterpolator< PixelType > * );
};



//----------------------------bicotiDexelCurveClosed----------------------------------
template< class CoordType, class PixelType >
class bicotiDexelCurveClosed : public bicotiDexelCurve< CoordType, PixelType >
{
public:
  bicotiDexelCurveClosed( bicotiImageImplementation< PixelType > *, const vector< CoordType >& , bicotiCoordinateInterpolator1DClosed< CoordType > * , bicotiImagePixelInterpolator< PixelType > * );
  bicotiDexelCurveClosed(){};
  
  void SetInfo( bicotiImageImplementation< PixelType > *, const vector< CoordType >& , bicotiCoordinateInterpolator1DClosed< CoordType > * , bicotiImagePixelInterpolator< PixelType > * );
};




//******************************IMPLEMENTATIONS*********************************
//-----------------------------bicotiDexelCurve---------------------------------
template< class CoordType, class PixelType >
PixelCoord< CoordType, PixelType > :: PixelCoord( PixelType pix, CoordType coord )
{
  pixel = pix;
  coordinate = coord;
};


template< class CoordType, class PixelType >
void bicotiDexelCurve< CoordType, PixelType > :: UpdateImageImplementation()
{
  bicotiCoordinate< INTEGER > coord_aux( list_pix_coord[ 0 ].coordinate.GetDimension() );
  unsigned index, coord_index;

  list_pix_coord_back = list_pix_coord;
  
  for( index = 0; index < list_pix_coord_back.size(); index++ )
    {for( coord_index = 0; coord_index < coord_aux.GetDimension(); coord_index++ )
      coord_aux( coord_index ) = ( INTEGER )list_pix_coord_back[ index ].coordinate( coord_index );
    list_pix_coord_back[ index ].pixel = ptr_image_imp->GetPixel( coord_aux );}

  while( list_pix_coord.size() != 0 )
    {for( coord_index = 0; coord_index < coord_aux.GetDimension(); coord_index++ )
      coord_aux( coord_index ) = ( INTEGER )list_pix_coord.back().coordinate( coord_index );
    ptr_image_imp->SetPixel( coord_aux, list_pix_coord.back().pixel );
    list_pix_coord.pop_back();}

  if( ptr_image_imp == NULL )
    throw bicotiException( "bicotiDexelCurve", "UpdateImageImplementation", 6 );

  unsigned index_fun ;
  unsigned points = (unsigned)( ptr_coord_interpolator->GetLength()); 
  PixelType temp_pixel;
  CoordType temp_coord;
  
  for( index_fun = 0; index_fun < operations.size(); index_fun++ )
    
    for( index = 0; index < points; index ++ )
      { temp_pixel = GetPixel( ( (double)index/points ) );
      temp_pixel = ( *( operations[ index_fun ] ) )(temp_pixel);
      temp_coord = GetCoordinate(( (double)index/points ) ); 
      
      for( coord_index = 0; coord_index < coord_aux.GetDimension(); coord_index++ )
	coord_aux( coord_index ) = ( INTEGER )temp_coord( coord_index );
      
      ptr_image_imp->SetPixel( coord_aux, temp_pixel );};
  
  while( operations.size() != 0 )
    operations.pop_back();
};


template< class CoordType, class PixelType >
void bicotiDexelCurve< CoordType, PixelType > :: UpdateDexel()
{
  ptr_coord_interpolator->Initiate( list_coordinates );
};


template< class CoordType, class PixelType >
void bicotiDexelCurve< CoordType, PixelType > :: UndoUpdateImageImplementation()
{
  bicotiCoordinate< INTEGER > coord_aux( list_pix_coord[ 0 ].coordinate.GetDimension() );
  unsigned index;

  while ( list_pix_coord_back.size() != 0 )
    {for( index = 0; index < coord_aux.GetDimension(); index++ )
      coord_aux( index ) = ( INTEGER )list_pix_coord_back.back().coordinate( index );    
    ptr_image_imp->SetPixel( coord_aux, list_pix_coord_back.back().pixel );
    list_pix_coord_back.pop_back();}
};


template< class CoordType, class PixelType >
CoordType bicotiDexelCurve< CoordType, PixelType > :: operator() ( double relative_offset )
{
  return( ( *ptr_coord_interpolator )( relative_offset ) );
};


template< class CoordType, class PixelType >
CoordType bicotiDexelCurve< CoordType, PixelType > :: GetCoordinate( double relative_offset )
{
  return( ( *ptr_coord_interpolator )( relative_offset ) );
};


template< class CoordType, class PixelType >
CoordType bicotiDexelCurve< CoordType, PixelType > :: GetDerivate( double relative_offset )
{
  return( ptr_coord_interpolator->GetDerivate( relative_offset ) );
};


template< class CoordType, class PixelType >
CoordType bicotiDexelCurve< CoordType, PixelType > :: GetDerivate2( double relative_offset )
{
  return( ptr_coord_interpolator->GetDerivate2( relative_offset ) );
};


template< class CoordType, class PixelType >
CoordType bicotiDexelCurve< CoordType, PixelType > :: GetDerivate3( double relative_offset )
{
  return( ptr_coord_interpolator->GetDerivate3( relative_offset ) );
};


template< class CoordType, class PixelType >
inline PixelType bicotiDexelCurve< CoordType, PixelType > :: GetPixel( double relative_offset )
{
  CoordType temp_coord = GetCoordinate( relative_offset );
  PixelType temp_pixel = ( *ptr_image_pixel_interp )( temp_coord );

  return( temp_pixel );
};


template< class CoordType, class PixelType >
vector< CoordType >& bicotiDexelCurve< CoordType, PixelType > :: ListCoordinates()
{
  return( list_coordinates );
};


template< class CoordType, class PixelType >
void bicotiDexelCurve< CoordType, PixelType > :: SetPixel( double relative_offset, PixelType pixel )
{
  PixelCoord< CoordType, PixelType > aux_pix_coord( pixel, GetCoordinate( relative_offset ) );
  list_pix_coord.push_back( aux_pix_coord );
};


template< class CoordType, class PixelType >
double bicotiDexelCurve< CoordType, PixelType > :: GetLength()
{
  return ( ptr_coord_interpolator->GetLength() * ptr_image_imp->GetScale() );
};

template< class CoordType, class PixelType >
unsigned bicotiDexelCurve< CoordType, PixelType > :: GetPointsNumber()
{
  return ( ptr_coord_interpolator->GetPointsNumber() );
};


template< class CoordType, class PixelType >
CoordType bicotiDexelCurve< CoordType, PixelType > :: GetPoint( unsigned  point_index)
{
  return ( ptr_coord_interpolator->GetPoint( point_index ) );
};


template< class CoordType, class PixelType >
double bicotiDexelCurve< CoordType, PixelType > :: GetOffset( unsigned point_index )
{
  return ( ptr_coord_interpolator->GetOffset( point_index ) );
};


template< class CoordType, class PixelType > 
void bicotiDexelCurve< CoordType, PixelType > :: ModifyPixels( bicotiPixelOperatorUnary< PixelType > * ptr_pix_oper )
{
  if( ptr_image_imp == NULL )
    throw bicotiException( "bicotiDexelCurve", "ModifyPixels", 6 );

  operations.push_back( ptr_pix_oper );
};


template< class CoordType, class PixelType > 
void bicotiDexelCurve< CoordType, PixelType > :: DrawCurve( PixelType pix_val )
{
  bicotiPixelOperatorUnary< PixelType > * ptr_oper;
  ptr_oper = new bicotiPixelOperatorUnaryConstant< PixelType >(pix_val);
  operations.push_back( ptr_oper );
  UpdateImageImplementation();
};


//-----------------------------bicotiDexelCurveOpen-----------------------------------
template< class CoordType, class PixelType >
bicotiDexelCurveOpen< CoordType, PixelType > :: bicotiDexelCurveOpen( bicotiImageImplementation< PixelType > * ptr_ima_imp, const vector< CoordType >& list_coords, bicotiCoordinateInterpolator1DOpen< CoordType > * ptr_coord_inter, bicotiImagePixelInterpolator< PixelType > * ptr_ima_pix_interp )
{
  ptr_image_imp = ptr_ima_imp;
  list_coordinates = list_coords;
  ptr_coord_interpolator = ptr_coord_inter;
  ptr_image_pixel_interp = ptr_ima_pix_interp;
  
  ptr_coord_interpolator->Initiate( list_coordinates );
  ptr_image_pixel_interp->SetImageImplementation( ptr_image_imp );
};


template< class CoordType, class PixelType >
void bicotiDexelCurveOpen< CoordType, PixelType > :: SetInfo( bicotiImageImplementation< PixelType > * ptr_ima_imp, const vector< CoordType >& list_coords, bicotiCoordinateInterpolator1DOpen< CoordType > * ptr_coord_inter, bicotiImagePixelInterpolator< PixelType > * ptr_ima_pix_interp )
{
  ptr_image_imp = ptr_ima_imp;
  list_coordinates = list_coords;
  ptr_coord_interpolator = ptr_coord_inter;
  ptr_image_pixel_interp = ptr_ima_pix_interp;
  
  ptr_coord_interpolator->Initiate( list_coordinates );
  ptr_image_pixel_interp->SetImageImplementation( ptr_image_imp );
};


//----------------------------bicotiDexelCurveClosed----------------------------------
template< class CoordType, class PixelType >
bicotiDexelCurveClosed< CoordType, PixelType > :: bicotiDexelCurveClosed( bicotiImageImplementation< PixelType > * ptr_ima_imp, const vector< CoordType >& list_coords, bicotiCoordinateInterpolator1DClosed< CoordType > * ptr_coord_inter, bicotiImagePixelInterpolator< PixelType > * ptr_ima_pix_interp )
{
  ptr_image_imp = ptr_ima_imp;
  list_coordinates = list_coords;
  ptr_coord_interpolator = ptr_coord_inter;
  ptr_image_pixel_interp = ptr_ima_pix_interp;
  
  ptr_coord_interpolator->Initiate( list_coordinates );
  ptr_image_pixel_interp->SetImageImplementation( ptr_image_imp );
};


template< class CoordType, class PixelType >
void bicotiDexelCurveClosed< CoordType, PixelType > :: SetInfo( bicotiImageImplementation< PixelType > * ptr_ima_imp, const vector< CoordType >& list_coords, bicotiCoordinateInterpolator1DClosed< CoordType > * ptr_coord_inter, bicotiImagePixelInterpolator< PixelType > * ptr_ima_pix_interp )
{
  ptr_image_imp = ptr_ima_imp;
  list_coordinates = list_coords;
  ptr_coord_interpolator = ptr_coord_inter;
  ptr_image_pixel_interp = ptr_ima_pix_interp;
  
  ptr_coord_interpolator->Initiate( list_coordinates );
  ptr_image_pixel_interp->SetImageImplementation( ptr_image_imp );
};



#endif   
