//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _dexel_histogram_basic
#define _dexel_histogram_basic


#include <source/dexel/dexel_basic.hpp>


//**************************DECLARATIONS************************
//---------------------------HistoInfo--------------------------
template< class CoordType, class ColorType >
class HistoInfo
{
private:
  ColorType color_value;
  vector< CoordType > list_coords;
  vector< CoordType > :: iterator first;
  vector< CoordType > :: iterator last;
  
public:
  HistoInfo( ColorType );
  HistoInfo( const HistoInfo< CoordType, ColorType >& );
  void operator=( const HistoInfo< CoordType, ColorType >& );
  
  ColorType& ColorValue();
  
  void Append( const CoordType& );
  unsigned Number();
  bool IsIn( const CoordType& );
  void First();
  void Next();
  CoordType& Current();
  bool IsDone();
};


//--------------------HistoInfoSearchCriterioum-----------------
template< class CoordType, class ColorType >
class HistoInfoSearchCriterion
{
public:
  bool operator() ( HistoInfo< CoordType, ColorType >& );
  ColorType& ColorValue();
  
private:
  ColorType color_value;
};


//----------------------Comparation Criterion-------------------
template< class CoordType, class ColorType >
class HistoInfoCompare
{
public:
  bool operator() ( HistoInfo< CoordType, ColorType >&, HistoInfo< CoordType, ColorType >& );
};


//-----------------------bicotiDexelHistogram-------------------
template< class CoordType, class ColorType, class PixelType >
class bicotiHistogramEqualizator;

template< class CoordType, class ColorType, class PixelType = ColorType >
class bicotiDexelHistogram : public bicotiDexel< PixelType >
{
  friend class bicotiHistogramEqualizator< CoordType, ColorType, PixelType >;

private:
  list< HistoInfo< CoordType, ColorType > > list_histo_info;
  list< HistoInfo< CoordType, ColorType > > :: iterator first;
  list< HistoInfo< CoordType, ColorType > > :: iterator last;
  
  vector< bicotiPixelOperatorUnary< ColorType > * > list_operations;
  
  void CreateMe();
  void Append( const ColorType&, const CoordType& );

  bicotiPixelAttributeSelector< ColorType, PixelType > * ptr_pixel_attribute_selector;
  
public:
  bicotiDexelHistogram( bicotiImageImplementation< PixelType > *, bicotiPixelAttributeSelector< ColorType, PixelType > *);
  
  void UpdateImageImplementation();
  void UpdateDexel();

  void First();
  void Next();
  bool IsDone();
  
  unsigned GetCurrentPointsNumber();
  ColorType GetCurrentColor();
  
  void SelfFit();
  void ModifyPixels( bicotiPixelOperatorUnary< ColorType > * );
};



//*************************IMPLEMENTATION***********************
//----------------------------HistoInfo-------------------------
template< class CoordType, class ColorType >
HistoInfo< CoordType, ColorType > :: HistoInfo( ColorType col_value ) 
{
  color_value = col_value;
};


template< class CoordType, class ColorType >
HistoInfo< CoordType, ColorType > :: HistoInfo( const HistoInfo< CoordType, ColorType >& histo_info_src ) 
{
  color_value = histo_info_src.color_value;
  list_coords = histo_info_src.list_coords;
  first = list_coords.begin() + (histo_info_src.first - histo_info_src.list_coords.begin());
  last  = list_coords.end()   - (histo_info_src.list_coords.end() - histo_info_src.last);
};


template< class CoordType, class ColorType >
void HistoInfo< CoordType, ColorType > :: operator=( const HistoInfo< CoordType, ColorType >& histo_info_src ) 
{
  color_value = histo_info_src.color_value;
  list_coords = histo_info_src.list_coords;
  first = list_coords.begin() + (histo_info_src.first - histo_info_src.list_coords.begin());
  last  = list_coords.end()   - (histo_info_src.list_coords.end() - histo_info_src.last);
};


template< class CoordType, class ColorType >
ColorType& HistoInfo< CoordType, ColorType > :: ColorValue()
{
  return ( color_value );
};


template< class CoordType, class ColorType >
void HistoInfo< CoordType, ColorType > :: Append( const CoordType& coord )
{
  list_coords.push_back( coord );
};


template< class CoordType, class ColorType >
unsigned HistoInfo< CoordType, ColorType > :: Number()
{
  return list_coords.size();
};


template< class CoordType, class ColorType >
bool HistoInfo< CoordType, ColorType > :: IsIn( const CoordType& coord )
{
  vector< CoordType > :: iterator first_aux;
  vector< CoordType > :: iterator last_aux;
  
  first_aux = list_coords.begin();
  last_aux  = list_coords.end();
  first_aux = find( first_aux, last_aux, coord );
  return ( first_aux != last_aux );
};


template< class CoordType, class ColorType >
void HistoInfo< CoordType, ColorType > :: First()
{
  first = list_coords.begin();
  last  = list_coords.end();
};


template< class CoordType, class ColorType >
void HistoInfo< CoordType, ColorType > :: Next()
{
  if (first != last) first++;
};


template< class CoordType, class ColorType >
CoordType& HistoInfo< CoordType, ColorType > :: Current()
{
  if ( first != last ) 
    return *first;
  else
    return *( first-1 );
};


template< class CoordType, class ColorType >
bool HistoInfo< CoordType, ColorType > :: IsDone()
{
  return ( first == last );
};



//---------------------HistoInfoSearchCriterion-----------------
template< class CoordType, class ColorType > 
ColorType& HistoInfoSearchCriterion< CoordType, ColorType > :: ColorValue()
{
  return ( color_value );
};


template< class CoordType, class ColorType > 
bool HistoInfoSearchCriterion< CoordType, ColorType > :: operator() ( HistoInfo< CoordType, ColorType >& hist_val ) 
{
  return ( color_value == hist_val.ColorValue() );
};



//----------------------Comparation Criterion-------------------
template< class CoordType, class ColorType >
bool HistoInfoCompare< CoordType, ColorType > :: operator() ( HistoInfo< CoordType, ColorType >& histo_info_1, HistoInfo< CoordType, ColorType >& histo_info_2 )
{
  return ( histo_info_1.ColorValue() <= histo_info_2.ColorValue() );
};



//-----------------------bicotiDexelHistogram-------------------
template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: CreateMe()
{
  while ( list_histo_info.size() != 0 )
    list_histo_info.pop_back();
  
  while ( list_operations.size() != 0 )
    list_operations.pop_back();

  bicotiImageIterator< PixelType > * ptr_iter;  
  ptr_iter = ptr_image_imp->CreateIterator();

  for( ptr_iter->First(); !ptr_iter->IsDone(); (*ptr_iter)++ )
    Append( ( * ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) ),  (CoordType)( ptr_iter->GetCurrentCoordinateInteger() ) );
  
  HistoInfoCompare< CoordType, ColorType > compar_criterium;

#ifdef VISUAL_60
  list< HistoInfo< CoordType, ColorType > > :: iterator first_iter;
  list< HistoInfo< CoordType, ColorType > > :: iterator last_iter;

  vector< HistoInfo< CoordType, ColorType > > list_histo_info_aux;
  first_iter = list_histo_info.begin();
  last_iter = list_histo_info.end();
  while( first_iter != last_iter )
  {list_histo_info_aux.push_back( *first_iter );
  first_iter++;}
  sort( list_histo_info_aux.begin(), list_histo_info_aux.end(), compar_criterium );
  first_iter = list_histo_info.begin();
  last_iter = list_histo_info.end();
  for( unsigned int aux_index = 0; aux_index < list_histo_info_aux.size(); aux_index++, first_iter++ )
	  *first_iter = list_histo_info_aux[ aux_index ];
#else
  list_histo_info.sort( compar_criterium );
#endif
  
  delete ptr_iter;
};


template< class CoordType, class ColorType, class PixelType >
bicotiDexelHistogram< CoordType, ColorType, PixelType > :: bicotiDexelHistogram( bicotiImageImplementation< PixelType > * ptr_ima_imp, bicotiPixelAttributeSelector< ColorType, PixelType > * ptr_color_selector )
{
  ptr_image_imp = ptr_ima_imp;
  ptr_pixel_attribute_selector = ptr_color_selector;

  CreateMe();
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: UpdateImageImplementation()
{
  unsigned index_fun;
  ColorType temp_color;
  
  first = list_histo_info.begin();
  last  = list_histo_info.end();
  
  for( index_fun = 0; index_fun < list_operations.size(); index_fun++ )
    for( First(); !IsDone(); Next() )
      for( (*first).First(); !(*first).IsDone(); (*first).Next() )
	{ temp_color = ( *first ).ColorValue();
	temp_color = (*(list_operations[ index_fun ]))(temp_color);
	(( *first ).ColorValue()) = temp_color; };
  
  while( list_operations.size() != 0 )
    list_operations.pop_back();
  
  
  for( First(); !IsDone(); Next() )
    {for( (*first).First(); !(*first).IsDone(); (*first).Next() )
      {ptr_image_imp->SetPixel( ( *first ).Current(), ptr_pixel_attribute_selector->Inverse( ptr_image_imp->GetPixel( ( *first ).Current() ), ( *first ).ColorValue() ) );}}
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: UpdateDexel()
{
  CreateMe();
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: Append( const ColorType& col_val, const CoordType& coord_val )
{
  HistoInfoSearchCriterion< CoordType, ColorType > criterioum;
  criterioum.ColorValue() = col_val;
  first = list_histo_info.begin();
  last  = list_histo_info.end();
  
  first = find_if( first, last, criterioum );
  
  if ( first != last )
    ( *first ).Append( coord_val );
  else
    {HistoInfo< CoordType, ColorType > histo_info_aux( col_val );
    histo_info_aux.Append( coord_val );
    list_histo_info.push_back( histo_info_aux );}
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: First()
{
  first = list_histo_info.begin();
  last  = list_histo_info.end();
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: Next()
{
  if ( first != last ) first++;
};


template< class CoordType, class ColorType, class PixelType >
unsigned bicotiDexelHistogram< CoordType, ColorType, PixelType > :: GetCurrentPointsNumber()
{
  return ( ( *first ).Number() );
};


template< class CoordType, class ColorType, class PixelType >
ColorType bicotiDexelHistogram< CoordType, ColorType, PixelType > :: GetCurrentColor()
{
  return ( ( *first ).ColorValue() );
};


template< class CoordType, class ColorType, class PixelType >
bool bicotiDexelHistogram< CoordType, ColorType, PixelType > :: IsDone()
{
  return ( first == last );
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: SelfFit()
{
  unsigned inner_offset = 0, index;
  
  list< HistoInfo< CoordType, ColorType > > :: iterator first_iter;
  list< HistoInfo< CoordType, ColorType > > :: iterator middle_iter;
  list< HistoInfo< CoordType, ColorType > > :: iterator last_iter;

  HistoInfoCompare< CoordType, ColorType > compar_criterium;

#ifdef VISUAL_60
  vector< HistoInfo< CoordType, ColorType > > list_histo_info_aux;
  first_iter = list_histo_info.begin();
  last_iter = list_histo_info.end();
  while( first_iter != last_iter )
  {list_histo_info_aux.push_back( *first_iter );
  first_iter++;}
  sort( list_histo_info_aux.begin(), list_histo_info_aux.end(), compar_criterium );
  first_iter = list_histo_info.begin();
  last_iter = list_histo_info.end();
  for( unsigned int aux_index = 0; aux_index < list_histo_info_aux.size(); aux_index++, first_iter++ )
	  *first_iter = list_histo_info_aux[ aux_index ];
#else
  list_histo_info.sort( compar_criterium );
#endif

  first_iter = list_histo_info.begin();
  last_iter = list_histo_info.end();
  middle_iter = first_iter;
  middle_iter++;

  while ( middle_iter != last_iter )
    {if ( ( *middle_iter ).ColorValue() == ( *first_iter ).ColorValue() )
      {for( ( *middle_iter ).First(); !( ( *middle_iter ).IsDone() ); ( *middle_iter ).Next() )
	( *first_iter ).Append( ( *middle_iter ).Current() );
      list_histo_info.erase( middle_iter );
      first_iter = list_histo_info.begin();
      for ( index=0; index < inner_offset; index++)
	first_iter++;
      middle_iter = first_iter;
      middle_iter++;
      last_iter = list_histo_info.end();}
    else
      {first_iter++;
      inner_offset++;
      middle_iter++;}}
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelHistogram< CoordType, ColorType, PixelType > :: ModifyPixels( bicotiPixelOperatorUnary< ColorType > * ptr_pix_oper )
{
  if( ptr_image_imp == NULL )
    throw bicotiException( "bicotiDexelRegion", "ModifyPixels", 6 );

  list_operations.push_back( ptr_pix_oper );
};



#endif
