//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _dexel_labeling_basic
#define _dexel_labeling_basic



#include <source/dexel/dexel_labeling/pixel_labeling_criterion.hpp>
#include <source/dexel/dexel_labeling/connectivity_labeling_strategy.hpp>
#include <source/dexel/dexel_region/dexel_region.hpp>



//********************************DECLARATIONS***********************************
//-----------------------------bicotiDexelLabeling-------------------------------
template< class CoordType, class PixelType > 
class bicotiDexelLabeling : public bicotiDexel< PixelType > 
{ 
private:
  bicotiPixelLabelingCriterion< PixelType > * ptr_pix_criter;
  bicotiConnectivityLabelingStrategy * ptr_connect_criter;

  list< bicotiDexelRegion< CoordType, PixelType > > regions;
  vector< bicotiPixelOperatorUnary< PixelType > * > operations;
  
  void CreateMe();
  
public:
  bicotiDexelLabeling( bicotiImageImplementation< PixelType > *, bicotiPixelLabelingCriterion< PixelType > * = NULL, bicotiConnectivityLabelingStrategy * = NULL );
  
  void UpdateImageImplementation();
  void ModifyPixels( vector< bicotiPixelOperatorUnary< PixelType > * > );

  void UpdateDexel();
  void SetConnectivityStrategy( bicotiConnectivityLabelingStrategy * );
  void SetPixelCriterion( bicotiPixelLabelingCriterion< PixelType > * );

  unsigned GetRegionsNumber();
  bicotiDexelRegion< CoordType, PixelType >& GetRegion( unsigned );
}; 



//******************************IMPLEMENTATIONS*********************************
//----------------------------bicotiDexelLabeling-------------------------------
template< class CoordType, class PixelType > 
bicotiDexelLabeling< CoordType, PixelType > :: bicotiDexelLabeling( bicotiImageImplementation< PixelType > * ptr_ima_imp, bicotiPixelLabelingCriterion< PixelType > * ptr_pix_crit, bicotiConnectivityLabelingStrategy * ptr_connect_crit )
{ 
  ptr_image_imp = ptr_ima_imp;
  ptr_connect_criter = ptr_connect_crit;
  ptr_pix_criter = ptr_pix_crit;
  if( ( ptr_pix_criter != NULL ) && ( ptr_connect_criter != NULL ) )
     CreateMe();    
};


template< class CoordType, class PixelType >
void bicotiDexelLabeling< CoordType, PixelType > :: SetConnectivityStrategy( bicotiConnectivityLabelingStrategy* ptr_connect_crit)
{
  ptr_connect_criter = ptr_connect_crit;
};


template< class CoordType, class PixelType >
void bicotiDexelLabeling< CoordType, PixelType > :: SetPixelCriterion( bicotiPixelLabelingCriterion< PixelType >* ptr_pix_crit)
{
  ptr_pix_criter = ptr_pix_crit;
};


template< class CoordType, class PixelType >
void bicotiDexelLabeling< CoordType, PixelType > :: UpdateDexel()
{
   CreateMe();
};


template< class CoordType, class PixelType > 
void bicotiDexelLabeling< CoordType, PixelType > :: CreateMe()
{ 
  if( ( ptr_pix_criter != NULL ) && ( ptr_connect_criter != NULL ) )
    {
      unsigned index_1, index_2, size;
      bicotiImageIterator< PixelType > * ptr_iter;
      bicotiMatrix< unsigned > * ptr_matrix;
      bicotiMatrixIterator< unsigned >* ptr_iter_matrix;
      vector< vector< unsigned > > equivalence_table;
  
      ptr_connect_criter->Initiate( ptr_image_imp->CreateImageDescriptor(), &equivalence_table ); 
 
      ptr_iter = ptr_image_imp->CreateIterator();

      for( ptr_iter->First(); !ptr_iter->IsDone(); ( *ptr_iter )++ )
	{if( ( *ptr_pix_criter )( **ptr_iter ) )
	  ( *ptr_connect_criter )( ptr_iter->GetCurrentCoordinateInteger() );}
  
      ptr_matrix = ptr_connect_criter->CreateResult();
      ptr_iter_matrix = ptr_matrix->CreateIterator();

      for( index_1 = 0; index_1 < equivalence_table.size(); index_1++ )
	{for( index_2 = 1; index_2 < equivalence_table[ index_1 ].size(); index_2++ )
	  equivalence_table[ equivalence_table[ index_1 ][ index_2 ] - 1 ][ 0 ] = equivalence_table[ index_1 ][ 0 ];}

      list< set< unsigned > > table_eq( equivalence_table.size() );
      list< set< unsigned > > :: iterator current_eq = table_eq.begin();
      list< set< unsigned > > :: iterator last_eq = table_eq.end();

      for( index_1 = 0; index_1 < equivalence_table.size(); index_1++ )
	{for( index_2 = 0; index_2 < equivalence_table[ index_1 ].size(); index_2++ )
	  (*current_eq).insert( equivalence_table[ index_1 ][ index_2 ] );
	current_eq++;}

      list< set< unsigned > > :: iterator next_eq;      
      list< unsigned > aux_list;
      list< unsigned > :: iterator first_aux;
      list< unsigned > :: iterator last_aux;
      bool resume;
  
 
      current_eq = table_eq.begin();
      last_eq = table_eq.end();
      resume = false;
      while( !resume )
	{next_eq = current_eq;
	next_eq++;
	if( next_eq != last_eq )
	  {
#ifdef VISUAL_60
		if( (*current_eq).size() <= (*next_eq).size() )
			aux_list = list< unsigned >((*current_eq).size());
		else
			aux_list = list< unsigned >((*next_eq).size());
#else
		aux_list = list< unsigned >( min( (*current_eq).size(), (*next_eq).size() ) );
#endif
	  first_aux = aux_list.begin();
	  last_aux = set_intersection( (*current_eq).begin(), (*current_eq).end(), (*next_eq).begin(), (*next_eq).end(), first_aux );}
	while( ( next_eq != last_eq ) && ( first_aux == last_aux ) )
	  {next_eq++;       
	  if( next_eq != last_eq )
	    {
#ifdef VISUAL_60
		  if( (*current_eq).size() <= (*next_eq).size() )
			  aux_list = list< unsigned >((*current_eq).size());
		  else
			  aux_list = list< unsigned >((*next_eq).size());
#else
		  aux_list = list< unsigned >( min( (*current_eq).size(), (*next_eq).size() ) );
#endif
	    first_aux = aux_list.begin();
	    last_aux = set_intersection( (*current_eq).begin(), (*current_eq).end(), (*next_eq).begin(), (*next_eq).end(), first_aux );}}
	if( next_eq == last_eq )
	  {current_eq++;
	  if( current_eq == last_eq )
	    resume = true;}
	else
	  {aux_list = list< unsigned >( (*current_eq).size() + (*next_eq).size() );
	  first_aux = aux_list.begin();
	  last_aux = set_union( (*current_eq).begin(), (*current_eq).end(), (*next_eq).begin(), (*next_eq).end(), first_aux );

#ifdef VISUAL_60
	  while( first_aux != last_aux )
	  {(*current_eq).insert( *first_aux );
	  first_aux++;}
#else
	  *current_eq = set< unsigned >( first_aux, last_aux );
#endif

	  table_eq.erase( next_eq );}}
  
      current_eq = table_eq.begin();
      last_eq = table_eq.end();
      set< unsigned > :: iterator first_set;
      set< unsigned > :: iterator last_set;
      index_1 = 1;
      while( current_eq != last_eq )
	{first_set = (*current_eq).begin();
	last_set = (*current_eq).end();
	while( first_set != last_set )
	  {equivalence_table[ *first_set - 1 ][ 0 ] = index_1;
	  first_set++;}
	index_1++;
	current_eq++;}
      CoordType coord_aux;
      regions = list< bicotiDexelRegion< CoordType, PixelType > >( table_eq.size() );
      list< bicotiDexelRegion< CoordType, PixelType > > :: iterator first = regions.begin();
      list< bicotiDexelRegion< CoordType, PixelType > > :: iterator last = regions.end();
  
      while( first != last )
	{(*first).SetImageImplementation( ptr_image_imp );
	first++;}
  
      for( ptr_iter_matrix->First(); !ptr_iter_matrix->IsDone(); ptr_iter_matrix->Next() )
	{if( ptr_iter_matrix->CurrentPixel() != 0 )
	  {coord_aux = (CoordType)(ptr_iter_matrix->CurrentCoordinate());
	  first = regions.begin();
	  for(index_1 = 0; index_1 < (equivalence_table[ ptr_iter_matrix->CurrentPixel() - 1 ][ 0 ] - 1 ); index_1++)
	    first++;      
	  (*first).Append( coord_aux );}}
  
      delete ptr_iter_matrix;
      delete ptr_matrix;
      delete ptr_iter;
    }
else
  throw bicotiException( "bicotiDexelLabeling", "CreateMe()", 6 );
};


template< class CoordType, class PixelType > 
void bicotiDexelLabeling< CoordType, PixelType > :: UpdateImageImplementation()
{

  if ( operations.size() == regions.size() )
    { list< bicotiDexelRegion< CoordType, PixelType > > :: iterator first;
    list< bicotiDexelRegion< CoordType, PixelType > > :: iterator last = regions.end();
    
    int index;
    for( first = regions.begin(), index = 0; first != last; first++, index++ ) 
      {(*first).ModifyPixels( operations[ index ] );
      (*first).UpdateImageImplementation();}}
  else 
    throw bicotiException( "bicotiDexelLabeling", "UpdateImageImplementation()", 2 );
};


template< class CoordType, class PixelType > 
void bicotiDexelLabeling< CoordType, PixelType > :: ModifyPixels( vector< bicotiPixelOperatorUnary< PixelType > * > vector_oper )
{ 
  if ( vector_oper.size() == regions.size() )
    operations = vector_oper;
  else
    throw bicotiException( "bicotiDexelLabeling", "ModifyPixels()", 2 );
};


template< class CoordType, class PixelType >
unsigned bicotiDexelLabeling< CoordType, PixelType > :: GetRegionsNumber()
{
  return( regions.size() );
};


template< class CoordType, class PixelType >
bicotiDexelRegion< CoordType, PixelType >& bicotiDexelLabeling< CoordType, PixelType > :: GetRegion( unsigned index )
{
  list< bicotiDexelRegion< CoordType, PixelType > > :: iterator first = regions.begin();
  list< bicotiDexelRegion< CoordType, PixelType > > :: iterator last = regions.end();
  
  unsigned count = 0;
  
  if ( index < regions.size() )
    {while( ( first != last ) && ( count < index ) )
	{first++;
	count++;}
    }
else
  throw bicotiException( "bicotiDexelLabeling", "GetRegion()", 2 );

  return( *first );
}; 



#endif  
