//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _pixel_labeling_criterion_basic
#define _pixel_labeling_criterion_basic



#include <source/dexel/dexel_basic.hpp>



//********************************DECLARATIONS***********************************
//------------------------bicotiPixelLabelingCriterion---------------------------
template< class PixelType >
class bicotiPixelLabelingCriterion
{
protected:
  bool direct;

public:
  virtual bool operator() ( PixelType ) = 0;
  bool& Direct() { return direct; };
};



//---------------------bicotiPixelLabelingCriterionSingle------------------------
template< class PixelType >
class bicotiPixelLabelingCriterionSingle : public bicotiPixelLabelingCriterion< PixelType >
{
public:
  virtual bool operator() ( PixelType ) = 0;
};



//--------------------bicotiPixelLabelingCriterionVectorial----------------------
template< class PixelType >
class bicotiPixelLabelingCriterionVectorial : public bicotiPixelLabelingCriterion< PixelType >
{
public:
  virtual bool operator() ( PixelType ) = 0;
};



//-------------------bicotiPixelLabelingCriterionMultipleOr----------------------
template< class PixelType >
class bicotiPixelLabelingCriterionMultipleOr : public bicotiPixelLabelingCriterion< PixelType >
{
private:
  vector< bicotiPixelLabelingCriterion< PixelType > * > list_criterions;
  
public:
  bicotiPixelLabelingCriterionMultipleOr( const vector< bicotiPixelLabelingCriterion< PixelType > * >&, bool = true );
  bool operator() ( PixelType );
};



//--------------------bicotiPixelLabelingCriterionMultipleAnd--------------------
template< class PixelType >
class bicotiPixelLabelingCriterionMultipleAnd : public bicotiPixelLabelingCriterion< PixelType >
{
private:
  vector< bicotiPixelLabelingCriterion< PixelType > * > list_criterions;
  
public:
  bicotiPixelLabelingCriterionMultipleAnd( const vector< bicotiPixelLabelingCriterion< PixelType > * >&, bool = true );
  bool operator() ( PixelType );
};




//********************************IMPLEMENTATIONS***********************************
//---------------------bicotiPixelLabelingCriterionMultipleOr-----------------------
template< class PixelType >
bicotiPixelLabelingCriterionMultipleOr< PixelType > :: bicotiPixelLabelingCriterionMultipleOr( const vector< bicotiPixelLabelingCriterion< PixelType > * >& criterions, bool val_dir )
{
  list_criterions = criterions;
  direct = val_dir;
};


template< class PixelType >
bool bicotiPixelLabelingCriterionMultipleOr< PixelType > :: operator()( PixelType pixel )
{
  bool salida = false;
  unsigned index = 0;
  
  while( !salida && ( index < list_criterions.size() ) )
    {salida = (*( list_criterions[ index ] ))( pixel );
    index++;}

  if( direct )
    return( salida );
  else
    return( !salida );
};



//--------------------bicotiPixelLabelingCriterionMultipleAnd------------------------
template< class PixelType >
bicotiPixelLabelingCriterionMultipleAnd< PixelType > :: bicotiPixelLabelingCriterionMultipleAnd( const vector< bicotiPixelLabelingCriterion< PixelType > * >& criterions, bool val_dir )
{
  list_criterions = criterions;
  direct = val_dir;
};


template< class PixelType >
bool bicotiPixelLabelingCriterionMultipleAnd< PixelType > :: operator()( PixelType pixel )
{
  bool salida = true;
  unsigned index = 0;
  
  while( salida && ( index < list_criterions.size() ) )
    {salida = (*( list_criterions[ index ] ))( pixel );
    index++;}

  if( direct )
    return( salida );
  else
    return( !salida );
};



#endif     
