//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _dexel_region_basic
#define _dexel_region_basic



#include <source/dexel/dexel_basic.hpp>



//********************************DECLARATIONS***********************************
//------------------------------bicotiDexelRegion--------------------------------
template< class CoordType, class PixelType > 
class bicotiDexelRegion : public bicotiDexel< PixelType > 
{
private:
  vector< bicotiPixelOperatorUnary< PixelType > * > operations;
  vector< CoordType > region;
  
public:
  bicotiDexelRegion( bicotiImageImplementation< PixelType > * = NULL );

  unsigned GetPixelsNumber();
  void Append( CoordType );
  CoordType& operator() ( unsigned );
  CoordType& GetCoordinate( unsigned );
  PixelType GetPixel( unsigned );
  vector< CoordType >& GetRegion();
  
  void ModifyPixels( bicotiPixelOperatorUnary< PixelType > * );
  void UpdateDexel(){};
  void UpdateImageImplementation();
}; 




//******************************IMPLEMENTATIONS*********************************
//------------------------------bicotiDexelRegion-------------------------------
template< class CoordType, class PixelType >
bicotiDexelRegion< CoordType, PixelType > :: bicotiDexelRegion( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp = ptr_ima_imp;
};


template< class CoordType, class PixelType > 
void bicotiDexelRegion< CoordType, PixelType > :: ModifyPixels( bicotiPixelOperatorUnary< PixelType > * ptr_pix_oper )
{
  if( ptr_image_imp == NULL )
    throw bicotiException( "bicotiDexelRegion", "ModifyPixels()", 6 );

  operations.push_back( ptr_pix_oper );
};


template< class CoordType, class PixelType > 
inline unsigned bicotiDexelRegion< CoordType, PixelType > :: GetPixelsNumber()
{
  return( region.size() );
};


template< class CoordType, class PixelType > 
inline void bicotiDexelRegion< CoordType, PixelType > :: Append( CoordType coord )
{
  region.push_back( coord );
};


template< class CoordType, class PixelType > 
inline CoordType& bicotiDexelRegion< CoordType, PixelType > :: operator() ( unsigned index )
{
  if( index < region.size() )
    return( region[ index ] );
  else
     throw bicotiException( "bicotiDexelRegion", "operator()", 1 );
};


template< class CoordType, class PixelType > 
inline CoordType& bicotiDexelRegion< CoordType, PixelType > :: GetCoordinate( unsigned index )
{
  if( index < region.size() )
    return( region[ index ] );
  else
     throw bicotiException( "bicotiDexelRegion", "GetCoordinate()", 1 );
};


template< class CoordType, class PixelType > 
inline PixelType bicotiDexelRegion< CoordType, PixelType > :: GetPixel( unsigned index )
{
  if( ptr_image_imp == NULL )
    throw bicotiException( "bicotiDexelRegion", "GetPixel()", 6 );

  return( ptr_image_imp->GetPixel( region[ index ] ) );
};


template< class CoordType, class PixelType > 
inline vector< CoordType >& bicotiDexelRegion< CoordType, PixelType > :: GetRegion()
{
  return( region );
};


template< class CoordType, class PixelType > 
void bicotiDexelRegion< CoordType, PixelType > :: UpdateImageImplementation()
{
  if( ptr_image_imp == NULL )
    throw bicotiException( "bicotiDexelRegion", "UpdateImageImplementation()", 6 );

  unsigned index_fun, index_coord;
  
  for( index_fun = 0; index_fun < operations.size(); index_fun++ )
    for( index_coord = 0; index_coord < region.size(); index_coord++ )
      ptr_image_imp->SetPixel( region[ index_coord ], ( *( operations[ index_fun ] ) )( ptr_image_imp->GetPixel( region[ index_coord ] ) ) );
  
  while( operations.size() != 0 )
    operations.pop_back();
};



#endif
