//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _dexel_statistical_basic
#define _dexel_statistical_basic


#include <source/dexel/dexel_basic.hpp>


//**************************DECLARATIONS****************************
//---------------------bicotiDexelStatistical-----------------------
template< class CoordType, class ColorType, class PixelType = ColorType >
class bicotiDexelStatistical : public bicotiDexel< PixelType >
{
private:
  ColorType max_value;
  ColorType min_value;

  vector< CoordType > list_maxs;
  vector< CoordType > list_mins;

  bicotiPixelAttributeSelector< ColorType, PixelType > * ptr_pixel_attribute_selector;

  void CreateMe();

public:
  bicotiDexelStatistical( bicotiImageImplementation< PixelType > *, bicotiPixelAttributeSelector< ColorType, PixelType > *);

  ColorType& MaximumValue();
  ColorType& MinimumValue();

  void UpdateImageImplementation();
  void UpdateDexel(){};//HACER
};




//*************************IMPLEMETATIONS***************************
//---------------------bicotiDexelStatistical-----------------------
template< class CoordType, class ColorType, class PixelType >
bicotiDexelStatistical< CoordType, ColorType, PixelType > :: bicotiDexelStatistical( bicotiImageImplementation< PixelType > * ptr_ima_imp, bicotiPixelAttributeSelector< ColorType, PixelType > * ptr_color_selector )
{
  ptr_image_imp = ptr_ima_imp;
  ptr_pixel_attribute_selector = ptr_color_selector;

  CreateMe();
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelStatistical< CoordType, ColorType, PixelType > :: CreateMe()
{
  bicotiImageIterator< PixelType > * ptr_iter;
  
  ptr_iter = ptr_image_imp->CreateIterator();
  ptr_iter->First();
  max_value = ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) );
  min_value = ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) );
  for( ; !ptr_iter->IsDone(); (*ptr_iter)++ )
    {if ( ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) ) > max_value )
      max_value = ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) );
    if (  ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) ) < min_value )
      min_value =  ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) );}
  
  for( ptr_iter->First(); !ptr_iter->IsDone(); (*ptr_iter)++ )
    {if ( ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) ) == max_value )
      list_maxs.push_back( (CoordType)( ptr_iter->GetCurrentCoordinateInteger() ) );
    if ( ( *ptr_pixel_attribute_selector )( *( ptr_iter->GetCurrentPixel() ) ) == min_value )
      list_mins.push_back( (CoordType)( ptr_iter->GetCurrentCoordinateInteger() ) );}
  
  delete ptr_iter;
};


template< class CoordType, class ColorType, class PixelType >
void bicotiDexelStatistical< CoordType, ColorType, PixelType >::UpdateImageImplementation()
{
  unsigned index;

  for( index = 0; index < list_maxs.size(); index++ )
    {ptr_image_imp->SetPixel( list_maxs[ index ], ptr_pixel_attribute_selector->Inverse( ptr_image_imp->GetPixel( list_maxs[ index ] ), max_value) );}
  
  for( index = 0; index < list_mins.size(); index++ )
    {ptr_image_imp->SetPixel( list_mins[ index ], ptr_pixel_attribute_selector->Inverse( ptr_image_imp->GetPixel( list_mins[ index ] ), min_value) );}

  while( list_maxs.size() > 0 )
    list_maxs.pop_back();

  while( list_mins.size() > 0 )
    list_mins.pop_back();

  CreateMe();
};


template< class CoordType, class ColorType, class PixelType >
ColorType& bicotiDexelStatistical< CoordType, ColorType, PixelType > :: MaximumValue()
{
  return ( max_value );
};


template< class CoordType, class ColorType, class PixelType >
ColorType& bicotiDexelStatistical< CoordType, ColorType, PixelType > :: MinimumValue()
{
  return ( min_value );
};   

#endif
