#ifndef _bicoti_event_loop_manager_qt
#define _bicoti_event_loop_manager_qt


#include <source/event_loop_manager/event_loop_manager.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_push_button.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_panel.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_image.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_label.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_field.hpp>
#include <source/event_loop_manager/qt/homologous_qt_timer.hpp>
#include <source/event_loop_manager/qt/homologous_qt_menu_bar.hpp>
#include <source/event_loop_manager/qt/homologous_qt_menu_popup.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_file_dialog.hpp>
#include <source/event_loop_manager/qt/homologous_qt_2D_scroll_bar.hpp>
#include <qapplication.h>
#include <qwidget.h>


//---------------bicotiEventLoopManagerQT---------------

/** Implementacin QT de bicotiEventLoopManager */
class bicotiEventLoopManagerQT : public bicotiEventLoopManager
{
private:
  QApplication * ptr_app;

public:
  
  /**
   * Constructor
   *
   *@param argv cantidad de argumentos pasados por el sistema
   *@param argc cadena de argumentos pasados por el sistema
   */
  bicotiEventLoopManagerQT( int argv, char ** argc );
  
  /** Destructor */
  ~bicotiEventLoopManagerQT();
  
  void Start();
  void Stop();
};



//---------------bicotiHomologousFactoryQT---------------

/** 
 * Implementacin QT del homologous Factory
 *
 * Crea homologos QT para los actores bicoti
 */
template< class PixelType, class PixelConverter >
class bicotiHomologousFactoryQT
{
private:

  /**
   * Crea un widget adecuado para un actor
   *
   *@param ptr_actor actor bicoti
   *@param ptr_widg widget padre del nuevo widget
   *@exception bicotiException si el actor no es soportado pr QT
   */
  void TranslateToQT( bicotiActor * ptr_actor, QWidget * ptr_widg = NULL );

public:
  bicotiHomologousFactoryQT();
  ~bicotiHomologousFactoryQT();

  
  /** 
   * Crea un homlogo adecuado para un actor
   *
   *@param ptr_actor actor bicoti
   *@exception bicotiException si el actor es NULL
   */
  void CreateHomologous( bicotiActor * );
  
  /** 
   * Crea un homlogo adecuado para un actor y para sus hijos
   *
   *@param ptr_actor actor bicoti
   */
  void RecursiveCreateHomologous( bicotiActor * );
};



//---------------bicotiHomologousFactoryQT---------------
template< class PixelType, class PixelConverter >
bicotiHomologousFactoryQT< PixelType, PixelConverter > :: bicotiHomologousFactoryQT()
{}


template< class PixelType, class PixelConverter >
bicotiHomologousFactoryQT< PixelType, PixelConverter > :: ~bicotiHomologousFactoryQT()
{}


template< class PixelType, class PixelConverter >
void bicotiHomologousFactoryQT< PixelType, PixelConverter > :: TranslateToQT( bicotiActor * ptr_actor, QWidget * ptr_widg )
{
  int i;
  bicotiHomologous * ptr_homo;
  bicotiActorDescriptor act_desc = ptr_actor->GetActorDescriptor();
  switch( act_desc.GetActorCase() ) 
    {case actor_image:
      {bicotiQT2DImage< PixelType, PixelConverter > * ptr_bico2D = new bicotiQT2DImage< PixelType, PixelConverter >( ptr_widg );
      ptr_bico2D->SetActor( ptr_actor );
      ptr_bico2D->EnableAlphaChannel( ((bicotiActorGraphic2DImage< PixelType >*)ptr_actor)->IsAlphaChannelEnabled() );// No va con lo anter
      ptr_actor->SetHomologous( ptr_bico2D );
      ptr_homo = ptr_bico2D;
      break;}
      
    case actor_push_button:
      {bicotiQT2DPushButton * ptr_push_button = new bicotiQT2DPushButton( ptr_widg );
      ptr_push_button->SetActor( ptr_actor );
      ptr_push_button->SetFont( ((bicotiActorGraphic*)ptr_actor)->GetFont() );
      ptr_push_button->SetText( ((bicotiActorGraphic2DPushButton*)ptr_actor)->GetText() );
      ptr_actor->SetHomologous( ptr_push_button );
      ptr_homo = ptr_push_button;
      break;}

    case actor_panel:
      {bicotiQT2DPanel * ptr_panel = new bicotiQT2DPanel( ptr_widg );
      ptr_panel->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_panel );
      ptr_homo = ptr_panel;
      break;}

     case actor_label:
      {bicotiQT2DLabel * ptr_label = new bicotiQT2DLabel( ptr_widg );
      ptr_label->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_label );
      ptr_label->SetFrame( ((bicotiActorGraphic2DLabel*)ptr_actor)->GetFrame() );
      ptr_homo = ptr_label;
      break;}

    case actor_field:
      {bicotiQT2DField * ptr_field = new bicotiQT2DField( ptr_widg );
      ptr_field->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_field );
      //ptr_field->SetParameters();
      ptr_field->SetText( ((bicotiActorGraphic2DField*)ptr_actor)->GetText() );
      ptr_homo = ptr_field;
      break;}

    case actor_scroll_bar:
      {bicotiQT2DScrollBar * ptr_scrbar = new bicotiQT2DScrollBar( ptr_widg );
      ptr_scrbar->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_scrbar );
      ptr_scrbar->SetRange( ((bicotiActorGraphic2DScrollBar*)ptr_actor)->GetMinimum(), ((bicotiActorGraphic2DScrollBar*)ptr_actor)->GetMaximum() );
      ptr_scrbar->SetLineStep( ((bicotiActorGraphic2DScrollBar*)ptr_actor)->GetLineStep() );
      ptr_scrbar->SetPageStep( ((bicotiActorGraphic2DScrollBar*)ptr_actor)->GetPageStep() );
      ptr_scrbar->SetValue( ((bicotiActorGraphic2DScrollBar*)ptr_actor)->GetValue() );
      ptr_scrbar->SetVertical( ((bicotiActorGraphic2DScrollBar*)ptr_actor)->GetVertical() );
      ptr_homo = ptr_scrbar;
      break;}

    case actor_file_dialog:
      {bicotiQT2DFileDialog * ptr_file_dialog = new bicotiQT2DFileDialog( ptr_widg );
      ptr_file_dialog->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_file_dialog );
      ptr_file_dialog->SetDirectory( ((bicotiActorGraphic2DFileDialog*)ptr_actor)->GetDirectory() );
      ptr_file_dialog->SetFilter( ((bicotiActorGraphic2DFileDialog*)ptr_actor)->GetFilter() );
      ptr_file_dialog->SetExistingFile( ((bicotiActorGraphic2DFileDialog*)ptr_actor)->GetExistingFile() );
      ptr_homo = ptr_file_dialog;
      break;}

    case actor_menu_bar:
      {bicotiQTMenuBar * ptr_mbr = new bicotiQTMenuBar( ptr_widg );
      bicotiActorGraphicMenuBar * ptr_actor_mbr = ((bicotiActorGraphicMenuBar*)ptr_actor);
      ptr_mbr->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_mbr );
      ptr_homo = ptr_mbr;
      break;}
      
    case actor_menu_popup:
      {bicotiQTMenuPopup * ptr_mbr = new bicotiQTMenuPopup( ptr_widg );
      bicotiActorGraphicMenuPopup * ptr_actor_mbr = ((bicotiActorGraphicMenuPopup*)ptr_actor);
      ptr_mbr->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_mbr );
      ptr_homo = ptr_mbr;
      break;}
      
    case actor_timer:
      {bicotiQTTimer * ptr_timer = new bicotiQTTimer;
      ptr_timer->SetActor( ptr_actor );
      ptr_actor->SetHomologous( ptr_timer );
      ptr_homo = ptr_timer;
      break;}
  
    default:
      { throw bicotiException( "bicotiHomologousFactoryQT", "TranslateToQT" , 10 );}}

  if( act_desc.GetActorType() == _timer )
    {bicotiHomologousTimer * ptr_homo_timer = (bicotiHomologousTimer*)ptr_homo;
    bicotiActorTimer * ptr_actor_timer = (bicotiActorTimer*)ptr_actor;
    ptr_homo_timer->SetPeriod( ptr_actor_timer->GetPeriod() );}
  
  if( ( act_desc.GetActorType() == _graphic ) && ( act_desc.GetActorDimension() == _2D ) )
    {bicotiHomologousGraphic2D * ptr_homo_2D = (bicotiHomologousGraphic2D*)ptr_homo;
    bicotiActorGraphic2D * ptr_actor_2D = (bicotiActorGraphic2D*)ptr_actor;
    ptr_homo_2D->SetMinimumWidth( ptr_actor_2D->GetMinimumWidth() );
    ptr_homo_2D->SetMaximumWidth( ptr_actor_2D->GetMaximumWidth() );
    ptr_homo_2D->SetMinimumHeight( ptr_actor_2D->GetMinimumHeight() );
    ptr_homo_2D->SetMaximumHeight( ptr_actor_2D->GetMaximumHeight() );
    ptr_homo_2D->SetSize( ptr_actor_2D->GetWidth(), ptr_actor_2D->GetHeight() );
    ptr_homo_2D->SetPosition( ptr_actor_2D->GetX(), ptr_actor_2D->GetY() );}

  if( ( act_desc.GetActorType() == _graphic ) && ( act_desc.GetActorDimension() == _menu ) )
    {bicotiHomologousGraphicMenu * ptr_homo_menu = (bicotiHomologousGraphicMenu*)ptr_homo;
    bicotiActorGraphicMenu * ptr_actor_menu = (bicotiActorGraphicMenu*)ptr_actor;

    for( int i = 0; i < ptr_actor_menu->GetElementsNumber(); i++ )
      {if( ptr_actor_menu->CheckItem( i ) )
	{if( ptr_actor_menu->IsItem( i ) )
	  {if( ptr_actor_menu->GetPopupForItem( i ) == NULL )
	    ptr_homo_menu->AddItem( i, ptr_actor_menu->GetItem( i ) );
	  else
	    ptr_homo_menu->AddItem( i, ptr_actor_menu->GetItem( i ), ptr_actor_menu->GetPopupForItem( i ) );}
	else
	  ptr_homo_menu->AddSeparator();}}}
  
  if( act_desc.GetActorType() == _graphic )
    {bicotiActorGraphic * ptr_actor_graphic = (bicotiActorGraphic *)ptr_actor;
    bicotiHomologousGraphic * ptr_homo_graphic = (bicotiHomologousGraphic *)ptr_homo;
    
    ptr_homo_graphic->SetBackgroundColor( ptr_actor_graphic->GetBackgroundColor() );
    ptr_homo_graphic->SetForegroundColor( ptr_actor_graphic->GetForegroundColor() );
    
    if( act_desc.GetActorCase() == actor_image ) ((bicotiQT2DImage< PixelType, PixelConverter >*)ptr_homo_graphic)->RefreshImage();
    
    if( ptr_actor_graphic->IsTrackingOn() )
      ptr_homo_graphic->ActivateMouseMotionTracking();
    else
      ptr_homo_graphic->DeactivateMouseMotionTracking();
    
    ptr_homo_graphic->SetFont( ptr_actor_graphic->GetFont() );
    ptr_homo_graphic->SetVisible( ptr_actor_graphic->GetVisible() );}
}


template< class PixelType, class PixelConverter >
void bicotiHomologousFactoryQT< PixelType, PixelConverter > :: CreateHomologous( bicotiActor * ptr_actor )
{
  QWidget * ptr_widget = NULL;
  
  if( ptr_actor == NULL )
    throw bicotiException( "bicotiHomologousFactoryQT", "CreateHomologous", 6 ) ; 
  else 
    {if( ptr_actor->GetHomologous() == NULL )
      {if( ptr_actor->GetActorDescriptor().GetActorType() == _graphic )
	{bicotiActorGraphic * ptr_actor_graphic = ((bicotiActorGraphic*)ptr_actor);
	if( ptr_actor_graphic->GetParent() != NULL  ) // TIENE PADRE
	  {if( ptr_actor_graphic->GetParent()->GetHomologous() == NULL  )
	    TranslateToQT( ptr_actor ); // TIENE PADRE sin HOMO
	  else
	    {
	      ((bicotiHomologousGraphic*)(ptr_actor_graphic->GetParent()->GetHomologous()))->LoadConcreteInstance();
	      ptr_widget = bicotiHomologousQTCaster :: ptr_concrete_instance;
	      TranslateToQT( ptr_actor, ptr_widget );}}
	else
	  TranslateToQT( ptr_actor ); // NO PADRE
	    
	for( int index = 0; index < ptr_actor_graphic->CountProgenies(); index ++ ) // PARA LOS HIJOS
	  if( ptr_actor_graphic->GetProgeny( index )->GetHomologous() != NULL )
	    ((bicotiHomologousGraphic*)(ptr_actor_graphic->GetHomologous()))->SetChild( ((bicotiHomologousGraphic*)(ptr_actor_graphic->GetProgeny( index )->GetHomologous())) );}
      else
	TranslateToQT( ptr_actor );}}
}


template< class PixelType, class PixelConverter >
void bicotiHomologousFactoryQT< PixelType, PixelConverter > :: RecursiveCreateHomologous( bicotiActor * ptr_actor )
{
  CreateHomologous( ptr_actor );
  
  if( (ptr_actor->GetActorDescriptor()).GetActorType() == _graphic )
    {for( int count = 0; count < ((bicotiActorGraphic*)ptr_actor)->CountProgenies(); count++ )
      RecursiveCreateHomologous( ((bicotiActorGraphic*)ptr_actor)->GetProgeny(count) );}
}



#endif
