//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _frame_2D_external
#define _frame_2D_external


#include <source/frame/frame_2D.hpp>

  

//*************************DECALARATIONS****************************
//----------------------bicotiFrame2DExternal-----------------------
template< class PixelType >  
class bicotiFrame2DExternal : public bicotiFrame2D< PixelType >  
{  
private:  
  bicotiMatrix2D< PixelType > * ptr_matrix;  
  int first_x1, first_x2;  
  
public:  
  bicotiFrame2DExternal( bicotiMatrix2D< PixelType > *, int );  
  bicotiFrame2DExternal( const bicotiFrame2DExternal< PixelType >& );  
  
  void First();  
  void Next();  
    
  void Locate(int, int);  
  
  void operator=( const bicotiFrame2DExternal< PixelType >& );  
  PixelType& operator()(int,int);  
};  
  
  
  
//************************IMPLEMENTATIONS***************************
//---------------------bicotiFrame2DExternal------------------------
template<class  PixelType >  
bicotiFrame2DExternal< PixelType > :: bicotiFrame2DExternal( bicotiMatrix2D< PixelType > * ptr_mat, int size )  
{  
  ptr_matrix = ptr_mat;  
  frame_size = size;  
   
  if ( size < 0 )  
    throw bicotiException( "bicotiFrame2DExternal", "Constructor", 4 ); 
   
  if ( (size > ptr_matrix->GetX0Size()) || (size > ptr_matrix->GetX1Size()) )   
    throw bicotiException( "bicotiFrame2DExternal", "Constructor", 4 ); 
   
  ptr_current = (ptr_matrix->ptr_data)[ 0 ];  
  current_x1 = current_x2 = first_x1 = first_x2 = 0;  
  is_done = false;  
};  
 
 
template< class PixelType >  
bicotiFrame2DExternal< PixelType > :: bicotiFrame2DExternal( const bicotiFrame2DExternal< PixelType >& frame_src )  
{  
  frame_size=frame_src.frame_size;  
  ptr_matrix = frame_src.ptr_matrix;  
  current_x1 = frame_src.current_x1;  
  current_x2 = frame_src.current_x2;  
  first_x1 = frame_src.first_x1;  
  first_x2 = frame_src.first_x2;  
  ptr_current = frame_src.ptr_current;  
  is_done = frame_src.is_done;  
};  
  
  
template< class PixelType >  
void bicotiFrame2DExternal< PixelType > :: First()  
{  
  ptr_current = (ptr_matrix->ptr_data)[ first_x2 ] + first_x1;  
  current_x1 = 0;  
  current_x2 = 0;  
  is_done = false;  
};  
  
  
template< class PixelType >  
inline void bicotiFrame2DExternal< PixelType > :: Next()  
{  
  if ( current_x1 < (frame_size - 1) )  
    {ptr_current++;  
    current_x1++;}  
  else  
    {if (current_x2 < (frame_size - 1) )  
      {current_x2++;  
      current_x1 = 0;  
      ptr_current = (ptr_matrix->ptr_data)[ first_x2 + current_x2 ] + first_x1;}  
    else  
      is_done = true;}  
};  
  
  
template< class PixelType >  
inline void bicotiFrame2DExternal< PixelType > :: Locate( int x1, int x2 )  
{  
#ifdef TOTAL_CHECK 
  if ( (x1 < 0) || (x2 < 0) || ((x1 + frame_size) > ptr_matrix->GetX0Size()) || ((x2 + frame_size) > ptr_matrix->GetX1Size()) )  
    throw bicotiException( "bicotiFrame2DExternal", "Locate", 1 ); 
#endif 
   
  first_x1 = x1;  
  first_x2 = x2;  
};  
 
 
template< class PixelType >  
void bicotiFrame2DExternal< PixelType > :: operator=( const bicotiFrame2DExternal< PixelType >& frame_src ) 
{  
  frame_size=frame_src.frame_size;  
  ptr_matrix = frame_src.ptr_matrix;  
  current_x1 = frame_src.current_x1;  
  current_x2 = frame_src.current_x2;  
  first_x1 = frame_src.first_x1;  
  first_x2 = frame_src.first_x2;  
  ptr_current = frame_src.ptr_current;  
  is_done = frame_src.is_done;  
};  
  
  
template< class PixelType >  
inline PixelType& bicotiFrame2DExternal< PixelType > :: operator()( int x1, int x2 )  
{  
#ifdef TOTAL_CHECK 
  if ( ( x1 > (frame_size-1) ) || ( x2 > (frame_size-1) ) || ( x1 < 0 ) || ( x2 <0 ) )  
    throw bicotiException( "bicotiFrame2DExternal", "operator()", 1 ); 
#endif 
   
  return( (ptr_matrix->ptr_data)[ first_x2 + x2 ][ first_x1 + x1 ] );  
};                    


#endif
