//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _affim_transformation
#define _affim_transformation


#include <source/geometrical/geometrical_basic.hpp>



//***************************************DECLARATIONS******************************************
//---------------------------------bicotiAffimTransformation-----------------------------------
template< class PixelType, class CoordType > 
class bicotiAffimTransformation 
{ 
private: 
  bicotiImageImplementation< PixelType > * ptr_image_source; 
  bicotiImageImplementation< PixelType > * ptr_image_dest;
  
  bicotiMatrix2D< double > affim_matrix;
  CoordType affim_coord;
  int dimension; 
  void CheckParameters();
  
public:   
  bicotiAffimTransformation( bicotiImageImplementation< PixelType > *, bicotiImageImplementation< PixelType > *, bicotiMatrix2D< double >&, CoordType& );
  bicotiAffimTransformation();
  ~bicotiAffimTransformation(){};
  
  void SetImagesImplementations( bicotiImageImplementation< PixelType > *, bicotiImageImplementation< PixelType > * );
  bicotiMatrix2D< double >& Matrix();
  CoordType& Vector();
  
  void operator()( bicotiImagePixelInterpolator< PixelType > * );
}; 



//*************************************IMPLEMENTATIONS*****************************************
//---------------------------------bicotiAffimTransformation-----------------------------------
template< class PixelType, class CoordType > 
bicotiAffimTransformation< PixelType, CoordType > :: bicotiAffimTransformation( bicotiImageImplementation< PixelType > * ptr_dest, bicotiImageImplementation< PixelType > * ptr_source, bicotiMatrix2D< double >& matrix, CoordType& coord)
{ 
  ptr_image_source = ptr_source; 
  ptr_image_dest = ptr_dest; 
  affim_matrix = matrix;
  dimension = coord.GetDimension();
  affim_coord = coord;
};

 
template< class PixelType, class CoordType > 
bicotiAffimTransformation< PixelType, CoordType > :: bicotiAffimTransformation()
{ 
  ptr_image_source = NULL; 
  ptr_image_dest = NULL; 
  affim_coord = CoordType();
  dimension = affim_coord.GetDimension();

  affim_matrix =bicotiMatrix2D< double >( 0, dimension, dimension );
};

 
template< class PixelType, class CoordType > 
void bicotiAffimTransformation< PixelType, CoordType > :: SetImagesImplementations( bicotiImageImplementation< PixelType > * ptr_ima_dest, bicotiImageImplementation< PixelType > * ptr_ima_src )
{
 ptr_image_source = ptr_ima_src;
 ptr_image_dest = ptr_ima_dest;
};


template< class PixelType, class CoordType > 
bicotiMatrix2D< double >& bicotiAffimTransformation< PixelType, CoordType > :: Matrix()
{
  return( affim_matrix );
};


template< class PixelType, class CoordType > 
CoordType& bicotiAffimTransformation< PixelType, CoordType > :: Vector()
{
  return( affim_coord );
};

 
template< class PixelType, class CoordType > 
void bicotiAffimTransformation< PixelType, CoordType > :: operator() ( bicotiImagePixelInterpolator< PixelType > * ptr_interpolator_criterion ) 
{ 
  CheckParameters();
 
  bicotiMatrix2D< double > affim_matrix_inverse( 0 );
  affim_matrix_inverse = affim_matrix.Inverse( 0 );

  bicotiImageIterator< PixelType > * ptr_iter_image_dest = ptr_image_dest->CreateIterator();
  CoordType aux_coord;
  
  while( !ptr_iter_image_dest->IsDone() ) 
    {aux_coord = (CoordType)( ptr_iter_image_dest->GetCurrentCoordinateReal() );
    aux_coord -= affim_coord;
    aux_coord = affim_matrix_inverse * aux_coord;
    ( **ptr_iter_image_dest ) = ( *ptr_interpolator_criterion )( aux_coord ) ;
    ( *ptr_iter_image_dest )++;} 
  
  delete ptr_iter_image_dest;
};

           
template< class PixelType , class CoordType >
void  bicotiAffimTransformation< PixelType , CoordType > :: CheckParameters()
{
   if( ( ptr_image_source != NULL ) && ( ptr_image_dest != NULL ) ) 
     {bicotiImageDescriptor *ptr_descript_image_source, *ptr_descript_image_dest;
     ptr_descript_image_source = ptr_image_source->CreateImageDescriptor();
     ptr_descript_image_dest = ptr_image_dest->CreateImageDescriptor();
     CoordType dummy;

     if ( ( dummy.GetDimension() != ptr_descript_image_source->GetDimension() ) || 
	  ( dummy.GetDimension() != ptr_descript_image_dest->GetDimension() ) || 
	  ( affim_coord.GetDimension() != dummy.GetDimension() ) ||
	  ( affim_matrix.GetX0Size() != dummy.GetDimension() ) || ( affim_matrix.GetX1Size() != dummy.GetDimension() ) )
       {delete ptr_descript_image_dest;
       delete ptr_descript_image_source;
       throw bicotiException( "bicotiAffimTransformation", "CheckParameters()", 2 );}    

     delete ptr_descript_image_dest;
     delete ptr_descript_image_source;}
   else
     throw bicotiException( "bicotiAffimTransformation", "CheckParameters()", 6 );
}


#endif
