//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _geometrical
#define _geometrical


#include <source/geometrical/affim_transformation.hpp>
#include <source/geometrical/cutter.hpp>


template< class CoordType >
class bicotiCoordinateNormCriterion
{
public:
  bool operator()( CoordType, CoordType );
};

		
template< class CoordType >
bool bicotiCoordinateNormCriterion< CoordType > :: operator() ( CoordType coord_1, CoordType coord_2)
{
  return( coord_1.Norm() <= coord_2.Norm() );
}


//****************************************DECLARATIONS*************************************
//----------------------------------bicotiGeometricalFacade--------------------------------
template< class CoordType, class PixelType >
class bicotiGeometricalFacade
{
protected: 
  bicotiImageImplementation< PixelType > * ptr_image_imp_src;
  bicotiImageImplementation< PixelType > * ptr_image_imp_dest;
  
  bicotiImagePixelInterpolator< PixelType > * ptr_interpolator_criterion;
  
  bicotiMatrix2D< double > affim_matrix;	 
  CoordType affim_coord;
  CoordType rotation_center;
  bool in_rotation;

  bicotiImageDescriptor * SizeImaDest(); 
  
  
public:
  virtual ~bicotiGeometricalFacade(){};
  
  virtual void BuildAffimTransformation() = 0;
  virtual void BuildRotation( double ) = 0;
  
  virtual void ApplyAffimTransformation( bool ) = 0;
  
  virtual void UsePixelInterpolatorLinear() = 0;
  virtual void UsePixelInterpolatorLagrange() = 0;
  
  bicotiMatrix2D< double >& Matrix();
  CoordType& Vector();
  CoordType& RotationCenter();
  
  bicotiImageImplementation< PixelType > * GetResult();
  bicotiImageImplementation< PixelType > * LoadResult();
};



//----------------------------------bicotiGeometricalFacade2D------------------------------
template< class PixelType, class PrecisionType = PixelType >
class bicotiGeometricalFacade2D : public bicotiGeometricalFacade< bicotiCoordinate2D< REAL >, PixelType >
{
  
public:
  bicotiGeometricalFacade2D( bicotiImageImplementation2D< PixelType >* );
  ~bicotiGeometricalFacade2D();	
    
  void BuildAffimTransformation();
  void BuildRotation( double );
  
  void ApplyAffimTransformation( bool = false );

  void UsePixelInterpolatorLinear();
  void UsePixelInterpolatorLagrange();
  
};



//----------------------------------bicotiGeometricalFacade3D------------------------------
template< class PixelType, class PrecisionType = PixelType >
class bicotiGeometricalFacade3D : public bicotiGeometricalFacade< bicotiCoordinate3D< REAL >, PixelType >
{
private:
  bicotiCoordinate3D< REAL > rotation_axis;
  int size_x0, size_x1, size_x2;
  bicotiCoordinate3D< REAL > origin, vector_I, vector_J;

  bool PointsInSameFace( bicotiCoordinate3D< REAL > &, bicotiCoordinate3D< REAL >& );
  bool PlaneForEdge( bicotiCoordinate3D< REAL >, bicotiCoordinate3D< REAL > );
  bool InImage( bicotiCoordinate3D< REAL > &);
  bool ParallelPrincipalsPlanes( bicotiCoordinate3D< REAL >& , bicotiCoordinate3D< REAL >& , bicotiCoordinate3D< REAL >& );
  bool EqualPoint( bicotiCoordinate3D< REAL >& , bicotiCoordinate3D< REAL >& );
  vector< bicotiCoordinate3D< REAL > > ComputePlaneEdgeCut( bicotiCoordinate3D< REAL >, bicotiCoordinate3D< REAL > );

public:
  bicotiGeometricalFacade3D( bicotiImageImplementation3D< PixelType >* );
  ~bicotiGeometricalFacade3D();	
    
  void BuildAffimTransformation();
  void BuildRotation( double );
  void BuildCutter( bicotiCoordinate3D< REAL >, bicotiCoordinate3D< REAL >, bicotiCoordinate3D< REAL >, bicotiImageImplementation2D< PixelType >* );

  void ApplyAffimTransformation( bool = false );

  void UsePixelInterpolatorLinear();
  void UsePixelInterpolatorLagrange();

  bicotiCoordinate3D< REAL > GetOriginSubspace();
  bicotiCoordinate3D< REAL > GetVectorISubspace();
  bicotiCoordinate3D< REAL > GetVectorJSubspace();
   
  bicotiCoordinate3D< REAL >& RotationAxis();
};



//**************************************IMPLEMENTATIONS************************************
//----------------------------------bicotiGeometricalFacade--------------------------------
template< class CoordType, class PixelType >
bicotiImageDescriptor* bicotiGeometricalFacade< CoordType, PixelType > :: SizeImaDest()
{
  unsigned i, j;
  unsigned aux;

  bicotiImageDescriptor * ptr_desc;
  CoordType coord_aux; 
  vector< CoordType > list_vertices;
  vector< CoordType > :: iterator first;
  vector< CoordType > :: iterator last;

  ptr_desc = ptr_image_imp_src->CreateImageDescriptor();
  for( i = 0; i < (unsigned)pow( 2, ptr_desc->GetDimension() ); i++ )
    list_vertices.push_back( coord_aux );
    
  for( i = 0; i < ptr_desc->GetDimension(); i++ )
    {for( j = 0; j < ( unsigned )pow( 2, ptr_desc->GetDimension() ); j++ )
      {aux = j % (unsigned)pow( 2, ptr_desc->GetDimension() - i );
	if( aux < (unsigned)pow( 2, ptr_desc->GetDimension() - i - 1 ) )
	  list_vertices[ j ]( i ) = 0;
	else
	  list_vertices[ j ]( i ) = ptr_desc->Size( i ) - 1;}}

  for( i = 0; i < (unsigned)pow( 2, ptr_desc->GetDimension() ); i++ )
    list_vertices[ i ] = affim_matrix * list_vertices[ i ] + affim_coord;
  
  first = list_vertices.begin();
  last = list_vertices.end();
  CoordinateCompareCriterion< CoordType > compar;
    
  for( i = 0; i < ptr_desc->GetDimension(); i++ )
    {compar.component = i;
     first = list_vertices.begin();
     last = list_vertices.end();
     sort( first, last, compar );
     if ( in_rotation )
     ptr_desc->Size( i ) = (unsigned)( list_vertices.back()( i ) - list_vertices[ 0 ]( i ) ) + 1;
    else     
      ptr_desc->Size( i ) = (unsigned)( list_vertices.back()( i ) /*- list_vertices[ 0 ]( i )*/ ) + 1  ;}

    
  return( ptr_desc );
};


template< class CoordType, class PixelType >
CoordType& bicotiGeometricalFacade< CoordType, PixelType > :: Vector()
{ 
    return ( affim_coord );		
};


template< class CoordType, class PixelType >
bicotiMatrix2D< double >& bicotiGeometricalFacade< CoordType, PixelType > :: Matrix()
{
    return ( affim_matrix );		
};


template< class CoordType, class PixelType >
bicotiImageImplementation< PixelType > * bicotiGeometricalFacade< CoordType, PixelType > :: GetResult()
{
  if( ptr_image_imp_dest == NULL )
    throw bicotiException( "bicotiGeometricalFacade", "GetResult()", 5 );
  
  return ( ptr_image_imp_dest );	
};


template< class CoordType, class PixelType >
bicotiImageImplementation< PixelType > * bicotiGeometricalFacade< CoordType, PixelType > :: LoadResult()
{
  if( ptr_image_imp_dest == NULL )
    throw bicotiException( "bicotiGeometricalFacade", "LoadResult()", 5 );

  ptr_image_imp_src = ptr_image_imp_dest;
  ptr_image_imp_dest = NULL;
  
  return( ptr_image_imp_src );
};


template< class CoordType, class PixelType >
CoordType& bicotiGeometricalFacade< CoordType, PixelType > :: RotationCenter()
{
  return( rotation_center );
};

//----------------------------------bicotiGeometricalFacade2D------------------------------
template< class PixelType, class PrecisionType >
bicotiGeometricalFacade2D< PixelType, PrecisionType > :: bicotiGeometricalFacade2D( bicotiImageImplementation2D< PixelType > * ptr_ima_imp )
{
  ptr_image_imp_src = ptr_ima_imp;
  ptr_image_imp_dest = NULL;
  
  ptr_interpolator_criterion = NULL;	
};


template< class PixelType, class PrecisionType >
bicotiGeometricalFacade2D< PixelType, PrecisionType > :: ~bicotiGeometricalFacade2D()
{    
  
  if ( ptr_image_imp_dest != NULL )
    delete ptr_image_imp_dest;

  if ( ptr_interpolator_criterion != NULL )
    delete ptr_interpolator_criterion;
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade2D< PixelType, PrecisionType > :: BuildAffimTransformation()
{ 
  bicotiMatrix2D< double > identity( 1 , 2 , 2 );
  identity(0,1) = 0;
  identity(1,0) = 0;
  affim_matrix = identity;//Asi se con que quedo inicializado    	    
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade2D< PixelType, PrecisionType > :: BuildRotation( double angle )
{ 
  if ( ( angle < -pi ) || ( angle > pi ) ) 
    throw bicotiException( "bicotiGeometricalFacade2D", "BuildRotation()", 4 );

  bicotiImageDescriptor *  ptr_descriptor;
  bicotiCoordinate2D< REAL > center_src, center_dest, correction;;
  center_src.X0() = ( ( (bicotiImageImplementation2D< PixelType >*)ptr_image_imp_src)->GetX0Size() ) / 2;
  center_src.X1() = ( ( (bicotiImageImplementation2D< PixelType >*)ptr_image_imp_src)->GetX1Size() ) / 2;

  bicotiMatrix2D< double > matrix_aux(0,2,2);//Hay que hacer esto porque no se inicializo el tama#o
  affim_matrix = matrix_aux;

  affim_matrix( 0, 0 ) = cos( angle );
  affim_matrix( 0, 1 ) = -1*sin( angle );
  affim_matrix( 1, 0 ) = sin( angle );
  affim_matrix( 1, 1 ) = cos( angle );

  affim_coord = ( rotation_center - ( affim_matrix  * rotation_center ) );

  in_rotation = true;
  ptr_descriptor = SizeImaDest();
  in_rotation = false;

  center_dest.X0() = ( ptr_descriptor->Size(0) ) / 2;
  center_dest.X1() = ( ptr_descriptor->Size(1) ) / 2;
   
  center_src -= rotation_center;
  correction = center_dest - (rotation_center) - ( affim_matrix * center_src );
  affim_coord = affim_coord + correction;
  
  delete ptr_descriptor;
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade2D< PixelType, PrecisionType > :: UsePixelInterpolatorLinear()
{
  if( ptr_interpolator_criterion != NULL )
    delete ptr_interpolator_criterion;

  ptr_interpolator_criterion = new bicotiImagePixelInterpolator2DLinear< PixelType, PrecisionType >( ( bicotiImageImplementation2D< PixelType > *)ptr_image_imp_src );
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade2D< PixelType, PrecisionType > :: UsePixelInterpolatorLagrange()
{
  if( ptr_interpolator_criterion != NULL )
    delete ptr_interpolator_criterion;

  ptr_interpolator_criterion = new bicotiImagePixelInterpolator2DLagrange< PixelType, PrecisionType >( ( bicotiImageImplementation2D< PixelType > *)ptr_image_imp_src );
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade2D< PixelType, PrecisionType > :: ApplyAffimTransformation( bool automatic_resize)
{
  in_rotation = false;
  if ( ptr_interpolator_criterion != NULL ) 
    {if ( ptr_image_imp_dest != NULL )
      delete ptr_image_imp_dest;
    
    if ( automatic_resize )
      {bicotiImageDescriptor * ptr_desc;
      ptr_desc = SizeImaDest();
      ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation( ptr_desc );
      delete ptr_desc;}
    else
      ptr_image_imp_dest =  ptr_image_imp_src->CreateImageImplementation( );

    bicotiAffimTransformation< PixelType, bicotiCoordinate2D< REAL > > affim( ptr_image_imp_dest, ptr_image_imp_src, affim_matrix, affim_coord ); 
    affim( ptr_interpolator_criterion );}
  else
    throw bicotiException( "bicotiGeometricalFacade2D", "Apply()", 6 );
};

//----------------------------------bicotiGeometricalFacade3D------------------------------

template< class PixelType, class PrecisionType >
bicotiGeometricalFacade3D< PixelType, PrecisionType > :: bicotiGeometricalFacade3D( bicotiImageImplementation3D< PixelType > * ptr_ima_imp )
{
  ptr_image_imp_src = ptr_ima_imp;
  ptr_image_imp_dest = NULL;
  
  ptr_interpolator_criterion = NULL;	

  size_x0 = ptr_ima_imp->GetX0Size();
  size_x1 = ptr_ima_imp->GetX1Size();
  size_x2 = ptr_ima_imp->GetX2Size();
};


template< class PixelType, class PrecisionType >
bicotiGeometricalFacade3D< PixelType, PrecisionType > :: ~bicotiGeometricalFacade3D()
{    
   if ( ptr_image_imp_dest != NULL )
     delete ptr_image_imp_dest;

  if ( ptr_interpolator_criterion != NULL )
    delete ptr_interpolator_criterion;
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade3D< PixelType, PrecisionType > :: BuildAffimTransformation()
{ 
  bicotiMatrix2D< double > identity( 1 , 3 , 3 );
  identity(0,1) = 0;
  identity(0,2) = 0;
  identity(1,0) = 0;
  identity(1,2) = 0;
  identity(2,0) = 0;
  identity(2,1) = 0;
  affim_matrix = identity;//Asi se con que quedo inicializado    	    
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade3D< PixelType, PrecisionType > :: UsePixelInterpolatorLinear()
{
  if( ptr_interpolator_criterion != NULL )
    delete ptr_interpolator_criterion;

  ptr_interpolator_criterion = new bicotiImagePixelInterpolator3DLinear< PixelType, PrecisionType >( ( bicotiImageImplementation3D< PixelType > *)ptr_image_imp_src );
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade3D< PixelType, PrecisionType > :: UsePixelInterpolatorLagrange()
{
  if( ptr_interpolator_criterion != NULL )
    delete ptr_interpolator_criterion;

  ptr_interpolator_criterion = new bicotiImagePixelInterpolator3DLagrange< PixelType, PrecisionType >( ( bicotiImageImplementation3D< PixelType > *)ptr_image_imp_src );
};


template< class PixelType, class PrecisionType >
bicotiCoordinate3D< REAL >& bicotiGeometricalFacade3D< PixelType, PrecisionType > :: RotationAxis()
{ 
    return ( rotation_axis );		
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade3D< PixelType, PrecisionType > :: BuildRotation( double angle )
{ 
  if ( ( angle < -pi ) || ( angle > pi ) ) 
    throw bicotiException( "bicotiGeometricalFacade3D", "BuildRotation()", 4 );

  bicotiImageDescriptor *  ptr_descriptor;
  bicotiCoordinate3D< REAL > center_src, center_dest, correction;;
  center_src.X0() = ( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp_src)->GetX0Size() ) / 2;
  center_src.X1() = ( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp_src)->GetX1Size() ) / 2;
  center_src.X2() = ( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp_src)->GetX2Size() ) / 2;

  bicotiMatrix2D< double > matrix_rotation( 0, 3, 3 );
  matrix_rotation( 0, 0 ) = cos( angle );
  matrix_rotation( 0, 1 ) = -sin( angle );
  matrix_rotation( 1, 0 ) = sin( angle );
  matrix_rotation( 1, 1 ) = cos( angle );
  matrix_rotation( 2, 2 ) = 1;
  
  bicotiCoordinate3D< REAL > coord_aux( 0, 0, 1 ), coord_new_vect_i, coord_new_vect_j, coord_new_vect_k;
  coord_new_vect_k = rotation_axis * ( (double)1 / rotation_axis.Norm() );
  coord_new_vect_j = coord_aux.VectorialProduct( coord_new_vect_k );

  if ( coord_new_vect_j.Norm() > EPSILON_MACHINE )
    {coord_new_vect_j = coord_new_vect_j * ( (double)1 / coord_new_vect_j.Norm() );
    coord_new_vect_i = coord_new_vect_j.VectorialProduct( coord_new_vect_k );
    coord_new_vect_i = coord_new_vect_i * ( (double)1 / coord_new_vect_i.Norm() );
    
    bicotiMatrix2D< double > matrix_change_coord( 0, 3, 3 );
    
    matrix_change_coord( 0, 0 ) = coord_new_vect_i.X0();
    matrix_change_coord( 1, 0 ) = coord_new_vect_i.X1();
    matrix_change_coord( 2, 0 ) = coord_new_vect_i.X2();
    
    matrix_change_coord( 0, 1 ) = coord_new_vect_j.X0();
    matrix_change_coord( 1, 1 ) = coord_new_vect_j.X1();
    matrix_change_coord( 2, 1 ) = coord_new_vect_j.X2();
    
    matrix_change_coord( 0, 2 ) = coord_new_vect_k.X0();
    matrix_change_coord( 1, 2 ) = coord_new_vect_k.X1();
    matrix_change_coord( 2, 2 ) = coord_new_vect_k.X2();
    
    bicotiMatrix2D< double > matrix_change_coord_inverse( 0, 3, 3 );   
    matrix_change_coord_inverse = matrix_change_coord.Inverse( 0 );    
    affim_matrix = matrix_change_coord_inverse * ( matrix_rotation * matrix_change_coord );}
  else
    affim_matrix = matrix_rotation;
 
  affim_coord = ( rotation_center - ( affim_matrix  * rotation_center ) );

  ptr_descriptor = SizeImaDest();
  center_dest.X0() = ( ptr_descriptor->Size(0) ) / 2;
  center_dest.X1() = ( ptr_descriptor->Size(1) ) / 2;
  center_dest.X2() = ( ptr_descriptor->Size(2) ) / 2;
   
  center_src -= rotation_center;
  correction = center_dest - (rotation_center) - ( affim_matrix * center_src );
  affim_coord = affim_coord + correction;
  
  delete ptr_descriptor;
};


template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade3D< PixelType, PrecisionType > :: ApplyAffimTransformation( bool automatic_resize)
{
  if ( ptr_interpolator_criterion != NULL ) 
    {if ( ptr_image_imp_dest != NULL )
      delete ptr_image_imp_dest;
    
    if ( automatic_resize )
      {bicotiImageDescriptor * ptr_desc;
      ptr_desc = SizeImaDest();
      ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation( ptr_desc );
      delete ptr_desc;}
    else
       ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

    bicotiAffimTransformation< PixelType, bicotiCoordinate3D< REAL > > affim( ptr_image_imp_dest, ptr_image_imp_src, affim_matrix, affim_coord );
 
    affim( ptr_interpolator_criterion );}
  else
    throw bicotiException( "bicotiGeometricalFacade3D", "Apply()", 6 );
};                  


template< class PixelType, class PrecisionType >
bool bicotiGeometricalFacade3D< PixelType, PrecisionType > :: ParallelPrincipalsPlanes( bicotiCoordinate3D< REAL >& pt_1, bicotiCoordinate3D< REAL >& pt_2, bicotiCoordinate3D< REAL >& pt_3) 
{
  if( (fabs( pt_1.X0() - pt_2.X0() ) < EPSILON_MACHINE ) && (fabs( pt_1.X0() - pt_3.X0() ) < EPSILON_MACHINE ) && ( 0 <= pt_1.X0() ) && ( pt_1.X0() < ( size_x0 - 1 ) ) )
    {origin *= 0;
    origin.X0() = pt_1.X0();
    vector_I *= 0;
    vector_I.X1()= 1;
    vector_J *= 0;
    vector_J.X2() = 1;
    bicotiImageSubspace2D< bicotiCoordinate3D< REAL> > subspace( origin, vector_I, vector_J, size_x1, size_x2 );
    bicotiCutter< PixelType, bicotiCoordinate3D< REAL > > cutter( ptr_image_imp_dest, ptr_image_imp_src, &subspace );
    cutter( ptr_interpolator_criterion);
    return true;}
  else
    {if( ( fabs( pt_1.X1() - pt_2.X1() ) < EPSILON_MACHINE ) && ( fabs( pt_1.X1() - pt_3.X1() ) < EPSILON_MACHINE ) && ( 0 <= pt_1.X1() ) && ( pt_1.X1() < ( size_x1 - 1 ) ) )
      {origin *= 0;
      origin.X1() = pt_1.X1();
      vector_I *= 0;
      vector_I.X0() = 1;
      vector_J *= 0;
      vector_J.X2() = 1;
      bicotiImageSubspace2D< bicotiCoordinate3D< REAL> > subspace( origin, vector_I, vector_J, size_x0, size_x2 );
      bicotiCutter< PixelType, bicotiCoordinate3D< REAL > > cutter( ptr_image_imp_dest, ptr_image_imp_src, &subspace);
      cutter( ptr_interpolator_criterion );
      return true;}
    else
      {if( ( fabs( pt_1.X2() - pt_2.X2() ) < EPSILON_MACHINE ) && ( fabs( pt_1.X2() - pt_3.X2() ) < EPSILON_MACHINE ) && ( 0 <= pt_1.X2() ) && ( pt_1.X2() < ( size_x2 - 1 ) ) )
	{origin *= 0;
	origin.X2() = pt_1.X2();
	vector_I *= 0;
	vector_I.X0() = 1;
	vector_J *= 0;
	vector_J.X1() = 1;
	bicotiImageSubspace2D< bicotiCoordinate3D< REAL> > subspace( origin, vector_I, vector_J, size_x0, size_x1 );
	bicotiCutter< PixelType, bicotiCoordinate3D< REAL > > cutter( ptr_image_imp_dest, ptr_image_imp_src, &subspace);
	cutter( ptr_interpolator_criterion);
	return true;}}}
  return false;
}	


template< class PixelType, class PrecisionType >
bool bicotiGeometricalFacade3D< PixelType, PrecisionType > :: InImage( bicotiCoordinate3D< REAL >& coord )
{/*return( ( 0 <= floor( coord.X0() ) ) && ( ( 1 + floor( coord.X0() ) ) <=  ptr_image_imp->GetX0Size() ) &&
	  ( 0 <= floor( coord.X1() ) ) && ( ( 1 + floor( coord.X1() ) ) <=  ptr_image_imp->GetX1Size() ) && 
	  ( 0 <= floor( coord.X2() ) ) && ( ( 1 + floor( coord.X2() ) ) <=  ptr_image_imp->GetX2Size() ) );*/
   return( ( 0 <= coord.X0() ) && ( coord.X0() <=  size_x0 - 1 ) &&
   	   ( 0 <= coord.X1() ) && ( coord.X1() <=  size_x1 - 1 ) && 
	   ( 0 <= coord.X2() ) && ( coord.X2() <=  size_x2 - 1 ) );
}


template< class PixelType, class PrecisionType >
bool bicotiGeometricalFacade3D< PixelType, PrecisionType > :: PlaneForEdge( bicotiCoordinate3D< REAL > vector_coef , bicotiCoordinate3D< REAL > vector_plane )
{
   bool find = false ;
   int temp_x0, temp_x1;   
   bicotiCoordinate3D< REAL > temp_coord, origen;
   temp_coord.X0() = 1;
   temp_coord.X1() = 0;
   temp_coord.X2() = 0;

   if ( fabs( vector_plane.ScalarProduct( temp_coord ) ) < EPSILON_MACHINE ) // La direccion de la recta Ox esta en el plano 
     {vector_I = temp_coord;
      temp_x0 = size_x0;      
      if ( fabs( vector_coef.X2() ) < EPSILON_MACHINE )
      {origin*= 0;
      find = true;}
      else
	{temp_coord.X0() = 0;
        temp_coord.X1() = size_x1 - 1;
        temp_coord.X2() = 0; 		
	if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	  {origin= temp_coord;                                   
          find = true;}
	else
	  {temp_coord.X0() = 0;
          temp_coord.X1() = 0;
          temp_coord.X2() = size_x2 - 1;                
	  if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	    {origin= temp_coord;                             
             find = true;}
	  else
	    {temp_coord.X0() = 0;
             temp_coord.X1() = size_x1 - 1;
             temp_coord.X2() = size_x2 - 1;	        		
	     if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	       {origin= temp_coord;		
               find = true;}}}}}
   else 
     {temp_coord.X0() = 0;
     temp_coord.X1() = 1;
     temp_coord.X2() = 0;        
     if ( fabs( vector_plane.ScalarProduct( temp_coord ) ) < EPSILON_MACHINE ) //la direccion de la recta Oy esta contenida en el plano
       {vector_I = temp_coord;	  
       temp_x0 = size_x1;	 	  
       if ( fabs( vector_coef.X2() ) < EPSILON_MACHINE ) 
	 {origin*= 0;
         find = true;}
       else
	 {temp_coord.X0() = size_x0 - 1;
         temp_coord.X1() = 0;
         temp_coord.X2() = 0;    					
	 if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	   {origin= temp_coord;
           find = true;}
	 else
	   {temp_coord.X0() = 0;
           temp_coord.X1() = 0;
           temp_coord.X2() = size_x2 - 1;                    	
	   if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	     {origin= temp_coord;   		 
             find = true;}
	   else
	     {temp_coord.X0() = size_x0 - 1;
             temp_coord.X1() = 0;
             temp_coord.X2() = size_x2 - 1;					    
	     if ( fabs( vector_plane.ScalarProduct( temp_coord) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	       {origin= temp_coord;			
               find = true;}}}}}
     else
       {temp_coord.X0() = 0;
       temp_coord.X1() = 0;
       temp_coord.X2() = 1;	      
       if ( fabs( vector_plane.ScalarProduct( temp_coord) ) < EPSILON_MACHINE ) //la direccion de la recta Oz esta contenida en el plano
	 {vector_I = temp_coord;            
	 temp_x0 = size_x2;                                 
	 if ( fabs( vector_coef.X2() ) < EPSILON_MACHINE ) 	  
	   {origin*= 0;	     
           find = true;}
	 else
	   {temp_coord.X0() = 0;
	   temp_coord.X1() = size_x1 - 1;
	   temp_coord.X2() = 0;                
	   if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	     {origin= temp_coord;		     
             find = true;}
	   else
	     {temp_coord.X0() = size_x0 - 1;
             temp_coord.X1() = 0;
             temp_coord.X2() = 0;		                       
	     if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
	       {origin= temp_coord;			     
               find = true;}
	     else
	       {temp_coord.X0() = size_x0 - 1;
               temp_coord.X1() = size_x1 - 1;
               temp_coord.X2() = 0;						
	       if ( fabs( vector_plane.ScalarProduct( temp_coord ) - vector_coef.X2() ) < EPSILON_MACHINE ) 
		 {origin= temp_coord;			      
                 find = true;}}}}}}}
    
   if ( find )
     {vector_J = vector_I.VectorialProduct( vector_plane );
     vector_J = vector_J /( vector_J.Norm() );
     temp_coord = origin + vector_J;
     if ( !InImage( temp_coord ) )
       vector_J = vector_J * (-1);
     
     temp_coord = origin + vector_J;
     for( temp_x1 = 1; InImage( temp_coord ); temp_x1++)
       temp_coord += vector_J;

     bicotiImageSubspace2D< bicotiCoordinate3D< REAL> > subspace( origin, vector_I, vector_J, temp_x0, temp_x1);
     bicotiCutter< PixelType, bicotiCoordinate3D< REAL > > cutter( ptr_image_imp_dest, ptr_image_imp_src, &subspace );
     cutter( ptr_interpolator_criterion);
     return true;}
   else
     return false;
}


template< class PixelType, class PrecisionType >	
vector< bicotiCoordinate3D< REAL > > bicotiGeometricalFacade3D< PixelType, PrecisionType > :: ComputePlaneEdgeCut( bicotiCoordinate3D< REAL > vector_coef, bicotiCoordinate3D< REAL > vector_plane )
{			  		
  vector< bicotiCoordinate3D< REAL > > temp_vector;
  REAL temp_real;

  if ( vector_plane.X1() != 0 )
  {temp_real = vector_coef.X2()/vector_plane.X1();//Arista x=0, z=0

  if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x1 - 1 ) )//( temp_real esta entre 0 y ymax)
    {bicotiCoordinate3D< REAL > temp_coord(0, temp_real, 0);
    temp_vector.push_back( temp_coord );}
	
  temp_real = ( vector_coef.X2() - vector_plane.X0() * ( size_x0 - 1 ) ) / vector_plane.X1();
  //Arista x=xmax, z=0
	
  if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x1 - 1 ) )//( temp_real esta entre 0 y ymax)
    {bicotiCoordinate3D< REAL > temp_coord(size_x0 - 1, temp_real, 0);
    temp_vector.push_back( temp_coord );}
	
  temp_real = ( vector_coef.X2() - vector_plane.X0() * ( size_x0 - 1 ) - vector_plane.X2() * ( size_x2 - 1 ) ) / vector_plane.X1();
  //Arista x=xmax, z=zmax
	
  if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x1 - 1 )  )//( temp_real esta entre 0 y ymax)
    {bicotiCoordinate3D< REAL > temp_coord(size_x0 - 1, temp_real, size_x2 - 1);
    temp_vector.push_back( temp_coord );}
	
  temp_real = ( vector_coef.X2() - vector_plane.X2() * ( size_x2 - 1 ) ) / vector_plane.X1();
  //Arista x=0, z=zmax
	
  if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x1 - 1 ) )//( temp_real esta entre 0 y ymax)
    {bicotiCoordinate3D< REAL > temp_coord(0, temp_real, size_x2 - 1);
    temp_vector.push_back( temp_coord );}}

  if ( vector_plane.X2() != 0 )
    {temp_real = vector_coef.X2() / vector_plane.X2();//Arista x=0, y=0
	
    if (  ( 0 <= temp_real ) && ( floor( temp_real ) < size_x2 - 1) )//( temp_real esta entre 0 y zmax)
      {bicotiCoordinate3D< REAL > temp_coord(0, 0, temp_real);
      temp_vector.push_back( temp_coord );}
	
    temp_real = ( vector_coef.X2() - vector_plane.X0() * ( size_x0 - 1 ) ) / vector_plane.X2();
    //Arista x=xmax, y=0
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x2 - 1 ) )//( temp_real esta entre 0 y zmax)
      {bicotiCoordinate3D< REAL > temp_coord(size_x0 - 1, 0, temp_real);
      temp_vector.push_back( temp_coord );}
	
    temp_real = ( vector_coef.X2() - vector_plane.X0() * ( size_x0 - 1) - vector_plane.X1() * ( size_x1 - 1 ) )/ vector_plane.X2();
    //Arista x=xmax, y=ymax
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x2 - 1 ) )//( temp_real esta entre 0 y zmax)
      {bicotiCoordinate3D< REAL > temp_coord( size_x0 - 1, size_x1 - 1, temp_real);
      temp_vector.push_back( temp_coord );}
	
    temp_real = ( vector_coef.X2() - vector_plane.X1() * ( size_x1 - 1 ) )/vector_plane.X2();
    //Arista x=0, y=ymax
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x2 - 1 ) )//( temp_real esta entre 0 y zmax)
      {bicotiCoordinate3D< REAL > temp_coord(0, size_x1 - 1, temp_real);
      temp_vector.push_back( temp_coord );}}

  if ( vector_plane.X0() != 0 )
    {temp_real = vector_coef.X2() / vector_plane.X0();//Arista y=0, z=0
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x0 - 1 ) )//( temp_real esta entre 0 y xmax)
      {bicotiCoordinate3D< REAL > temp_coord( temp_real,0 , 0);
      temp_vector.push_back( temp_coord );}
	
    temp_real = ( vector_coef.X2() - vector_plane.X2() * ( size_x2 - 1 ) ) / vector_plane.X0();
    //Arista y=0, z=zmax
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x0 - 1 ) )//( temp_real esta entre 0 y xmax)
      {bicotiCoordinate3D< REAL > temp_coord( temp_real, 0, size_x2 - 1 );
      temp_vector.push_back( temp_coord );}
	
    temp_real = ( vector_coef.X2() - vector_plane.X1() * ( size_x1 - 1 ) - vector_plane.X2() * ( size_x2 - 1 ) ) / vector_plane.X0();
    //Arista y=ymax, z=zmax
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x0 - 1) )//( temp_real esta entre 0 y xmax)
      {bicotiCoordinate3D< REAL > temp_coord( temp_real, size_x1 - 1, size_x2 - 1);
      temp_vector.push_back( temp_coord );}
	
    temp_real = ( vector_coef.X2() - vector_plane.X1() * ( size_x1 - 1 ) ) / vector_plane.X0();//Arista y=ymax, z=0
	
    if ( ( 0 <= temp_real ) && ( floor( temp_real ) < size_x0 - 1 ) )//( temp_real esta entre 0 y xmax)
      {bicotiCoordinate3D< REAL > temp_coord( temp_real, size_x1 - 1, 0);
      temp_vector.push_back( temp_coord );}}

  return temp_vector;
}

			 
template< class PixelType, class PrecisionType >
void bicotiGeometricalFacade3D< PixelType, PrecisionType > :: BuildCutter( bicotiCoordinate3D< REAL > point_1, bicotiCoordinate3D< REAL > point_2, bicotiCoordinate3D< REAL > point_3, bicotiImageImplementation2D< PixelType > * ptr_ima_imp )
{
  if ( InImage( point_1 ) && InImage( point_2 ) && InImage( point_3 ) && !EqualPoint( point_1, point_2) && !EqualPoint( point_1, point_3) && !EqualPoint( point_2, point_3 ) )
  {bool end = false;
  bicotiCoordinate3D< REAL > vector_coefficient, vect_plane;
  ptr_image_imp_dest = ptr_ima_imp->CreateImageImplementation();

  end = ParallelPrincipalsPlanes( point_1, point_2, point_3);
	
  if ( !end )
    {bicotiCoordinate3D< REAL > temp_1, temp_2, temp_vector;
    temp_1 = point_2 - point_1;
    temp_2 = point_3 - point_1;
    vect_plane = temp_1.VectorialProduct( temp_2 );
    temp_1 = point_1 - point_2;
    temp_2 = point_3 - point_1;
    temp_vector = temp_1.VectorialProduct( temp_2 );
    temp_vector += vect_plane;
    temp_1 = point_3 - point_2;
    temp_2 = point_1 - point_2;
    vect_plane = temp_1.VectorialProduct( temp_2 );
    vect_plane = ( vect_plane + temp_vector )/3;   
    vector_coefficient = vect_plane;
    vector_coefficient.X2() = ( vect_plane.ScalarProduct( point_1 ) + vect_plane.ScalarProduct( point_2 ) + vect_plane.ScalarProduct( point_3) ) / 3;
    end = PlaneForEdge( vector_coefficient, vect_plane );}
  else
    return;
 
   if ( !end ) 
     {vector< bicotiCoordinate3D< REAL > >  vector_edge_cut = ComputePlaneEdgeCut( vector_coefficient, vect_plane );                  
     int h,p;

     for( h = vector_edge_cut.size() - 1 ; 0 <= h ; h --)
       for( p = 0; p < vector_edge_cut.size() ; p++ ) 
	 {if( EqualPoint( vector_edge_cut[ h ], vector_edge_cut[ p ] ) && ( h != p ) )
	   {vector< bicotiCoordinate3D< REAL > > :: iterator first = vector_edge_cut.begin();
	   first += h;
	   vector_edge_cut.erase( first );
	   break;}}//Si los puntos de corte con las aristas difieren poco( < EPSILON_MACHINE), los tomo como iguales.
      
     if ( ( vector_edge_cut.size() <= 2 ) || ( vector_edge_cut.size() >= 7 ) ) 
       throw bicotiException( "bicotiGeometricalFacade3D", "BuildCutter", 7 ); 
     else 
       {vector< bicotiCoordinate3D< REAL > > :: iterator list_first;
       vector< bicotiCoordinate3D< REAL > > :: iterator list_last;
       list_first = vector_edge_cut.begin();
       list_last = vector_edge_cut.end();
       bicotiCoordinateNormCriterion< bicotiCoordinate3D< REAL > > sort_criterion;
       sort( list_first, list_last, sort_criterion);
         	
       bicotiCoordinate3D< REAL >  temp_coord;
       REAL temp_real;
       origin = vector_edge_cut[ 0 ];//Tomo como origen el punto mas cercano al (0,0,0)
         	
       unsigned i;      

       for( i = 1; ( i < vector_edge_cut.size() ) && ( !PointsInSameFace( origin, vector_edge_cut[ i ] ) ); i++ ){}
       //Eleccion de los vectores del subespacio

       vector_I = vector_edge_cut[ i ] - origin;
       vector_I = vector_I /( vector_I.Norm() );
       vector_J = vector_I.VectorialProduct( vect_plane );
       vector_J = vector_J /( vector_J.Norm() );
                     
       bool every_ok = true;
       for( i = 1; ( i < vector_edge_cut.size() ) && every_ok ; i++)
	 {temp_coord = vector_edge_cut[ i ] - origin;
	 if ( !( ( vector_I.ScalarProduct( temp_coord ) >= 0 ) && 
		 ( vector_J.ScalarProduct( temp_coord ) >= 0 ) &&
		 ( fabs( vector_I.ScalarProduct( temp_coord ) ) < EPSILON_MACHINE ) && 
		 ( fabs( vector_J.ScalarProduct( temp_coord ) ) < EPSILON_MACHINE ) ) )//Por Problemas Numericos
	   every_ok = false;}
       
       if ( !every_ok )
	 {temp_coord = vector_edge_cut[ i ] - origin;
	 temp_real = vector_I.ScalarProduct( temp_coord );
	 origin = origin + vector_I * temp_real;}//Correccion del origen
 
	every_ok = true;

	for( i = 1; ( i < vector_edge_cut.size() ) && every_ok ; i++)
	 {temp_coord = vector_edge_cut[ i ] - origin;
	 if ( !( ( vector_I.ScalarProduct( temp_coord ) >= 0 ) && 
                 ( vector_J.ScalarProduct( temp_coord ) >= 0 ) &&
		 ( fabs( vector_I.ScalarProduct( temp_coord ) ) < EPSILON_MACHINE ) && 
		 ( fabs( vector_J.ScalarProduct( temp_coord ) ) < EPSILON_MACHINE ) ) )//Por Problemas Numericos
	   every_ok = false;}

	if ( !every_ok ) 
	 origin = vector_edge_cut[ 0 ];//Mal corregido, lo retorno al volor inicial
	 	
        for( i = 1; i < vector_edge_cut.size(); i++)
	 {temp_coord = vector_edge_cut[ i ] - origin;
	 if ( !( ( vector_I.ScalarProduct( temp_coord ) >= 0 ) && ( vector_J.ScalarProduct( temp_coord ) >= 0  ) ) )
	   {if( ( fabs( vector_J.ScalarProduct( temp_coord ) ) > EPSILON_MACHINE ) &&
		( fabs( vector_I.ScalarProduct( temp_coord ) ) > EPSILON_MACHINE  ) )//Por Problemas Numericos				
	     vector_J = vector_J * (-1);}}//Corrijo el vector_J segun la direccion correcta
              
       REAL temp_x0, temp_x1;
       temp_x0 = 0;
       temp_x1 = 0;
         	
       for( i = 0 ; i <  vector_edge_cut.size() ; i++ )
	 {temp_coord = vector_edge_cut[ i ] - origin;

	 if ( temp_x0 < ( vector_I.VectorialProduct( temp_coord ) ).Norm() )       
	   temp_x0 = ( vector_I.VectorialProduct( temp_coord ) ).Norm();
	         	
	 if ( temp_x1 < ( vector_J.VectorialProduct( temp_coord ) ).Norm() )       	
	   temp_x1 = ( vector_J.VectorialProduct( temp_coord ) ).Norm();}//Calculo de dimensiones de la imagen resultado
         	
       int dim_x0, dim_x1;
       dim_x1 = (int)( floor( temp_x0 ) + 1 );
       dim_x0 = (int)( floor( temp_x1 ) + 1 );
		    	
       bicotiImageSubspace2D< bicotiCoordinate3D< REAL> > subspace( origin, vector_I, vector_J, dim_x0, dim_x1);
       bicotiCutter< PixelType, bicotiCoordinate3D< REAL > > cutter( ptr_image_imp_dest, ptr_image_imp_src, &subspace);
       cutter( ptr_interpolator_criterion );}}}
else
   throw bicotiException( "bicotiGeometricalFacade3D", "BuildCutter", 4 ); 
}


template< class PixelType, class PrecisionType >
bool bicotiGeometricalFacade3D< PixelType, PrecisionType > :: PointsInSameFace( bicotiCoordinate3D< REAL >& pt_1 , bicotiCoordinate3D< REAL >& pt_2)
{
  return ( ( fabs( pt_1.X0() - pt_2.X0() ) < EPSILON_MACHINE ) || 
	   ( fabs( pt_1.X1() - pt_2.X1() ) < EPSILON_MACHINE ) || 
	   ( fabs( pt_1.X2() - pt_2.X2() ) < EPSILON_MACHINE ) );
}


template< class PixelType, class PrecisionType >
bicotiCoordinate3D< REAL >  bicotiGeometricalFacade3D< PixelType, PrecisionType > :: GetOriginSubspace()
{
  return origin;
}


template< class PixelType, class PrecisionType >
bicotiCoordinate3D< REAL >  bicotiGeometricalFacade3D< PixelType, PrecisionType > :: GetVectorISubspace()
{
  return vector_I;
}


template< class PixelType, class PrecisionType >
bicotiCoordinate3D< REAL >  bicotiGeometricalFacade3D< PixelType, PrecisionType > :: GetVectorJSubspace()
{
  return vector_J;
}


template< class PixelType, class PrecisionType >
bool bicotiGeometricalFacade3D< PixelType, PrecisionType > :: EqualPoint( bicotiCoordinate3D< REAL >& pt_1, bicotiCoordinate3D< REAL >& pt_2)
{
  return( (  fabs( pt_1.X0() - pt_2.X0() ) < EPSILON_MACHINE ) && 
	  (  fabs( pt_1.X1() - pt_2.X1() ) < EPSILON_MACHINE ) && 
	  (  fabs( pt_1.X2() - pt_2.X2() ) < EPSILON_MACHINE ) );
}


#endif
