//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _image_subspace
#define _image_subspace


#include <source/geometrical/geometrical_basic.hpp>


//--------------------------DECLARACIONES------------------------- 
//*******************DECLARACIONES DE SUBSPACE********************** 
//----bicotiImageSubspace-----
template < class CoordType > 
class bicotiImageSubspace 
{ 
protected: 
  bool is_done; 
  CoordType current_coord; 
  
public: 
  virtual void First() = 0; 
  virtual void Next() = 0; 
  bool IsDone(); 
  CoordType& GetCurrentCoordinate();

  virtual unsigned GetDimension() = 0;
  virtual int GetSize( unsigned ) = 0;
  
}; 


//-----bicotiImageSubspace1D----------
template < class CoordType > 
class bicotiImageSubspace1D : public bicotiImageSubspace< CoordType > 
{ 
private:
  CoordType origin,vect_X0; 
  int x0_size, current_x0; 

public: 
  bicotiImageSubspace1D(CoordType,CoordType,int); 
  void First(); 
  void Next();

  unsigned GetDimension();
  int GetSize( unsigned );
}; 


//-----bicotiImageSubspace2D----------
template < class CoordType > 
class bicotiImageSubspace2D : public bicotiImageSubspace< CoordType > 
{ 
private:
  CoordType origin,vect_X0,vect_X1; 
  int x0_size,x1_size, current_x0,current_x1; 

public: 
  bicotiImageSubspace2D(CoordType,CoordType, CoordType,int,int); 
  void First(); 
  void Next();

  unsigned GetDimension();
  int GetSize( unsigned );
}; 



//--------------------------IMPLEMENTACIONES------------------------- 
//*******************IMPLEMENTACION DE SUBSPACE********************** 
//-----bicotiImageSubspace--------
template< class CoordType >
bool bicotiImageSubspace< CoordType > :: IsDone()
{
  return (is_done);
} 

template< class CoordType >
CoordType& bicotiImageSubspace< CoordType > :: GetCurrentCoordinate()
{
  return( current_coord );
} 


//------Sunspace1D--------
template< class CoordType > 
bicotiImageSubspace1D< CoordType > :: bicotiImageSubspace1D(CoordType orig,CoordType vect_X_1,int x_1) 
{ 
  origin = orig; 
  vect_X0 = vect_X_1;  
  if( abs( vect_X0.Norm() - 1) > norm_eps )
    throw bicotiException( "bicotiImageSubspace1D", "Constructor", 4 );

  x1_size = x_1; 
  First(); 
} 


template< class CoordType >
void bicotiImageSubspace1D< CoordType > :: First()
{
  current_x0 = 0;
  current_coord = origin;
  is_done = false;
}


template< class CoordType > 
void bicotiImageSubspace1D< CoordType > :: Next() 
{ 
  if ( current_x0 + 1 < x0_size ) 
    {current_x0++; 
    current_coord = ( origin + vect_X0 * current_x0 ) ;} 
  else
    is_done = true;
} 


template< class CoordType > 
unsigned  bicotiImageSubspace1D< CoordType > :: GetDimension() 
{ 
  return 1;
}


template< class CoordType > 
int  bicotiImageSubspace1D< CoordType > :: GetSize( unsigned index ) 
{ 
  if ( index != 0 )
    throw bicotiException( "bicotiImageSubspace1D", "GetSize()", 1 );
  else
    return x0_size;
}


//-----bicotiImageSubspace2D--------
template< class CoordType > 
bicotiImageSubspace2D< CoordType > :: bicotiImageSubspace2D( CoordType orig, CoordType vect_X_1, CoordType vect_X_2, int x_1, int x_2) 
{ 
  origin = orig; 
  vect_X0 = vect_X_1; 
  vect_X1 = vect_X_2; 

    if( ( vect_X0.ScalarProduct( vect_X1 ) ) > EPSILON_MACHINE) 
      throw bicotiException( "bicotiImageSubspace2D", "Constructor", 4 );

    if (( fabs( vect_X0.Norm()-1 ) > EPSILON_MACHINE )||( fabs(( vect_X1.Norm()-1 ) > EPSILON_MACHINE ))) 
      throw bicotiException( "bicotiImageSubspace2D", "Constructor", 4 );
    
  x0_size = x_1; 
  x1_size = x_2; 
  First(); 
} 


template< class CoordType >
void bicotiImageSubspace2D< CoordType > :: First()
{
  current_x0 = 0;
  current_x1 = 0;
  current_coord = origin;
  is_done = false;
}


template< class CoordType > 
void bicotiImageSubspace2D< CoordType > :: Next() 
{ 
  if ( ( current_x0 + 1 ) < x0_size ) 
    {current_x0++; 
    current_coord = (origin + vect_X0 * current_x0 + vect_X1 * current_x1 );} 
  else 
    {if ( ( current_x1 + 1 ) < x1_size ) 
      {current_x1++; 
      current_x0 = 0; 
      current_coord = (origin + vect_X1 * current_x1 );} 
    else 
      is_done = true;} 
};              


template< class CoordType > 
unsigned  bicotiImageSubspace2D< CoordType > :: GetDimension() 
{ 
  return 2;
}


template< class CoordType > 
int  bicotiImageSubspace2D< CoordType > :: GetSize( unsigned index ) 
{ 
  if ( index > 1 )
    throw bicotiException( "bicotiImageSubspace2D", "GetSize()", 1 );
  else
  {switch ( index ){
   case 0: 
     return x0_size;
   case 1:
     return x1_size;}};
}



#endif
