//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _image_2D_single
#define _image_2D_single



#include <source/image/image_2D.hpp>



//**********************DECLARATIONS**************************
template< class ImageType, class ColorType, class PixelConverterMapper >
class bicotiImage2DSingle : public bicotiImage2D< ImageType, ColorType, PixelConverterMapper, ColorType, REAL >
{
public:
  void BuildImplementation( unsigned, unsigned );
  
  void BuildProperty();
  bicotiPropertyFacade< bicotiCoordinate2D< INTEGER >, ColorType, ColorType, REAL > * CreateProperty();

  void BuildNeighbourFilter();
  bicotiNeighbourFilterFacade2D< ColorType, REAL > * CreateNeighbourFilter();

  void BuildDexel();
  bicotiDexelFacade2DSingle< ColorType > * CreateDexel();
};




//********************IMPLEMENTATIONS*************************
template< class ImageType, class ColorType, class PixelConverterMapper >
void bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > :: BuildImplementation( unsigned x0_size, unsigned x1_size )
{
  if(ptr_implementation != NULL)
    delete ptr_implementation;
  
  ptr_implementation = new ImageType( 0, x0_size, x1_size, 1, single );
};


template< class ImageType, class ColorType, class PixelConverterMapper >
void bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > :: BuildProperty()
{
  if( ptr_property != NULL )
    delete ptr_property;
  
  ptr_property = new bicotiPropertyFacadeSingle< bicotiCoordinate2D< INTEGER >, ColorType >( ptr_implementation );
};


template< class ImageType, class ColorType, class PixelConverterMapper >
bicotiPropertyFacade< bicotiCoordinate2D< INTEGER >, ColorType, ColorType, REAL >* bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > :: CreateProperty()
{
  return ( new bicotiPropertyFacadeSingle< bicotiCoordinate2D< INTEGER >, ColorType >( ptr_implementation ));
};


template< class ImageType, class ColorType, class PixelConverterMapper >
void bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > :: BuildNeighbourFilter()
{
  if( ptr_neighbour_filter != NULL )
    delete ptr_neighbour_filter;

  ptr_neighbour_filter = new bicotiNeighbourFilterFacade2DSingle< ColorType, REAL > ( ptr_implementation );
};


template< class ImageType, class ColorType, class PixelConverterMapper >
bicotiNeighbourFilterFacade2D< ColorType, REAL >* bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > ::  CreateNeighbourFilter()
{
  return( new bicotiNeighbourFilterFacade2DSingle< ColorType, REAL > ( ptr_implementation ) );
};


template< class ImageType, class ColorType, class PixelConverterMapper >
void bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > :: BuildDexel()
{
  if( ptr_dexel != NULL )
    delete ptr_dexel;

  ptr_dexel = new bicotiDexelFacade2DSingle< ColorType >( ptr_implementation );

};


template< class ImageType, class ColorType, class PixelConverterMapper >
bicotiDexelFacade2DSingle< ColorType > * bicotiImage2DSingle< ImageType, ColorType, PixelConverterMapper > :: CreateDexel()
{
  return ( new bicotiDexelFacade2DSingle< ColorType >( ptr_implementation ) );
};



#endif
