//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _image_pixel_interpolator_2D_lagrange
#define _image_pixel_interpolator_2D_lagrange



#include <source/image_pixel_interpolator/image_pixel_interpolator_2D.hpp>



//************************DECLARATIONS****************************
template< class PixelType, class PrecisionType = PixelType >  
class bicotiImagePixelInterpolator2DLagrange : public bicotiImagePixelInterpolator2D< PixelType > 
{  
public:   
  bicotiImagePixelInterpolator2DLagrange( bicotiImageImplementation2D< PixelType > * = NULL );  
  
  PixelType operator() ( bicotiCoordinate< REAL >& );
};  




//***********************IMPLEMENTATIONS**************************
//--------------bicotiImagePixelInterpolator2DLagrange------------
template< class PixelType, class PrecisionType >  
bicotiImagePixelInterpolator2DLagrange< PixelType, PrecisionType > :: bicotiImagePixelInterpolator2DLagrange(bicotiImageImplementation2D< PixelType >* ptr_ima_imp)  
{  
  ptr_image_imp = ptr_ima_imp;
}  


template< class PixelType, class PrecisionType >
inline PixelType bicotiImagePixelInterpolator2DLagrange< PixelType, PrecisionType > :: operator() ( bicotiCoordinate< REAL >& coord )     
{  
  PrecisionType pixel_inter;
  bicotiCoordinate2D< REAL > aux_coord;
  aux_coord = (bicotiCoordinate2D< REAL >)coord;
  
#ifdef TOTAL_CHECK
  if ( ( aux_coord.GetDimension() != 2 ) || ( ptr_image_imp == NULL ) )
    throw bicotiException( "bicotiImagePixelInterpolator2DLagrange", "operator()", 3 );
#endif
  
  if ( InImage( aux_coord )&& ( !InBorder( aux_coord )) )
    {bicotiCoordinate2D< REAL > coord_0, coord_1, coord_2, coord_3;
    REAL aux_inter_x0, aux_inter_x1;
    
    aux_inter_x0 = ( REAL )floor( aux_coord.X0() );
    aux_inter_x1 = ( REAL )floor( aux_coord.X1() );
    
    coord_0.X0() = aux_inter_x0;
    coord_0.X1() = aux_inter_x1 + 1;
    coord_1.X0() = aux_inter_x0 + 1;
    coord_1.X1() = aux_inter_x1 + 1;
    coord_2.X0() = aux_inter_x0;
    coord_2.X1() = aux_inter_x1;
    coord_3.X0() = aux_inter_x0 + 1;
    coord_3.X1() = aux_inter_x1;
    
    REAL aux_r, aux_s, w0, w1, w2, w3;
    aux_r = ( aux_coord.X0() - coord_0.X0() );
    aux_s = ( coord_0.X1() - aux_coord.X1() );
    
    w0 = ( 1 - aux_r ) * ( 1 - aux_s );
    w1 = aux_r * ( 1 - aux_s );
    w2 = ( 1 - aux_r ) * aux_s;
    w3 = aux_r * aux_s ;
    
    PrecisionType pixel_aux_0, pixel_aux_1, pixel_aux_2, pixel_aux_3;
    int temp_x0, temp_x1;
    
    temp_x0 = (int)coord_0.X0(); 
    temp_x1 = (int)coord_0.X1();
    pixel_aux_0 = (PrecisionType)( ( ( bicotiImageImplementation2D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1) );
    temp_x0 = (int)coord_1.X0(); 
    temp_x1 = (int)coord_1.X1();
    pixel_aux_1 = (PrecisionType)( ( ( bicotiImageImplementation2D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1) );
    temp_x0 = (int)coord_2.X0(); 
    temp_x1 = (int)coord_2.X1();
    pixel_aux_2 = (PrecisionType)( ( ( bicotiImageImplementation2D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1) );
    temp_x0 = (int)coord_3.X0(); 
    temp_x1 = (int)coord_3.X1();
    pixel_aux_3 = (PrecisionType)( ( ( bicotiImageImplementation2D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1) );
    
    pixel_inter = pixel_aux_0 * w0 + pixel_aux_1 * w1 + pixel_aux_2 * w2 + pixel_aux_3 * w3;}
  else
    pixel_inter = (PrecisionType)( ptr_image_imp->GetDefaultPixel() );
  
  return( (PixelType)pixel_inter );
};


#endif
