//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _image_pixel_interpolator_2D_linear
#define _image_pixel_interpolator_2D_linear



#include <source/image_pixel_interpolator/image_pixel_interpolator_2D.hpp>



//************************DECLARATIONS****************************
//-----------bicotiImagePixelInterpolator2DLinear-----------------
template< class PixelType, class PrecisionType = PixelType >  
class bicotiImagePixelInterpolator2DLinear : public bicotiImagePixelInterpolator2D< PixelType > 
{
private:
  vector< bicotiCoordinateDistance< bicotiCoordinate2D< REAL > > > lista_coord;
  vector< bicotiCoordinateDistance< bicotiCoordinate2D< REAL > > > :: iterator list_first;
  vector< bicotiCoordinateDistance< bicotiCoordinate2D< REAL > > > :: iterator list_last;  

public:   
  bicotiImagePixelInterpolator2DLinear( bicotiImageImplementation2D< PixelType > * = NULL ); 
 
  
  PixelType operator() ( bicotiCoordinate< REAL >& );
};  




//**********************IMPLEMENTATIONS***************************
//-------------bicotiImagePixelInterpolator2DLinear---------------
template< class PixelType, class PrecisionType >  
bicotiImagePixelInterpolator2DLinear< PixelType, PrecisionType > :: bicotiImagePixelInterpolator2DLinear( bicotiImageImplementation2D< PixelType > * ptr_ima_imp )  
{  
  ptr_image_imp = ptr_ima_imp;
  vector< bicotiCoordinateDistance< bicotiCoordinate2D< REAL > > > lista_coord_aux( 4 );
  lista_coord = lista_coord_aux;
};


template< class PixelType, class PrecisionType >  
inline PixelType bicotiImagePixelInterpolator2DLinear< PixelType, PrecisionType > :: operator() ( bicotiCoordinate< REAL >& coord )  
{  
  PrecisionType pixel_inter;
  bicotiCoordinate2D< REAL > aux_coord;
  aux_coord = (bicotiCoordinate2D< REAL >)coord;
  
#ifdef TOTAL_CHECK
  if ( ( aux_coord.GetDimension() != 2 ) || ( ptr_image_imp == NULL ) )
    throw bicotiException( "bicotiImagePixelInterpolator2DLinear", "operator()", 3 );
#endif
  
  if ( InImage( aux_coord ) && ( !InBorder( aux_coord ) ) )
    {lista_coord[ 0 ].internal_coord = aux_coord;
    lista_coord[ 1 ].internal_coord = aux_coord;
    lista_coord[ 2 ].internal_coord = aux_coord;
    lista_coord[ 3 ].internal_coord = aux_coord;
    
    REAL aux_inter_x0, aux_inter_x1;
    
    aux_inter_x0 = ( REAL )floor( aux_coord.X0() );
    aux_inter_x1 = ( REAL )floor( aux_coord.X1() );

    (lista_coord[ 0 ].external_coord).X0() = aux_inter_x0;
    (lista_coord[ 0 ].external_coord).X1() = aux_inter_x1;
    (lista_coord[ 1 ].external_coord).X0() = aux_inter_x0;
    (lista_coord[ 1 ].external_coord).X1() = aux_inter_x1 + 1;
    (lista_coord[ 2 ].external_coord).X0() = aux_inter_x0 + 1;
    (lista_coord[ 2 ].external_coord).X1() = aux_inter_x1;
    (lista_coord[ 3 ].external_coord).X0() = aux_inter_x0 + 1;
    (lista_coord[ 3 ].external_coord).X1() = aux_inter_x1 + 1;
    
    lista_coord[ 0 ].Update();
    lista_coord[ 1 ].Update();
    lista_coord[ 2 ].Update();
    lista_coord[ 3 ].Update();
    
    bicotiCoordinateDistanceCriterion< bicotiCoordinate2D< REAL > > compar_crit;
    list_first = lista_coord.begin();
    list_last = lista_coord.end();
    sort( list_first, list_last, compar_crit );
    
    REAL aux_x1, aux_y1, aux_x2, aux_y2, aux_x3, aux_y3, Det;

    aux_x1 = ( lista_coord[ 0 ].external_coord ).X0();
    aux_y1 = ( lista_coord[ 0 ].external_coord ).X1();
    aux_x2 = ( lista_coord[ 1 ].external_coord ).X0();
    aux_y2 = ( lista_coord[ 1 ].external_coord ).X1();
    aux_x3 = ( lista_coord[ 2 ].external_coord ).X0();
    aux_y3 = ( lista_coord[ 2 ].external_coord ).X1();

    Det = ( ( ( aux_x3 * aux_y2 ) - ( aux_x2 * aux_y3 ) ) + ( ( aux_x1 * aux_y3 ) - ( aux_x3 * aux_y1 ) ) + ( ( aux_x2 * aux_y1 ) - ( aux_x1 * aux_y2 ) ) ); // HABRIA QUE VER SI PUEDE SER (-1)POW ALGO.
      
    PrecisionType pixel_aux_0, pixel_aux_1, pixel_aux_2;

     pixel_aux_0 = (PrecisionType)( ( (bicotiImageImplementation2D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x1, (int)aux_y1) );
     pixel_aux_1 = (PrecisionType)( ( (bicotiImageImplementation2D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x2, (int)aux_y2) );
     pixel_aux_2 = (PrecisionType)( ( (bicotiImageImplementation2D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x3, (int)aux_y3) );

    pixel_inter = (PrecisionType)( ( ( pixel_aux_0 * ( aux_y3 - aux_y2 ) ) + ( pixel_aux_1 * ( aux_y1 - aux_y3 ) ) + ( pixel_aux_2 * ( aux_y2 - aux_y1 ) ) )* ( aux_coord.X0() / Det ) ) ;    
    pixel_inter += (PrecisionType)( ( ( pixel_aux_0 * ( aux_x2 - aux_x3 ) ) + ( pixel_aux_1 * ( aux_x3 - aux_x1 ) ) + ( pixel_aux_2 * ( aux_x1 - aux_x2 ) ) )* ( aux_coord.X1() / Det ) );    
    pixel_inter -= (PrecisionType)( ( ( pixel_aux_0 * ( ( aux_x2 * aux_y3 ) - ( aux_x3 * aux_y2 ) ) ) +( pixel_aux_1 * ( ( aux_x3 * aux_y1 ) - ( aux_x1 * aux_y3 ) ) ) + ( pixel_aux_2 * ( ( aux_x1 * aux_y2 ) - ( aux_x2 * aux_y1 ) ) ) ) * ( 1 / Det ) ) ;}
  else
     pixel_inter =(PrecisionType)(  ptr_image_imp->GetDefaultPixel() );
 
  return( (PixelType)pixel_inter );
};                  


#endif
