//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _image_pixel_interpolator_3D_linear
#define _image_pixel_interpolator_3D_linear



#include <source/image_pixel_interpolator/image_pixel_interpolator_3D.hpp>



//************************DECLARATIONS****************************
//-------------bicotiImagePixelInterpolator3DLinear---------------
template< class PixelType, class PrecisionType = PixelType >  
class bicotiImagePixelInterpolator3DLinear : public bicotiImagePixelInterpolator3D< PixelType > 
{  
private:
  vector< bicotiCoordinateDistance< bicotiCoordinate3D< REAL > > > lista_coord;
  vector< bicotiCoordinateDistance< bicotiCoordinate3D< REAL > > > :: iterator list_first;
  vector< bicotiCoordinateDistance< bicotiCoordinate3D< REAL > > > :: iterator list_last;
  
public:   
  bicotiImagePixelInterpolator3DLinear( bicotiImageImplementation3D< PixelType > * = NULL );  
  
  PixelType operator() ( bicotiCoordinate< REAL >& );
};  




//***********************IMPLEMENTATIONS**************************
//--------------bicotiImagePixelInterpolator3DLinear--------------
template< class PixelType, class PrecisionType >  
bicotiImagePixelInterpolator3DLinear< PixelType, PrecisionType > :: bicotiImagePixelInterpolator3DLinear( bicotiImageImplementation3D< PixelType > * ptr_ima_imp )  
{  
  ptr_image_imp = ptr_ima_imp;
  vector< bicotiCoordinateDistance< bicotiCoordinate3D< REAL > > > lista_coord_aux( 8 );
  lista_coord = lista_coord_aux;
};


template< class PixelType, class PrecisionType >  
inline PixelType bicotiImagePixelInterpolator3DLinear< PixelType, PrecisionType > :: operator() ( bicotiCoordinate< REAL >& coord )  
{  
  PrecisionType pixel_inter;
  bicotiCoordinate3D< REAL > aux_coord;
  aux_coord = (bicotiCoordinate3D< REAL >)coord;

#ifdef TOTAL_CHECK
  if ( ( aux_coord.GetDimension() != 3 ) || (ptr_image_imp == NULL ) )
    throw bicotiException( "bicotiImagePixelInterpolator3DLinear", "operator()", 3 );
#endif
  
  if ( InImage( aux_coord ) && ( !InBorder( aux_coord ) ) )
    {lista_coord[ 0 ].internal_coord = aux_coord;
    lista_coord[ 1 ].internal_coord = aux_coord;
    lista_coord[ 2 ].internal_coord = aux_coord;
    lista_coord[ 3 ].internal_coord = aux_coord;
    lista_coord[ 4 ].internal_coord = aux_coord;
    lista_coord[ 5 ].internal_coord = aux_coord;
    lista_coord[ 6 ].internal_coord = aux_coord;
    lista_coord[ 7 ].internal_coord = aux_coord;
    
    REAL aux_inter_x0, aux_inter_x1, aux_inter_x2;
    
    aux_inter_x0 = ( REAL )floor( aux_coord.X0() );
    aux_inter_x1 = ( REAL )floor( aux_coord.X1() );
    aux_inter_x2 = ( REAL )floor( aux_coord.X2() );
    
    (lista_coord[ 0 ].external_coord).X0() = aux_inter_x0;
    (lista_coord[ 0 ].external_coord).X1() = aux_inter_x1;
    (lista_coord[ 0 ].external_coord).X2() = aux_inter_x2;
    
    (lista_coord[ 1 ].external_coord).X0() = aux_inter_x0;
    (lista_coord[ 1 ].external_coord).X1() = aux_inter_x1;
    (lista_coord[ 1 ].external_coord).X2() = aux_inter_x2 + 1;
    
    (lista_coord[ 2 ].external_coord).X0() = aux_inter_x0;
    (lista_coord[ 2 ].external_coord).X1() = aux_inter_x1 + 1;
    (lista_coord[ 2 ].external_coord).X2() = aux_inter_x2;

    (lista_coord[ 3 ].external_coord).X0() = aux_inter_x0;
    (lista_coord[ 3 ].external_coord).X1() = aux_inter_x1 + 1;
    (lista_coord[ 3 ].external_coord).X2() = aux_inter_x2 + 1;
    
    (lista_coord[ 4 ].external_coord).X0() = aux_inter_x0 + 1;
    (lista_coord[ 4 ].external_coord).X1() = aux_inter_x1;
    (lista_coord[ 4 ].external_coord).X2() = aux_inter_x2;
    
    (lista_coord[ 5 ].external_coord).X0() = aux_inter_x0 + 1;
    (lista_coord[ 5 ].external_coord).X1() = aux_inter_x1;
    (lista_coord[ 5 ].external_coord).X2() = aux_inter_x2 + 1;
    
    (lista_coord[ 6 ].external_coord).X0() = aux_inter_x0 + 1;
    (lista_coord[ 6 ].external_coord).X1() = aux_inter_x1 + 1;
    (lista_coord[ 6 ].external_coord).X2() = aux_inter_x2;
    
    (lista_coord[ 7 ].external_coord).X0() = aux_inter_x0 + 1;
    (lista_coord[ 7 ].external_coord).X1() = aux_inter_x1 + 1;
    (lista_coord[ 7 ].external_coord).X2() = aux_inter_x2 + 1;
      
    lista_coord[ 0 ].Update();
    lista_coord[ 1 ].Update();
    lista_coord[ 2 ].Update();
    lista_coord[ 3 ].Update();
    lista_coord[ 4 ].Update();
    lista_coord[ 5 ].Update();
    lista_coord[ 6 ].Update();
    lista_coord[ 7 ].Update();
    
    bicotiCoordinateDistanceCriterion< bicotiCoordinate3D< REAL > > compar_crit;
    list_first = lista_coord.begin();
    list_last = lista_coord.end();
    sort( list_first, list_last, compar_crit );
    
    REAL aux_x0, aux_y0, aux_z0,aux_x1, aux_y1, aux_z1, aux_x2, aux_y2, aux_z2, aux_x3, aux_y3, aux_z3, Det;
    
    aux_x0 = ( lista_coord[ 0 ].external_coord ).X0();
    aux_y0 = ( lista_coord[ 0 ].external_coord ).X1();
    aux_z0 = ( lista_coord[ 0 ].external_coord ).X2();
    
    vector< bicotiCoordinateDistance< bicotiCoordinate3D< REAL > > > lista_coord_aux(7);
  
    int i;
    for( i = 0; i < 7 ; i++)
      {lista_coord_aux[ i ].internal_coord = lista_coord[ 0 ].external_coord;
      lista_coord_aux[ i ].external_coord = lista_coord[ i+1 ].external_coord;
      lista_coord_aux[ i ].Update();}
    
    
    list_first = lista_coord_aux.begin();
    list_last = lista_coord_aux.end();
    sort( list_first, list_last, compar_crit );
    
    aux_x1 = ( lista_coord_aux[ 0 ].external_coord ).X0();
    aux_y1 = ( lista_coord_aux[ 0 ].external_coord ).X1();
    aux_z1 = ( lista_coord_aux[ 0 ].external_coord ).X2();
    
    aux_x2 = ( lista_coord_aux[ 1 ].external_coord ).X0();
    aux_y2 = ( lista_coord_aux[ 1 ].external_coord ).X1();
    aux_z2 = ( lista_coord_aux[ 1 ].external_coord ).X2();
    
    aux_x3 = ( lista_coord_aux[ 2 ].external_coord ).X0();
    aux_y3 = ( lista_coord_aux[ 2 ].external_coord ).X1();
    aux_z3 = ( lista_coord_aux[ 2 ].external_coord ).X2();
	
    Det =  (aux_x1 - aux_x0) * ( ( (aux_y2-aux_y0)*(aux_z3-aux_z0) )-( (aux_z2-aux_z0)*(aux_y3-aux_y0) ) );
    Det -= (aux_y1 - aux_y0) * ( ( (aux_x2-aux_x0)*(aux_z3-aux_z0) )-( (aux_z2-aux_z0)*(aux_x3-aux_x0) ) );
    Det += (aux_z1 - aux_z0) * ( ( (aux_x2-aux_x0)*(aux_y3-aux_y0) )-( (aux_y2-aux_y0)*(aux_x3-aux_x0) ) );
    
    PrecisionType pixel_aux_0, pixel_aux_1, pixel_aux_2, pixel_aux_3;
    
    pixel_aux_0 = (PrecisionType)( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x0, (int)aux_y0, (int)aux_z0 ) );
    pixel_aux_1 = (PrecisionType)( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x1, (int)aux_y1, (int)aux_z1 ) );
    pixel_aux_2 = (PrecisionType)( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x2, (int)aux_y2, (int)aux_z2 ) );
    pixel_aux_3 = (PrecisionType)( ( (bicotiImageImplementation3D< PixelType >*)ptr_image_imp )->GetPixel( (int)aux_x3, (int)aux_y3, (int)aux_z3 ) );
   
    pixel_inter = (PrecisionType)( ( 
		   ( pixel_aux_0 * ( aux_y1 * (aux_z3-aux_z2) - aux_z1 * (aux_y3-aux_y2) - (aux_y2*aux_z3 - aux_y3*aux_z2) ) ) +
		   ( pixel_aux_1 * ( aux_y0 * (aux_z2-aux_z3) + aux_z0 * (aux_y3-aux_y2) + (aux_y2*aux_z3 - aux_y3*aux_z2) ) ) +
		   ( pixel_aux_2 * ( aux_y0 * (aux_z3-aux_z1) - aux_z0 * (aux_y3-aux_y1) - (aux_y1*aux_z3 - aux_y3*aux_z1) ) ) +
		   ( pixel_aux_3 * ( aux_y0 * (aux_z1-aux_z2) + aux_z0 * (aux_y2-aux_y1) + (aux_y1*aux_z2 - aux_y2*aux_z1) ) ) 
		   )* ( aux_coord.X0() / Det ) );
    
    
    pixel_inter += (PrecisionType)( ( 
		    ( pixel_aux_0 * ( aux_x1 * (aux_z2-aux_z3) + aux_z1 * (aux_x3-aux_x2) + (aux_x2*aux_z3 - aux_x3*aux_z2) ) ) +
		    ( pixel_aux_1 * ( aux_x0 * (aux_z3-aux_z2) - aux_z0 * (aux_x3-aux_x2) - (aux_x2*aux_z3 - aux_x3*aux_z2) ) ) +
		    ( pixel_aux_2 * ( aux_x0 * (aux_z1-aux_z3) + aux_z0 * (aux_x3-aux_x1) + (aux_x1*aux_z3 - aux_x3*aux_z1) ) ) +
		    ( pixel_aux_3 * ( aux_x0 * (aux_z2-aux_z1) - aux_z0 * (aux_x2-aux_x1) - (aux_x1*aux_z2 - aux_x2*aux_z1) ) ) 
		    )* ( aux_coord.X1() / Det ) );
    
    pixel_inter += (PrecisionType)( (
		    ( pixel_aux_0 * ( aux_x1 * (aux_y3-aux_y2) - aux_y1 * (aux_x3-aux_x2) - (aux_x2*aux_y3 - aux_x3*aux_y2) ) ) +
		    ( pixel_aux_1 * ( aux_x0 * (aux_y2-aux_y3) + aux_y0 * (aux_x3-aux_x2) + (aux_x2*aux_y3 - aux_x3*aux_y2) ) ) +
		    ( pixel_aux_2 * ( aux_x0 * (aux_y3-aux_y1) - aux_y0 * (aux_x3-aux_x1) - (aux_x1*aux_y3 - aux_x3*aux_y1) ) ) +
		    ( pixel_aux_3 * ( aux_x0 * (aux_y1-aux_y2) + aux_y0 * (aux_x2-aux_x1) + (aux_x1*aux_y2 - aux_x2*aux_y1) ) ) 
		    )* ( aux_coord.X2() / Det ) );
    
    pixel_inter -= (PrecisionType)( 
		    ( ( pixel_aux_0 * ( aux_x1 * (aux_y3*aux_z2 - aux_y2*aux_z3) + aux_y1 * (aux_x2*aux_z3 - aux_x3*aux_z2) + aux_z1 * (aux_x3*aux_y2 - aux_x2*aux_y3) ) ) +
		    ( pixel_aux_1 * ( aux_x0 * (aux_y2*aux_z3 - aux_y3*aux_z2) - aux_y0 * (aux_x2*aux_z3 - aux_x3*aux_z2) + aux_z0 * (aux_x2*aux_y3 - aux_x3*aux_y2) ) ) +
		    ( pixel_aux_2 * ( aux_x0 * (aux_y3*aux_z1 - aux_y1*aux_z3) + aux_y0 * (aux_x1*aux_z3 - aux_x3*aux_z1) + aux_z0 * (aux_y1*aux_x3 - aux_y3*aux_x1) ) ) +
		    ( pixel_aux_3 * ( aux_x0 * (aux_y1*aux_z2 - aux_y2*aux_z1) - aux_y0 * (aux_x1*aux_z2 - aux_x2*aux_z1) + aux_z0 * (aux_x1*aux_y2 - aux_x2*aux_y1) ) ) 
		    ) * ( 1/Det) );}
  else
    pixel_inter = (PrecisionType)( ptr_image_imp->GetDefaultPixel() );
  
  return( (PixelType)(pixel_inter) );
};                                            



#endif
