#ifndef _bicoti_mapper_basic
#define _bicoti_mapper_basic

#include <source/device/device.hpp>
#include <source/image_implementation/image_implementation.hpp>


//--------------------DECLARATIONS---------------------
//--------------------bicotiMapper---------------------

/** 
 * Clase base de los mappers
 *
 * Un mapper permite convertir una imagen en algn tipo grfico a bicotiImageImplementation
 */
template< class PixelType >
class bicotiMapper
{
protected:
  bicotiDevice * ptr_device;
  bicotiImageImplementation< PixelType > * ptr_image;

public:
  /** 
   * Determina el device a usar por el mapper 
   *
   *@param ptr_dev device
   *@see bicotiDevice
   */
  void SetDevice( bicotiDevice * ptr_dev );

  /** 
   * Determina la imagen bicoti de donde el  mapper lee o escribe 
   *
   *@param ptr_ima imagen bicoti
   */
  void SetImplementation( bicotiImageImplementation< PixelType > * ptr_ima);
  
  /** 
   * Lee una imagen y la pone en la bicotiImageImplementation
   *
   */
  virtual void Read()=0;
  
  /**
   * Escribe una imagen a partir de la bicotiImageImplementation
   *
   */
  virtual void Write()=0;
};


//------------------IMPLEMENTATIONS--------------------
//--------------------bicotiMapper---------------------
template< class PixelType >
inline void bicotiMapper< PixelType > :: SetDevice( bicotiDevice * ptr_dev )
{ptr_device = ptr_dev;}


template< class PixelType >
inline void bicotiMapper< PixelType > :: SetImplementation( bicotiImageImplementation< PixelType > * ptr_ima )
{ptr_image = ptr_ima;}






#endif
