#include <source/mapper/mapper_jpeg_ijg.hpp>
#include <source/pixel/pixel_rgb_tools.hpp>


//-----------------bicotiMapperJPEGIJG-----------------
template< class PixelType, class PixelConverter >
bicotiMapperJPEGIJG< PixelType,PixelConverter  > :: bicotiMapperJPEGIJG()
{ptr_device = (bicotiDevice *)NULL;
 ptr_image = (bicotiImageImplementation< PixelType > *)NULL;
 SetDefaults();
}


template< class PixelType, class PixelConverter >
void bicotiMapperJPEGIJG< PixelType,PixelConverter > :: reload_buffer( bicotiDevice * dev_ptr , char * work_buffer, struct jpeg_decompress_struct * cinfo )
{char trash_buffer[WORK_BUFFER_SIZE];
 
 for ( int i=0; i < cinfo->src->bytes_in_buffer ; i++)
   work_buffer[i] = (char)cinfo->src->next_input_byte[i];
 if (bytes_to_skip > 0) 
   {dev_ptr->BlockSize() = bytes_to_skip;
   dev_ptr->Get((char *)trash_buffer);}
 
 if ( byte_count < WORK_BUFFER_SIZE - cinfo->src->bytes_in_buffer )
   {dev_ptr->BlockSize() = byte_count;
   cinfo->src->next_input_byte = (JOCTET *)work_buffer + cinfo->src->bytes_in_buffer;
   dev_ptr->Get((char *)cinfo->src->next_input_byte);
   cinfo->src->next_input_byte = (JOCTET *)work_buffer;
   cinfo->src->bytes_in_buffer = cinfo->src->bytes_in_buffer + byte_count + 1 ;
   work_buffer[cinfo->src->bytes_in_buffer] = JPEG_EOI;}
 else 
   {byte_count-= (WORK_BUFFER_SIZE - cinfo->src->bytes_in_buffer);
   dev_ptr->BlockSize() = WORK_BUFFER_SIZE - cinfo->src->bytes_in_buffer;
   // cinfo->src->next_input_byte = work_buffer + cinfo->src->bytes_in_buffer;
   dev_ptr->Get(work_buffer + cinfo->src->bytes_in_buffer);
   cinfo->src->next_input_byte = (JOCTET *)work_buffer;
   cinfo->src->bytes_in_buffer = WORK_BUFFER_SIZE;}}


template< class PixelType, class PixelConverter >
void bicotiMapperJPEGIJG< PixelType, PixelConverter > :: Read()
{
  JSAMPROW buffer;
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  int i;
  
  if( ( ptr_image == NULL ) || ( ptr_device == NULL ) ) 
    throw bicotiException( "bicotiMapperJPEGIJG", "Read", 6 );
  
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  ptr_device->Open( "rb" );
  byte_count = ptr_device->GetImageSize();
  jpeg_buff_src(&cinfo);
  
  
  
  while ( jpeg_read_header(&cinfo, TRUE) == JPEG_SUSPENDED )
    reload_buffer( ptr_device, work_buffer, &cinfo );
  
  
  
  cinfo.out_color_space = ColorSpaceConvert( image_colorspace );
  
  
  if (scFlag){
    cinfo.scale_num   = scale_num;
    cinfo.scale_denom = scale_den;
  }
  
  if (diFlag) cinfo.dither_mode = DitherModeConvert( dither_mode );
  
  jpeg_calc_output_dimensions( &cinfo );

  
  while ( !jpeg_start_decompress( &cinfo ) )
    reload_buffer( ptr_device, work_buffer, &cinfo );
  
  int COLS = cinfo.output_width;
  int ROWS = cinfo.output_height;

  
  
  bicotiPixelRGBA def_pix;
  
  ( ( bicotiImageImplementation2D<PixelType> *)ptr_image )->Resize( COLS, ROWS );

  buffer = (JSAMPROW)malloc(COLS * cinfo.output_components * sizeof(JSAMPLE));
  
  if ( cinfo.output_components == 3)
 	 while ( cinfo.output_scanline < ROWS )
	   {if (jpeg_read_scanlines(&cinfo, &buffer, 1))
	     for( i=0; i<COLS; i++ )
	       {def_pix = bicotiRGBTools :: CreateRGBA( buffer[ 3 * i ], buffer[ 3 * i + 1 ], buffer[ 3 * i + 2 ] );
	       ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->SetPixel( i, cinfo.output_scanline - 1, PixelConverter :: Unconvert( def_pix ) );}	
	   else
      			reload_buffer( ptr_device, work_buffer, &cinfo );}
  else
    while ( cinfo.output_scanline < ROWS )
      {
	if (jpeg_read_scanlines(&cinfo, &buffer, 1))
	  for( i=0; i<COLS; i++ )
	    {def_pix = bicotiRGBTools :: CreateRGBA( buffer[  i ], buffer[  i  ], buffer[ i  ] );
	                (  (   bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->SetPixel( i, cinfo.output_scanline - 1, PixelConverter :: Unconvert( def_pix ) );}
	else
	  reload_buffer( ptr_device, work_buffer, &cinfo );}
  
  while ( !jpeg_finish_decompress(&cinfo) )
    reload_buffer( ptr_device, work_buffer, &cinfo );
  
  delete buffer;
  ptr_device->Close();
}


template< class PixelType, class PixelConverter >
void bicotiMapperJPEGIJG< PixelType,PixelConverter > :: Write()
{ 
  bicotiPixelRGBA def_pix;
  
  JSAMPROW buffer;
  //JSAMPARRAY img_buff; 
  
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  
  int i;
  
  cinfo.err = jpeg_std_error(&jerr);
  ptr_device->Open("wb");
  jpeg_create_compress(&cinfo);
  jpeg_buff_dest(&cinfo);

  
  if( ( ptr_image == NULL ) || ( ptr_device == NULL ) ) 
    throw bicotiException( "bicotiMapperJPEGIJG", "Read", 6 );
  
  
  int COLS = ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetX0Size();
  int ROWS = ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetX1Size();
  int COLORS = 3;//def_pix.GetColorsNumber();
  
  cinfo.image_width = COLS;
  cinfo.image_height = ROWS;
  cinfo.input_components = COLORS;
  cinfo.in_color_space = ColorSpaceConvert( image_colorspace );
  
  
  jpeg_set_defaults(&cinfo);
  
  if (csFlag) jpeg_set_colorspace(&cinfo, ColorSpaceConvert( file_colorspace ) );
  if (qtFlag) jpeg_set_quality(&cinfo,quality,force_baseline);
  
  
  jpeg_start_compress(&cinfo, TRUE);
  
  buffer = (JSAMPROW)malloc(COLS * cinfo.input_components * sizeof(JSAMPLE));
  
  
  if (cinfo.input_components == 3){

    do {
      for( i = 0; ((i < COLS) && (cinfo.next_scanline < ROWS)); i++ )
	{def_pix = PixelConverter :: Convert( ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetPixel( i, cinfo.next_scanline ) );
	buffer[ 3 * i ] = (JSAMPLE)( bicotiRGBTools :: GetRed( def_pix ) );
	buffer[ 3 * i + 1 ] = (JSAMPLE)( bicotiRGBTools :: GetGreen( def_pix ) );
	buffer[ 3 * i + 2 ] = (JSAMPLE)( bicotiRGBTools :: GetBlue( def_pix ) );}
      
      if( !jpeg_write_scanlines(&cinfo, &buffer, 1) )
	{ptr_device->BlockSize() =  WORK_BUFFER_SIZE - cinfo.dest->free_in_buffer;
	ptr_device->Put((const char *)work_buffer);
	cinfo.dest->next_output_byte = (JOCTET *)work_buffer;
	cinfo.dest->free_in_buffer = WORK_BUFFER_SIZE;}
      
      
      } while ( cinfo.next_scanline < cinfo.image_height ); }
  else {
    do {
      for( i = 0; ((i < COLS) && (cinfo.next_scanline < ROWS)); i++ )
	{def_pix = PixelConverter :: Convert( ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetPixel( i, cinfo.next_scanline ) );
	buffer[ i ] = (JSAMPLE)( bicotiRGBTools :: GetBlue( def_pix ) );}
      
      if( !jpeg_write_scanlines(&cinfo, &buffer, 1) )
	{ptr_device->BlockSize() =  WORK_BUFFER_SIZE - cinfo.dest->free_in_buffer;
	ptr_device->Put((const char *)work_buffer);
	cinfo.dest->next_output_byte = (JOCTET *)work_buffer;
	cinfo.dest->free_in_buffer = WORK_BUFFER_SIZE;}

      
    } while ( cinfo.next_scanline < cinfo.image_height ) ;

  }
  jpeg_finish_compress(&cinfo);
  
  if (cinfo.dest->free_in_buffer != WORK_BUFFER_SIZE)
    {ptr_device->BlockSize() =  WORK_BUFFER_SIZE - cinfo.dest->free_in_buffer;
    ptr_device->Put(work_buffer);
    cinfo.dest->next_output_byte = (JOCTET *)work_buffer;
    cinfo.dest->free_in_buffer = WORK_BUFFER_SIZE;}

  
  ptr_device->Close();
}


template< class PixelType, class PixelConverter >
J_COLOR_SPACE bicotiMapperJPEGIJG< PixelType , PixelConverter > :: ColorSpaceConvert( bicotiColorSpace cs )
{
  switch( cs )
    {
    case cs_unknown:{
      return JCS_UNKNOWN;
      break;}
    case cs_grayscale:{
      return JCS_GRAYSCALE;     
      break;} 
    case cs_rgb:{
      return JCS_RGB;     
      break;}
    case cs_YCbCr:{
      return JCS_YCbCr;      
      break;}
    case cs_CMYK:{
      return JCS_CMYK;     
      break;}
    case cs_YCCK:{
      return JCS_YCCK;     
      break;}
    }
}

template< class PixelType, class PixelConverter >
J_DITHER_MODE bicotiMapperJPEGIJG< PixelType , PixelConverter>  :: DitherModeConvert( bicotiDitherMode dm )
{
  switch( dm )
    {
    case dm_none:{
      return JDITHER_NONE;
      break;}
    case dm_ordered:{
      return JDITHER_ORDERED;      
      break;} 
    case dm_floyd_steinberg:{
      return JDITHER_FS;      
      break;}
    }

}
