#include <source/mapper/mapper_tiff_sgi.hpp>
#include <source/pixel/pixel_rgb_tools.hpp>
#include <source/mapper/tmpio.hpp>
extern "C" {
#include <tiffio.h>
}

//-----------------bicotiMappetTIFFSGI-----------------


template< class PixelType, class PixelConverter >
bicotiMapperTIFFSGI< PixelType , PixelConverter > :: bicotiMapperTIFFSGI():bicotiMapperTIFF<PixelType>::bicotiMapperTIFF()
{ptr_device = (bicotiDevice *)NULL;
 ptr_image = (bicotiImageImplementation< PixelType > *)NULL;
}

template< class PixelType, class PixelConverter >
void bicotiMapperTIFFSGI< PixelType , PixelConverter  > :: Read()
{
  char datos[1024], *TMPTIFF;
  int len;
  FILE *tmp;
  TIFF *tif;
  int COLS, ROWS;
  uint32 *raster;

  if( ( ptr_image == NULL ) || ( ptr_device == NULL ) ) 
    throw bicotiException( " bicotiMapperTIFFSGI " ," Read " , 6 );

  ptr_device->Open( "rb" );

  TMPTIFF = tempFileCreate();
  if (tmp = fopen(TMPTIFF,"w"))
   
  ptr_device->BlockSize() = 1;
 
  len = ptr_device->GetImageSize();
  while ( len = ptr_device->Get(datos)) {

   fwrite(datos,1,len,tmp);

  }

 

  fclose(tmp);
  ptr_device->Close();

  tif = TIFFOpen(TMPTIFF,"r") ;
 

  TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &COLS);
 
  TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &ROWS);
 

  bicotiPixelRGBA def_pix;
  
  ( ( bicotiImageImplementation2D<PixelType> *)ptr_image )->Resize( COLS, ROWS );

  raster = (uint32 *) _TIFFmalloc(ROWS * COLS * sizeof(uint32) );

  if (raster != NULL)
  if ( TIFFReadRGBAImage(tif,COLS,ROWS, raster, 0 )) {
      for (int k = 0 ; k < ROWS ; k++)
        for ( int i= 0; i < COLS-1; i++ )
	  {def_pix = bicotiRGBTools :: CreateRGBA( TIFFGetR(raster[ k * COLS + i ]), TIFFGetG(raster[ k * COLS + i ]), TIFFGetB(raster[ k * COLS + i ]) );
	((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->SetPixel(  i ,ROWS - k-1 , PixelConverter :: Unconvert( def_pix ));}
     }
  _TIFFfree(raster);
  TIFFClose(tif) ;
  remove(TMPTIFF);
}


template< class PixelType, class PixelConverter >
void bicotiMapperTIFFSGI< PixelType , PixelConverter  > :: Write()
{

  int *data;//tdata_t data;
  int next_line, column, bcount=0;
  tsize_t data_size;
  tstrip_t next_strip = 0;
  uint32 row_per_strip, row_in_strip=0;
  FILE *tmp;
  TIFF *tif;
  char buff,*TMPTIFF ;



  bicotiPixelRGBA def_pix;

  uint32 i;
  


  if( ( ptr_image == NULL ) || ( ptr_device == NULL ) ) 
    throw bicotiException( " bicotiMapperTIFFSGI " , " Write " , 6 );


  int COLS = ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetX0Size();
  int ROWS = ((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetX1Size();
  int COLORS = 3 ;// def_pix.GetColorsNumber();

 
 
 

  TMPTIFF = tempFileCreate();
  tif = TIFFOpen(TMPTIFF, "w");

 

  TIFFSetField(tif, TIFFTAG_IMAGELENGTH, ROWS );
  TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, COLS);
  TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, COLORS+1);
  TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
  TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
  TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, color_space);
  TIFFSetField(tif, TIFFTAG_COMPRESSION, compression_algorithm );

 

  row_per_strip = TIFFDefaultStripSize(tif,0);
   TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, row_per_strip);

 

  data = (int*) _TIFFmalloc( TIFFStripSize(tif));

   next_line = 0 ;

  while (   next_line < ROWS ) {
    	
	if (row_in_strip < row_per_strip) {
	
	   for (column = 0 ; column < COLS ; column++ ) {
    	     def_pix =  PixelConverter :: Convert(((bicotiImageImplementation2DSimple<PixelType>*)ptr_image)->GetPixel( column , next_line));
    	
    	    i =  bicotiRGBTools :: GetRed( def_pix ) ;
    	    i = i  +   bicotiRGBTools :: GetGreen( def_pix ) * 256;
    	    i = i  +    bicotiRGBTools :: GetBlue( def_pix ) * 65536;
    	    data[row_in_strip * COLS + column ] =  i;
    	    bcount+=4;
    	
    	
    	
        
    	    }
    	
    	
    	    row_in_strip++;
    	    next_line++ ;
    	
        }
 	else {
 	   if (TIFFWriteEncodedStrip(tif, next_strip, data, TIFFStripSize(tif) )) {
 	
 	      next_strip++;
 	      row_in_strip = 0;
 	      bcount = 0;
 	   }
 	   else {
 	      // printf( " BICOTI: TIFF I/O  ERROR\n");
 	      exit(0);
 	   }
    	
 	}
 	
 	    	
    }	

     if (row_in_strip > 0){
          TIFFWriteEncodedStrip(tif, next_strip , data,TIFFVStripSize(tif, row_in_strip+1) );
          // printf("ltima tira %d de %d grabada \n",next_strip,TIFFVStripSize(tif, row_in_strip+1)  ) ;
         }



  TIFFClose(tif);
  ptr_device->Open("wb");
  ptr_device->BlockSize()=1;
  tmp = fopen(TMPTIFF , "r");
  while (fread(&buff,1,1,tmp)) ptr_device->Put(&buff);
  fclose(tmp);
  ptr_device->Close();
  remove(TMPTIFF);

}



