//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _operator_multiple_basic
#define _operator_multiple_basic



#include <source/abstract_image.hpp>



//***********************DECLARATIONS******************************
//--------------------bicotiOperatorMultiple-----------------------
template<class PixelType>
class bicotiOperatorMultiple
{
protected:
  vector< bicotiImageImplementation< PixelType > * > image_imps_list;

public: 
  bicotiOperatorMultiple();

  void Append( bicotiImageImplementation< PixelType > * );
  void Reset( unsigned = 0 );

  void Apply( bicotiPixelOperatorBinary< PixelType > * );
};




//**********************IMPLEMENTATIONS****************************
//-------------------bicotiOperatorMultiple------------------------
template<class PixelType>
bicotiOperatorMultiple< PixelType > :: bicotiOperatorMultiple()
{
};


template<class PixelType>
void bicotiOperatorMultiple< PixelType > :: Append( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  bicotiImageDescriptor *ptr_desc_first, * ptr_desc_src;
  unsigned index;

  if ( image_imps_list.size() != 0 )
    {ptr_desc_src = ptr_ima_imp->CreateImageDescriptor();
    ptr_desc_first = ( image_imps_list[0] )->CreateImageDescriptor();
    
    if ( ptr_desc_src->GetDimension() == ptr_desc_first->GetDimension() )
      {for( index = 0; index < ptr_desc_src->GetDimension(); index++ )
	{if ( ptr_desc_src->Size(index) != ptr_desc_first->Size(index) )
	  {delete ptr_desc_src;
	  delete ptr_desc_first;
	  throw bicotiException( "bicotiOperatorMultiple", "Append", 2 );}}
      delete ptr_desc_src;
      delete ptr_desc_first;}
    else
      {delete ptr_desc_src;
      delete ptr_desc_first;
      throw bicotiException( "bicotiOperatorMultiple", "Append", 2 );}}
  
  image_imps_list.push_back( ptr_ima_imp );
};


template<class PixelType>
void bicotiOperatorMultiple< PixelType > :: Reset( unsigned leave )
{
  while ( image_imps_list.size() > leave )
    image_imps_list.pop_back();
};


template<class PixelType>
void bicotiOperatorMultiple< PixelType > :: Apply( bicotiPixelOperatorBinary< PixelType > * ptr_pix_oper )
{
  unsigned index;
  bicotiImageIterator<PixelType> ** ptr_iters;

  if ( image_imps_list.size() < 2 )
    throw bicotiException( "bicotiOperatorMultiple", "Apply", 5 );
  
  ptr_iters = new bicotiImageIterator< PixelType >* [ image_imps_list.size() ]; 
  
  for( index = 0; index < image_imps_list.size(); index++ )
    {ptr_iters[ index ] = ( image_imps_list[ index ] )->CreateIterator();
    ( ptr_iters[ index ] )->First();}
  
  do
    {for( index = 1; index < image_imps_list.size(); index++ )
      {**ptr_iters[ 0 ] = ( *ptr_pix_oper )( **ptr_iters[ 0 ], **ptr_iters[ index ] );
      (*ptr_iters[ index ])++;}
    (*ptr_iters[ 0 ])++;} while( !( ptr_iters[ 0 ]->IsDone() ) );
  
  for(index = 0; index < image_imps_list.size(); index++)
    delete ptr_iters[ index ]; 
  delete [] ptr_iters;
};    



#endif
