//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _pixel_implementation_hsv
#define _pixel_implementation_hsv

#include <source/pixel/pixel_implementation_basic.hpp>



//**********************DECLARATIONS*******************************
template< class ColorType, class PrecisionType = ColorType > 
class bicotiHSV
{
private:
  ColorType hue;
  ColorType saturation;
  ColorType value;

  bool undefined;

public:
  bicotiHSV( const ColorType = 0, const ColorType = 0, const ColorType = 0 ); 
  bicotiHSV( const bicotiHSV< ColorType, PrecisionType >& );
  
  void operator=( const bicotiHSV< ColorType, PrecisionType >& );
  
  bool operator==( const bicotiHSV< ColorType, PrecisionType >& );

  bicotiHSV< ColorType, PrecisionType > operator+( double );
  bicotiHSV< ColorType, PrecisionType > operator-( double );
  bicotiHSV< ColorType, PrecisionType > operator*( double );
  bicotiHSV< ColorType, PrecisionType > operator/( double );

  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );

  bicotiHSV< ColorType, PrecisionType > operator+( const bicotiHSV< ColorType, PrecisionType >& );
  bicotiHSV< ColorType, PrecisionType > operator-( const bicotiHSV< ColorType, PrecisionType >& );
  bicotiHSV< ColorType, PrecisionType > operator*( const bicotiHSV< ColorType, PrecisionType >& );
  bicotiHSV< ColorType, PrecisionType > operator/( const bicotiHSV< ColorType, PrecisionType >& );

  void operator+=( const bicotiHSV< ColorType, PrecisionType >& );
  void operator-=( const bicotiHSV< ColorType, PrecisionType >& );
  void operator*=( const bicotiHSV< ColorType, PrecisionType >& );
  void operator/=( const bicotiHSV< ColorType, PrecisionType >& );

  ColorType& operator() ( int );
  unsigned GetColorsNumber();

  ColorType& Hue();
  ColorType& Saturation();
  ColorType& Value();

  bool& Undefined();

  operator bicotiHSV< PrecisionType, ColorType >();
};




//*********************IMPLEMENTATIONS*****************************
template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > :: bicotiHSV( const ColorType h, const ColorType s, const ColorType v )
{
  hue = h;
  saturation = s;
  value = v;
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > :: bicotiHSV( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  hue = pixel_src.hue;
  saturation = pixel_src.saturation;
  value = pixel_src.value;
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator=( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  hue = pixel_src.hue;
  saturation = pixel_src.saturation;
  value = pixel_src.value;
};


template< class ColorType, class PrecisionType > 
inline bool bicotiHSV< ColorType, PrecisionType > :: operator==( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  return ( (hue == pixel_src.hue ) && ( saturation == pixel_src.saturation ) && ( value == pixel_src.value ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator+( double scalar )
{
  return( bicotiHSV< ColorType, PrecisionType >( (ColorType)(hue + scalar), (ColorType)(saturation + scalar), (ColorType)(value + scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator-( double scalar )
{
  return( bicotiHSV< ColorType, PrecisionType >( (ColorType)(hue - scalar), (ColorType)(saturation - scalar), (ColorType)(value - scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator*( double scalar )
{
  return( bicotiHSV< ColorType, PrecisionType >( (ColorType)(hue * scalar), (ColorType)(saturation * scalar), (ColorType)(value * scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator/( double scalar )
{
  return( bicotiHSV< ColorType, PrecisionType >( (ColorType)(hue / scalar), (ColorType)(saturation / scalar), (ColorType)(value / scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator+=( double scalar )
{
  hue = (ColorType)( hue + scalar );
  saturation = (ColorType)( saturation + scalar );
  value = (ColorType)( value + scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator-=( double scalar )
{
  hue = (ColorType)( hue - scalar );
  saturation = (ColorType)( saturation - scalar );
  value = (ColorType)( value - scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator*=( double scalar )
{
  hue = (ColorType)( hue * scalar );
  saturation = (ColorType)( saturation * scalar );
  value = (ColorType)( value * scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator/=( double scalar )
{
  hue = (ColorType)( hue / scalar );
  saturation = (ColorType)( saturation / scalar );
  value = (ColorType)( value / scalar );
};


template< class ColorType, class PrecisionType >
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator+( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  bicotiHSV< ColorType, PrecisionType > pixel_res;

  pixel_res.hue = hue + pixel_src.hue;
  pixel_res.saturation = saturation + pixel_src.saturation;
  pixel_res.value = value + pixel_src.value;

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator-( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  bicotiHSV< ColorType, PrecisionType > pixel_res;

  pixel_res.hue = hue - pixel_src.hue;
  pixel_res.saturation = saturation - pixel_src.saturation;
  pixel_res.value = value - pixel_src.value;

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator*( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  bicotiHSV< ColorType, PrecisionType > pixel_res;

  pixel_res.hue = hue * pixel_src.hue;
  pixel_res.saturation = saturation * pixel_src.saturation;
  pixel_res.value = value * pixel_src.value;

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > bicotiHSV< ColorType, PrecisionType > :: operator/( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  bicotiHSV< ColorType, PrecisionType > pixel_res;

  pixel_res.hue = ( ColorType ) (hue / pixel_src.hue);
  pixel_res.saturation = ( ColorType ) (saturation / pixel_src.saturation);
  pixel_res.value = ( ColorType ) (value / pixel_src.value);

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator+=( const bicotiHSV< ColorType, PrecisionType >& pixel_src)
{
  hue = hue + pixel_src.hue;
  saturation = saturation + pixel_src.saturation;
  value = value + pixel_src.value;
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator-=( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  hue = hue - pixel_src.hue;
  saturation = saturation - pixel_src.saturation;
  value = value - pixel_src.value;
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator*=( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  hue = hue * pixel_src.hue;
  saturation = saturation * pixel_src.saturation;
  value = value * pixel_src.value;
};


template< class ColorType, class PrecisionType > 
inline void bicotiHSV< ColorType, PrecisionType > :: operator/=( const bicotiHSV< ColorType, PrecisionType >& pixel_src )
{
  hue = hue / pixel_src.hue;
  saturation = saturation / pixel_src.saturation;
  value = value / pixel_src.value;
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiHSV< ColorType, PrecisionType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( 3 <= index ) )
    throw bicotiException( "bicotiHSV", "operator()", 1 );
#endif
  
  switch ( index ){
  case 0: 
    return hue;
  case 1:
    return saturation;
  case 2:
    return value;}
};


template< class ColorType, class PrecisionType > 
inline unsigned bicotiHSV< ColorType, PrecisionType > :: GetColorsNumber()
{
  return 3;
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiHSV< ColorType, PrecisionType >  :: Hue()
{
  return( hue );
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiHSV< ColorType, PrecisionType >  :: Saturation()
{
  return( saturation );
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiHSV< ColorType, PrecisionType >  :: Value()
{
  return( value );
};


template< class ColorType, class PrecisionType >
inline bool& bicotiHSV< ColorType, PrecisionType >  :: Undefined()
{
  return( undefined );
};


template< class ColorType, class PrecisionType > 
inline bicotiHSV< ColorType, PrecisionType > :: operator bicotiHSV< PrecisionType, ColorType >()
{
  return(  bicotiHSV< PrecisionType, ColorType >( (PrecisionType)hue, (PrecisionType)saturation, (PrecisionType)value ) );
};


template< class ColorType, class PrecisionType >
inline bicotiHSV< ColorType, PrecisionType > fabs( bicotiHSV< ColorType, PrecisionType >& pixel )
{
  bicotiHSV< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Hue() = (ColorType) fabs( pixel.Hue() );
  pixel_temp.Saturation() = (ColorType) fabs( pixel.Saturation() );
  pixel_temp.Value() = (ColorType) fabs( pixel.Value() );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiHSV< ColorType, PrecisionType > sqrt( bicotiHSV< ColorType, PrecisionType >& pixel )
{
  bicotiHSV< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Hue() = (ColorType) sqrt( pixel.Hue() );
  pixel_temp.Saturation() = (ColorType) sqrt( pixel.Saturation() );
  pixel_temp.Value() = (ColorType) sqrt( pixel.Value() );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiHSV< ColorType, PrecisionType > atan( bicotiHSV< ColorType, PrecisionType >& pixel )
{
  bicotiHSV< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Hue() = (ColorType) atan( pixel.Hue() );
  pixel_temp.Saturation() = (ColorType) atan( pixel.Saturation() );
  pixel_temp.Value() = (ColorType) atan( pixel.Value() );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiHSV< ColorType, PrecisionType > log( bicotiHSV< ColorType, PrecisionType >& pixel )
{
  bicotiHSV< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Hue() = (ColorType) log( pixel.Hue() );
  pixel_temp.Saturation() = (ColorType) log( pixel.Saturation() );
  pixel_temp.Value() = (ColorType) log( pixel.Value() );

  return( pixel_temp );
};            

#endif
