//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _pixel_implementation_rgb
#define _pixel_implementation_rgb


#include <source/pixel/pixel_implementation_basic.hpp>



//**********************DECLARATIONS*******************************
template< class ColorType, class PrecisionType = ColorType > 
class bicotiRGB
{
private:
  ColorType red;
  ColorType green;
  ColorType blue;
  
public:
  bicotiRGB( const ColorType = 0, const ColorType = 0, const ColorType = 0 ); 
  bicotiRGB( const bicotiRGB< ColorType, PrecisionType >& );
  
  void operator=( const bicotiRGB< ColorType, PrecisionType >& );
  
  bool operator==( const bicotiRGB< ColorType, PrecisionType >& );

  bicotiRGB< ColorType, PrecisionType > operator+( double );
  bicotiRGB< ColorType, PrecisionType > operator-( double );
  bicotiRGB< ColorType, PrecisionType > operator*( double );
  bicotiRGB< ColorType, PrecisionType > operator/( double );

  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );

  bicotiRGB< ColorType, PrecisionType > operator+( const bicotiRGB< ColorType, PrecisionType >& );
  bicotiRGB< ColorType, PrecisionType > operator-( const bicotiRGB< ColorType, PrecisionType >& );
  bicotiRGB< ColorType, PrecisionType > operator*( const bicotiRGB< ColorType, PrecisionType >& );
  bicotiRGB< ColorType, PrecisionType > operator/( const bicotiRGB< ColorType, PrecisionType >& );

  void operator+=( const bicotiRGB< ColorType, PrecisionType >& );
  void operator-=( const bicotiRGB< ColorType, PrecisionType >& );
  void operator*=( const bicotiRGB< ColorType, PrecisionType >& );
  void operator/=( const bicotiRGB< ColorType, PrecisionType >& );

  ColorType& operator() ( int );
  unsigned GetColorsNumber();

  ColorType& Red() const;
  ColorType& Green() const;
  ColorType& Blue() const;
  
  operator bicotiRGB< PrecisionType, ColorType >();
};




//*********************IMPLEMENTATIONS*****************************
template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > :: bicotiRGB( const ColorType r, const ColorType g, const ColorType b )
{
  red = r;
  green = g;
  blue = b;
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > :: bicotiRGB( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  red = pixel_src.red;
  green = pixel_src.green;
  blue = pixel_src.blue;
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator=( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  red = pixel_src.red;
  green = pixel_src.green;
  blue = pixel_src.blue;
};


template< class ColorType, class PrecisionType > 
inline bool bicotiRGB< ColorType, PrecisionType > :: operator==( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  return ( (red == pixel_src.red ) && ( green == pixel_src.green ) && ( blue == pixel_src.blue ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator+( double scalar )
{
  return( bicotiRGB< ColorType, PrecisionType >( (ColorType)(red + scalar), (ColorType)(green + scalar), (ColorType)(blue + scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator-( double scalar )
{
  return( bicotiRGB< ColorType, PrecisionType >( (ColorType)(red - scalar), (ColorType)(green - scalar), (ColorType)(blue - scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator*( double scalar )
{
  return( bicotiRGB< ColorType, PrecisionType >( (ColorType)(red * scalar), (ColorType)(green * scalar), (ColorType)(blue * scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator/( double scalar )
{
  return( bicotiRGB< ColorType, PrecisionType >( (ColorType)(red / scalar), (ColorType)(green / scalar), (ColorType)(blue / scalar) ) );
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator+=( double scalar )
{
  red = (ColorType)( red + scalar );
  green = (ColorType)( green + scalar );
  blue = (ColorType)( blue + scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator-=( double scalar )
{
  red = (ColorType)( red - scalar );
  green = (ColorType)( green - scalar );
  blue = (ColorType)( blue - scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator*=( double scalar )
{
  red = (ColorType)( red * scalar );
  green = (ColorType)( green * scalar );
  blue = (ColorType)( blue * scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator/=( double scalar )
{
  red = (ColorType)( red / scalar );
  green = (ColorType)( green / scalar );
  blue = (ColorType)( blue / scalar );
};


template< class ColorType, class PrecisionType >
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator+( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  bicotiRGB< ColorType, PrecisionType > pixel_res;

  pixel_res.red = red + pixel_src.red;
  pixel_res.green = green + pixel_src.green;
  pixel_res.blue = blue + pixel_src.blue;

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator-( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  bicotiRGB< ColorType, PrecisionType > pixel_res;

  pixel_res.red = red - pixel_src.red;
  pixel_res.green = green - pixel_src.green;
  pixel_res.blue = blue - pixel_src.blue;

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator*( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  bicotiRGB< ColorType, PrecisionType > pixel_res;

  pixel_res.red = red * pixel_src.red;
  pixel_res.green = green * pixel_src.green;
  pixel_res.blue = blue * pixel_src.blue;

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > bicotiRGB< ColorType, PrecisionType > :: operator/( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  bicotiRGB< ColorType, PrecisionType > pixel_res;

  pixel_res.red = ( ColorType ) (red / pixel_src.red);
  pixel_res.green = ( ColorType ) (green / pixel_src.green);
  pixel_res.blue = ( ColorType ) (blue / pixel_src.blue);

  return( pixel_res );
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator+=( const bicotiRGB< ColorType, PrecisionType >& pixel_src)
{
  red = red + pixel_src.red;
  green = green + pixel_src.green;
  blue = blue + pixel_src.blue;
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator-=( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  red = red - pixel_src.red;
  green = green - pixel_src.green;
  blue = blue - pixel_src.blue;
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator*=( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  red = red * pixel_src.red;
  green = green * pixel_src.green;
  blue = blue * pixel_src.blue;
};


template< class ColorType, class PrecisionType > 
inline void bicotiRGB< ColorType, PrecisionType > :: operator/=( const bicotiRGB< ColorType, PrecisionType >& pixel_src )
{
  red = red / pixel_src.red;
  green = green / pixel_src.green;
  blue = blue / pixel_src.blue;
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiRGB< ColorType, PrecisionType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( 3 <= index ) )
    throw bicotiException( "bicotiRGB", "operator()", 1 );
#endif
  
  switch ( index ){
  case 0: 
    return red;
  case 1:
    return green;
  case 2:
    return blue;}
};


template< class ColorType, class PrecisionType > 
inline unsigned bicotiRGB< ColorType, PrecisionType > :: GetColorsNumber()
{
  return 3;
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiRGB< ColorType, PrecisionType >  :: Red() const
{
  return( red );
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiRGB< ColorType, PrecisionType >  :: Green() const
{
  return( green );
};


template< class ColorType, class PrecisionType >
inline ColorType& bicotiRGB< ColorType, PrecisionType >  :: Blue() const
{
  return( blue );
};


template< class ColorType, class PrecisionType > 
inline bicotiRGB< ColorType, PrecisionType > :: operator bicotiRGB< PrecisionType, ColorType >()
{
  return(  bicotiRGB< PrecisionType, ColorType >( (PrecisionType)red, (PrecisionType)green, (PrecisionType)blue ) );
};


template< class ColorType, class PrecisionType >
inline bicotiRGB< ColorType, PrecisionType > fabs( bicotiRGB< ColorType, PrecisionType >& pixel )
{
  bicotiRGB< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Red() = (ColorType) fabs( pixel.Red() );
  pixel_temp.Green() = (ColorType) fabs( pixel.Green() );
  pixel_temp.Blue() = (ColorType) fabs( pixel.Blue() );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiRGB< ColorType, PrecisionType > sqrt( bicotiRGB< ColorType, PrecisionType >& pixel )
{
  bicotiRGB< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Red() = (ColorType) sqrt( pixel.Red() );
  pixel_temp.Green() = (ColorType) sqrt( pixel.Green() );
  pixel_temp.Blue() = (ColorType) sqrt( pixel.Blue() );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiRGB< ColorType, PrecisionType > atan( bicotiRGB< ColorType, PrecisionType >& pixel )
{
  bicotiRGB< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Red() = (ColorType) atan( pixel.Red() );
  pixel_temp.Green() = (ColorType) atan( pixel.Green() );
  pixel_temp.Blue() = (ColorType) atan( pixel.Blue() );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiRGB< ColorType, PrecisionType > log( bicotiRGB< ColorType, PrecisionType >& pixel )
{
  bicotiRGB< ColorType, PrecisionType > pixel_temp;

  pixel_temp.Red() = (ColorType) log( pixel.Red() );
  pixel_temp.Green() = (ColorType) log( pixel.Green() );
  pixel_temp.Blue() = (ColorType) log( pixel.Blue() );

  return( pixel_temp );
};            

#endif
