//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _pixel_transformation_rgb_hsv
#define _pixel_transformation_rgb_hsv

#include <source/pixel/pixel_transformation_basic.hpp>



//***********************DECLARATIONS******************************
//--------------bicotiPixelTransformationRGBToHSV------------------
template< class ColorSource, class ColorTarget = ColorSource, class PrecisionSource = ColorSource, class PrecisionTarget = ColorTarget >
class bicotiPixelTransformationRGBToHSV : public bicotiPixelTransformation< bicotiRGB< ColorSource, PrecisionSource >, bicotiHSV< ColorTarget, PrecisionTarget > >
{
private:
  ColorTarget Maximum( ColorSource, ColorSource, ColorSource );

public:
  bicotiHSV< ColorTarget, PrecisionTarget > operator() ( bicotiRGB< ColorSource, PrecisionSource >& );
};



//--------------bicotiPixelTransformationHSVToRGB------------------
template< class ColorSource, class ColorTarget = ColorSource, class PrecisionSource = ColorSource, class PrecisionTarget = ColorTarget >
class bicotiPixelTransformationHSVToRGB : public bicotiPixelTransformation< bicotiHSV< ColorSource, PrecisionSource >, bicotiRGB< ColorTarget, PrecisionTarget > >
{
public:
  bicotiRGB< ColorTarget, PrecisionTarget > operator() ( bicotiHSV< ColorSource, PrecisionSource >& );
};




//**********************IMPLEMENTATIONS****************************
//----------------bicotiPixelTransformationRGBToHSV----------------
template< class ColorSource, class ColorTarget, class PrecisionSource, class PrecisionTarget >
ColorTarget bicotiPixelTransformationRGBToHSV< ColorSource, ColorTarget, PrecisionSource, PrecisionTarget > :: Maximum( ColorSource red, ColorSource green, ColorSource blue )
{
  if ( red > green )
    {if ( red > blue ) 
      return( (ColorTarget)red );
    else
      return( (ColorTarget)blue );}
  else
    {if ( green > blue )
      return( (ColorTarget)green );
    else
      return( (ColorTarget)blue );}
};


template< class ColorSource, class ColorTarget, class PrecisionSource, class PrecisionTarget >
bicotiHSV< ColorTarget, PrecisionTarget > bicotiPixelTransformationRGBToHSV< ColorSource, ColorTarget, PrecisionSource, PrecisionTarget > :: operator() ( bicotiRGB< ColorSource, PrecisionSource >& pixel_rgb )
{
  bicotiHSV< ColorTarget, PrecisionTarget > pixel_hsv;
  pixel_hsv.Value() = Maximum( pixel_rgb.Red(), pixel_rgb.Green(), pixel_rgb.Blue() );

  REAL phi = (REAL)( (REAL)(pixel_rgb.Green()) - (REAL)(pixel_rgb.Blue()) );
  REAL psi = (REAL)( (REAL)(pixel_rgb.Green()) + (REAL)(pixel_rgb.Blue()) );
  REAL lambda = (REAL)( (REAL)(pixel_rgb.Red()) - (REAL)(0.5 * psi) );

  pixel_hsv.Saturation() = (ColorTarget)sqrt( lambda * lambda + 0.75 * phi * phi );
  
  if ( lambda > 0 )
    {if ( phi >= 0 )
      pixel_hsv.Hue() = (ColorTarget)( 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );
    else
      pixel_hsv.Hue() = (ColorTarget)( 360.0 + 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );}
  else
    {if ( lambda < 0 )
      pixel_hsv.Hue() = 180 + (ColorTarget)( 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );
    else
      {if ( phi > 0 )
	pixel_hsv.Hue() = 90;
      else
	{if ( phi < 0 )
	  pixel_hsv.Hue() = 270;
	else
	  pixel_hsv.Undefined() = true;}}}
  
  return( pixel_hsv );
};



//----------------bicotiPixelTransformationHSVToRGB----------------
template< class ColorSource, class ColorTarget, class PrecisionSource, class PrecisionTarget >
bicotiRGB< ColorTarget, PrecisionTarget > bicotiPixelTransformationHSVToRGB< ColorSource, ColorTarget, PrecisionSource, PrecisionTarget > :: operator() ( bicotiHSV< ColorSource, PrecisionSource >& pixel_hsv )
{
  bicotiRGB< ColorTarget, PrecisionTarget > pixel_rgb;

  if( pixel_hsv.Saturation() == 0 )
    {pixel_rgb.Red() = (ColorTarget)pixel_hsv.Value();
    pixel_rgb.Green() = (ColorTarget)pixel_hsv.Value();
    pixel_rgb.Blue() = (ColorTarget)pixel_hsv.Value();}
  else
    {REAL phi = (REAL)( pixel_hsv.Saturation() * cos( pi * pixel_hsv.Hue() / 180 ) );
    REAL psi = (REAL)( pixel_hsv.Saturation() * sin( pi * pixel_hsv.Hue() / 180 ) / sqrt(3) );
    
    if( ( 60 <= pixel_hsv.Hue() ) && ( pixel_hsv.Hue() <= 180 ) )
      {pixel_rgb.Red() = (ColorTarget)( pixel_hsv.Value() + phi - psi );
      pixel_rgb.Green() = (ColorTarget)pixel_hsv.Value();
      pixel_rgb.Blue() = (ColorTarget)( pixel_hsv.Value() - 2 * psi );}
    else
      {if( ( 180 < pixel_hsv.Hue() ) && ( pixel_hsv.Hue() <= 300 ) )
	{pixel_rgb.Red() = (ColorTarget)( pixel_hsv.Value() + phi + psi );
	pixel_rgb.Green() = (ColorTarget)( pixel_hsv.Value() + 2 * psi );
	pixel_rgb.Blue() = (ColorTarget)pixel_hsv.Value();}
      else
	{pixel_rgb.Red() = (ColorTarget)pixel_hsv.Value();
	pixel_rgb.Green() = (ColorTarget)( pixel_hsv.Value() + psi - phi );
	pixel_rgb.Blue() = (ColorTarget)( pixel_hsv.Value() - phi - psi);}}}
  
  return( pixel_rgb );
};

#endif
