//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _property_histogram_extended
#define _property_histogram_extended


#include <source/property/property_histogram_basic.hpp>


//---------------DECLARATIONS-----------------
//-------PropertyHistogramExtended------------
template< class CoordType, class ColorType, class PixelType >
class bicotiHistogramEqualizator;

template<class CoordType, class ColorType, class PixelType=ColorType>
class bicotiPropertyHistogramExtended : public bicotiPropertyHistogram<ColorType, PixelType>
{
  friend class bicotiHistogramEqualizator< CoordType, ColorType, PixelType >;

private:
  unsigned dexels_number;
  bicotiDexelHistogram<CoordType, ColorType, PixelType> ** ptr_dexels;
  
public:
  bicotiPropertyHistogramExtended(bicotiImageImplementation<PixelType>*, const vector< bicotiPixelAttributeSelector< ColorType, PixelType > * >& );
  ~bicotiPropertyHistogramExtended();
  
  void operator=(bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType>&);
  
  void UpdateImageImplementation();
  
  virtual void First( unsigned = 0 );
  virtual void Next( unsigned = 0 );
  virtual ColorType GetCurrentColor( unsigned = 0 );
  virtual unsigned GetCurrentPointsNumber( unsigned = 0 );
  virtual bool IsDone( unsigned = 0 );

  virtual unsigned GetHistogramsNumber();
  bicotiDexelHistogram<CoordType, ColorType, PixelType> * GetDexel( unsigned = 0 );
};


//-------------IMPLEMENTATIONS----------------
//-------bicotiPropertyHistogramExtended------------
template<class CoordType, class ColorType, class PixelType>
bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: bicotiPropertyHistogramExtended(bicotiImageImplementation<PixelType>* ptr_ima_imp, const vector< bicotiPixelAttributeSelector< ColorType, PixelType > * >& color_sel_list)
{
  ptr_image_imp = ptr_ima_imp;
  dexels_number = color_sel_list.size();
  ptr_dexels = new bicotiDexelHistogram<CoordType, ColorType, PixelType>* [ dexels_number ];
  
  unsigned index;
  
  for( index = 0; index < dexels_number; index++ )
    ptr_dexels[index] = new bicotiDexelHistogram<CoordType, ColorType, PixelType>( ptr_image_imp, color_sel_list[ index ] );
};


template<class CoordType, class ColorType, class PixelType>
bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: ~bicotiPropertyHistogramExtended()
{
  for(unsigned index=0; index < dexels_number; index++)
    delete ptr_dexels[index];

  delete [] ptr_dexels;
};


template<class CoordType, class ColorType, class PixelType>
void bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: operator=(bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType>& proper_histo_extend_src)
{
  throw bicotiException( "bicotiPropertyHistogramExtended", "operator=", 0 );
}; 


template<class CoordType, class ColorType, class PixelType>
void bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: UpdateImageImplementation()
{
  throw bicotiException( "bicotiPropertyHistogramExtended", "UpdateImageImplementation()", 0 );
}; 


template<class CoordType, class ColorType, class PixelType>
void bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: First(unsigned col_index)
{
  if ( col_index >= dexels_number )
    throw bicotiException( "bicotiPropertyHistogramExtended", "First()", 1 );

  ptr_dexels[col_index]->First();
};


template<class CoordType, class ColorType, class PixelType>
void bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: Next(unsigned col_index)
{
  if ( col_index >= dexels_number )
    throw bicotiException( "bicotiPropertyHistogramExtended", "Next()", 1 );

  ptr_dexels[col_index]->Next();
};


template<class CoordType, class ColorType, class PixelType>
ColorType bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: GetCurrentColor(unsigned col_index)
{
  if ( col_index >= dexels_number )
    throw bicotiException( "bicotiPropertyHistogramExtended", "GetCurrentColor()", 1 );

  return( ptr_dexels[col_index]->GetCurrentColor() );
};


template<class CoordType, class ColorType, class PixelType>
unsigned bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: GetCurrentPointsNumber(unsigned col_index)
{
  if ( col_index >= dexels_number )
    throw bicotiException( "bicotiPropertyHistogramExtended", "GetCurrentPointsNumber()", 1 );
  
  return( ptr_dexels[col_index]->GetCurrentPointsNumber() );
};


template<class CoordType, class ColorType, class PixelType>
bool bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: IsDone(unsigned col_index)
{
  if ( col_index >= dexels_number )
    throw bicotiException( "bicotiPropertyHistogramExtended", "IsDone()", 1 );

  return( ptr_dexels[col_index]->IsDone() );
};


template<class CoordType, class ColorType, class PixelType>
unsigned  bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: GetHistogramsNumber()
{
  return ( dexels_number );
};


template<class CoordType, class ColorType, class PixelType>
bicotiDexelHistogram<CoordType, ColorType, PixelType> * bicotiPropertyHistogramExtended<CoordType, ColorType, PixelType> :: GetDexel( unsigned col_index )
{
  return( ptr_dexels[ col_index ] );
}


#endif
