//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _properties_statistical_basic
#define _properties_statistical_basic


#include <source/property/property_basic.hpp>


//---------------DECLARATIONS-----------------
//-------bicotiPropertyStatistical------------
template< class CoordType, class ColorType, class PixelType=ColorType, class PrecisionType=PixelType >
class bicotiPropertyStatistical : public bicotiProperty< PixelType >
{
protected:
  unsigned dexels_number;
  bicotiDexelStatistical<CoordType, ColorType, PixelType> ** ptr_dexels;

  PixelType mean_value;

  void UpdateMeanValue();

public:
  bicotiPropertyStatistical( bicotiImageImplementation< PixelType > *, const vector< bicotiPixelAttributeSelector< ColorType, PixelType > * >& color_sel_list );
  ~bicotiPropertyStatistical();

  void UpdateImageImplementation();

  ColorType& MaximumValue( unsigned = 0 );
  ColorType& MinimumValue( unsigned = 0 );

  PixelType MeanValue();
};


//-------------IMPLEMENTATIONS----------------
//-------bicotiPropertyStatistical------------
template< class CoordType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyStatistical< CoordType, ColorType, PixelType, PrecisionType > :: bicotiPropertyStatistical(bicotiImageImplementation< PixelType >* ptr_ima_imp, const vector< bicotiPixelAttributeSelector< ColorType, PixelType > * >& color_sel_list )
{
  ptr_image_imp = ptr_ima_imp;
  dexels_number = color_sel_list.size();
  ptr_dexels = new bicotiDexelStatistical<CoordType, ColorType, PixelType>* [ dexels_number ];
  
  for( unsigned index = 0; index < dexels_number; index++ )
    ptr_dexels[index] = new bicotiDexelStatistical<CoordType, ColorType, PixelType>( ptr_image_imp, color_sel_list[ index ] );

  UpdateMeanValue();
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyStatistical< CoordType, ColorType, PixelType, PrecisionType > :: ~bicotiPropertyStatistical()
{
  for(unsigned index = 0; index < dexels_number; index++)
    delete ptr_dexels[index];

  delete [] ptr_dexels;
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyStatistical<CoordType, ColorType, PixelType, PrecisionType > :: UpdateMeanValue()
{
  PrecisionType pixel_acum;
  pixel_acum *= 0.0;
  unsigned count = 0;
  bicotiImageIterator< PixelType > * ptr_iter;
  ptr_iter = ptr_image_imp->CreateIterator();
  for( ptr_iter->First(); !ptr_iter->IsDone(); (*ptr_iter)++ )
   {pixel_acum += (**ptr_iter);
   count++;}
  pixel_acum /= count;
  mean_value = (PixelType)pixel_acum;
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyStatistical<CoordType, ColorType, PixelType, PrecisionType > :: UpdateImageImplementation()
{
  for( unsigned index = 0; index < dexels_number; index++ )
    ptr_dexels[ index ]->UpdateImageImplementation();

  UpdateMeanValue();
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
ColorType& bicotiPropertyStatistical<CoordType, ColorType, PixelType, PrecisionType > :: MaximumValue( unsigned col_index )
{
  if ( col_index >= dexels_number )
     throw bicotiException( "bicotiPropertyStatistical", "MaximumValue()", 1 );
  
  return ( ptr_dexels[col_index]->MaximumValue() );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
ColorType& bicotiPropertyStatistical<CoordType, ColorType, PixelType, PrecisionType > :: MinimumValue( unsigned col_index )
{
  if ( col_index >= dexels_number )
     throw bicotiException( "bicotiPropertyStatistical", "MinimumValue()", 1 );
  
  return ( ptr_dexels[col_index]->MinimumValue() );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
PixelType bicotiPropertyStatistical<CoordType, ColorType, PixelType, PrecisionType > :: MeanValue()
{
  return( mean_value );
};



#endif           
