//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _transformation_fourier
#define _transformation_fourier



#include <source/transformation/transformation_basic.hpp>



//*************************DECLARATION*****************************
//----------------------bicotiDataContainer------------------------
template< class DataType, class PixelType >
class bicotiDataContainer
{
private:
  vector< DataType > data;
  unsigned size;
  
public:
  bicotiDataContainer( int = 0 );
  bicotiDataContainer( const bicotiDataContainer< DataType, PixelType >& );
  
  void operator=( const bicotiDataContainer< DataType, PixelType >& );
  
  bicotiDataContainer< DataType, PixelType > operator+( const bicotiDataContainer< DataType, PixelType >& );
  bicotiDataContainer< DataType, PixelType > operator-( const bicotiDataContainer< DataType, PixelType >& );
  bicotiDataContainer< DataType, PixelType > operator*( const bicotiDataContainer< DataType, PixelType >& );
  bicotiDataContainer< DataType, PixelType > operator/( const bicotiDataContainer< DataType, PixelType >& );
  
  bicotiDataContainer< DataType, PixelType > operator*( bicotiPixelComplex< PixelType > );
  
  DataType& operator() ( int );

  unsigned GetSize();
 
  void One();
};



//------------------------------bicotiTransformationFourier--------------------------------
template< class PixelType, class DataType >
void FFT( bicotiDataContainer< DataType, PixelType >&, bicotiDataContainer< DataType, PixelType >&, bool = true );


template< class PixelType >
class bicotiTransformationFourier : public bicotiTransformation< PixelType, bicotiPixelComplex< PixelType > >
{
public:
  virtual void SetDirectExtrapolationStrategy( bicotiImageExtrapolationStrategy< PixelType > * ) = 0;  
  virtual void SetInverseExtrapolationStrategy( bicotiImageExtrapolationStrategy< bicotiPixelComplex< PixelType > > * ) = 0;  
  
  virtual void Transform() = 0;
  virtual void Antitransform() = 0;
};




//***********************IMPLEMENTATIONS***************************
//---------------------bicotiDataContainer-------------------------
template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > :: bicotiDataContainer( int size_in )
{
  if( size < 0 )
    throw bicotiException( "bicotiDataContainer", "Contructor", 4 );
  
  size = size_in;
  data = vector< DataType >( size );
};


template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > :: bicotiDataContainer( const bicotiDataContainer< DataType, PixelType >& cont_src )
{
  data = cont_src.data;
  size = cont_src.size;
};


template< class DataType, class PixelType >
void bicotiDataContainer< DataType, PixelType > :: operator=( const bicotiDataContainer< DataType, PixelType >& cont_src )
{
  data = cont_src.data;
  size = cont_src.size;
};


template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > bicotiDataContainer< DataType, PixelType > :: operator+( const bicotiDataContainer< DataType, PixelType >& cont_src )
{
  if( size != cont_src.size )
    throw bicotiException( "bicotiDataContainer", "operator+", 2 );

  bicotiDataContainer< DataType, PixelType > cont_result( cont_src );
  
  for( unsigned index = 0; index < size; index++ )
    cont_result.data[ index ] = data[ index ] + cont_result.data[ index ];

  return( cont_result );
};


template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > bicotiDataContainer< DataType, PixelType > :: operator-( const bicotiDataContainer< DataType, PixelType >& cont_src )
{
  if( size != cont_src.size )
    throw bicotiException( "bicotiDataContainer", "operator-", 2 );

  bicotiDataContainer< DataType, PixelType > cont_result( cont_src );
  
  for( unsigned index = 0; index < size; index++ )
    cont_result.data[ index ] = data[ index ] - cont_result.data[ index ];
  
  return( cont_result );
};


template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > bicotiDataContainer< DataType, PixelType > :: operator*( const bicotiDataContainer< DataType, PixelType >& cont_src )
{
  if( size != cont_src.size )
    throw bicotiException( "bicotiDataContainer", "operator*", 2 );
  
  bicotiDataContainer< DataType, PixelType > cont_result( cont_src );
  
  for( unsigned index = 0; index < size; index++ )
    cont_result.data[ index ] = data[ index ] * cont_result.data[ index ];
  
  return( cont_result );
};


template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > bicotiDataContainer< DataType, PixelType > :: operator/( const bicotiDataContainer< DataType, PixelType >& cont_src )
{
  if( size != cont_src.size )
    throw bicotiException( "bicotiDataContainer", "operator/", 2 );

  bicotiDataContainer< DataType, PixelType > cont_result( cont_src );
  
  for( unsigned index = 0; index < size; index++ )
    cont_result.data[ index ] = data[ index ] / cont_result.data[ index ];
  
  return( cont_result );
};


template< class DataType, class PixelType >
bicotiDataContainer< DataType, PixelType > bicotiDataContainer< DataType, PixelType > :: operator*( bicotiPixelComplex< PixelType > scalar )
{
  bicotiDataContainer< DataType, PixelType > cont_result( size );
  
  for( unsigned index = 0; index < size; index++ )
    cont_result.data[ index ] = data[ index ] * scalar;
  
  return( cont_result );
};


template< class DataType, class PixelType >
inline DataType& bicotiDataContainer< DataType, PixelType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if( ( index < 0 ) || ( size <= index ) )
    throw bicotiException( "bicotiDataContainer", "operator()", 2 );
#endif
  
  return( data[ index ] );
};


template< class DataType, class PixelType >
unsigned bicotiDataContainer< DataType, PixelType > :: GetSize()
{
  return( size );
};


template< class DataType, class PixelType >
void bicotiDataContainer< DataType, PixelType > :: One()
{
  for( unsigned index = 0; index < size; index++ )
    data[ index ].One();
};



//------------------bicotiTransformationFourier--------------------
template< class PixelType, class DataType >
void FFT( bicotiDataContainer< DataType, PixelType >& data_targ, bicotiDataContainer< DataType, PixelType >& data_src, bool direct )
{
  bicotiPixelComplex< PixelType > W;
  DataType A, B;
  A = data_src( 0 );
  B = data_src( 0 );

  unsigned i, j, k, index, N, NU, Ndiv2, pow2, pow2m1;
  
  data_targ = data_src;
  for( NU = 0, N = data_src.GetSize(); N > 1; N /= 2, NU++ ){}
  N = data_src.GetSize();  
  
  //*****Se ordena el vector -- bit reversal -- ****
  Ndiv2 = N / 2;
  j = 1;
  for( i = 1; i < N; i++ )
    {if( i < j )
      {B = data_targ( j - 1 );
      data_targ( j - 1 ) = data_targ( i - 1 );
      data_targ( i - 1 ) = B;}

    k = Ndiv2;
    
    while( k < j )
      {j = j - k;
      k = k / 2;}
    
    j = j + k;}
  
  //******Calculo de la FFT ****
  double angle;
  double exponent;
  if( direct )
    exponent = -1;
  else
    exponent = 1;
  
  pow2 = 1;
  
  for( k = 1; k <= NU; k++ )
    {pow2 *= 2;
    pow2m1 = pow2 / 2;
    angle = exponent * (pi / (double)pow2m1);
    
    A.One();
    W.Real() = cos( angle );
    W.Imaginary() = sin( angle );

    for( j = 0; j < pow2m1; j++ )
      {for( i = j; i < N; i += pow2 )
	{index = i + pow2m1;
	B = data_targ( index ) * A;
	data_targ( index ) = data_targ( i ) - B;
	data_targ( i ) = data_targ( i ) + B;}
      A = A * W;}}

  if( !direct )
    {bicotiPixelComplex< PixelType > aux_comp;
    PixelType pix_aux;
    aux_comp.One();
    pix_aux *= 0;
    pix_aux += N;
    aux_comp /= pix_aux;
    data_targ = data_targ * aux_comp;}
};




#endif   
