//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _transformation_fourier_2D
#define _transformation_fourier_2D



#include <source/transformation/transformation_fourier.hpp>



//*************************DECLARATION*****************************
//------------------bicotiTransformationFourier2D------------------
template< class PixelType >
class bicotiTransformationFourier2D : public bicotiTransformationFourier< PixelType >
{
private:
  bicotiImageImplementation2DSimple< PixelType > * ptr_src;
  bicotiImageImplementation2DSimple< bicotiPixelComplex< PixelType > > * ptr_dest;
  
  bicotiImageExtrapolationStrategy2D< PixelType > * ptr_extrapol_strat_src;
  bicotiImageExtrapolationStrategy2D< bicotiPixelComplex< PixelType > > * ptr_extrapol_strat_dest;
  
public:
  bicotiTransformationFourier2D( bicotiImageImplementation2D< bicotiPixelComplex< PixelType > > *, bicotiImageImplementation2D< PixelType > * );
  bicotiTransformationFourier2D();
  ~bicotiTransformationFourier2D(){};
  
  void SetDirectExtrapolationStrategy( bicotiImageExtrapolationStrategy< PixelType > * );  
  void SetInverseExtrapolationStrategy( bicotiImageExtrapolationStrategy< bicotiPixelComplex< PixelType > > * );  

  void Transform();
  void Antitransform();

  bicotiImageImplementation2D< PixelType > * GetTransformedModule();
  bicotiImageImplementation2D< PixelType > * GetTransformedArgument();
};




//***********************IMPLEMENTATIONS***************************
//----------------bicotiTransformationFourier2D--------------------
template< class PixelType >
bicotiTransformationFourier2D< PixelType > :: bicotiTransformationFourier2D( bicotiImageImplementation2D< bicotiPixelComplex< PixelType > > * ptr_ima_dest, bicotiImageImplementation2D< PixelType > * ptr_ima_src )
{
  SetImplementations( ptr_ima_dest, ptr_ima_src );
  
  ptr_src = (bicotiImageImplementation2DSimple< PixelType > *)ptr_real_image;
  ptr_dest = (bicotiImageImplementation2DSimple< bicotiPixelComplex< PixelType > > *)ptr_transformed_image;

  ptr_extrapol_strat_src = NULL;
  ptr_extrapol_strat_dest = NULL;
};


template< class PixelType >
bicotiTransformationFourier2D< PixelType > :: bicotiTransformationFourier2D()
{
  ptr_src = NULL;
  ptr_dest = NULL;

  ptr_extrapol_strat_src = NULL;
  ptr_extrapol_strat_dest = NULL;
};


template< class PixelType >
void bicotiTransformationFourier2D< PixelType > :: SetDirectExtrapolationStrategy( bicotiImageExtrapolationStrategy< PixelType > * ptr_strat )
{
  ptr_extrapol_strat_src = (  bicotiImageExtrapolationStrategy2D< PixelType > * ) ptr_strat;
};


template< class PixelType >
void bicotiTransformationFourier2D< PixelType > :: SetInverseExtrapolationStrategy( bicotiImageExtrapolationStrategy< bicotiPixelComplex < PixelType > > * ptr_strat )
{
  ptr_extrapol_strat_dest = (  bicotiImageExtrapolationStrategy2D< bicotiPixelComplex< PixelType > > * ) ptr_strat;
};


template< class PixelType >
void bicotiTransformationFourier2D< PixelType > :: Transform()
{
  unsigned x0_dim, x1_dim;

  ptr_src = (bicotiImageImplementation2DSimple< PixelType > *)ptr_real_image;
  ptr_dest = (bicotiImageImplementation2DSimple< bicotiPixelComplex< PixelType > > *)ptr_transformed_image;
  
  for( x0_dim = 1 ; x0_dim < ptr_src->GetX0Size(); x0_dim *= 2 );
  for( x1_dim = 1 ; x1_dim < ptr_src->GetX1Size(); x1_dim *= 2 );
  
  bicotiDataContainer< bicotiDataContainer< bicotiPixelComplex< PixelType >, PixelType >, PixelType > data_real( x0_dim );
  bicotiDataContainer< bicotiDataContainer< bicotiPixelComplex< PixelType >, PixelType >, PixelType > data_transformed( x0_dim );
  
  bicotiImageIterator< PixelType > * ptr_iter_src;
  bicotiImageIterator< bicotiPixelComplex< PixelType > > * ptr_iter_dest;
  unsigned index_0, index_1;
  PixelType zero_pix;
  zero_pix *= 0;
  
  for( index_0 = 0; index_0 < x0_dim; index_0++ )
    data_real( index_0 ) = bicotiDataContainer< bicotiPixelComplex< PixelType >, PixelType >( x1_dim );
  if( ( x0_dim != ptr_src->GetX0Size() ) || ( x1_dim != ptr_src->GetX1Size() ) )
    {if( ptr_extrapol_strat_src == NULL )
      throw bicotiException( "bicotiTransformationFourier2D", "Transform", 6 );
    ptr_extrapol_strat_src->SetImplementation( ptr_src );
    for( index_1 = 0; index_1 < x1_dim; index_1++ )
      for( index_0 = 0; index_0 < x0_dim; index_0++ )
	{data_real( index_0 )( index_1 ).Real() = (*ptr_extrapol_strat_src)( index_0, index_1 );
	data_real( index_0 )( index_1 ).Imaginary() = zero_pix;}}
  else
    {ptr_iter_src = ptr_src->CreateIterator();
    ptr_iter_src->First();
    for( index_1 = 0; index_1 < x1_dim; index_1++ )
      for( index_0 = 0; index_0 < x0_dim; index_0++ )
	{data_real( index_0 )( index_1 ).Real() = **ptr_iter_src;
	(*ptr_iter_src)++;}
    delete ptr_iter_src;}
  
  for( index_0 = 0; index_0 < x0_dim; index_0++ )
    FFT( data_transformed( index_0 ), data_real( index_0 ), true );
  
  FFT( data_real, data_transformed, true );
  
  ptr_dest->Resize( x0_dim, x1_dim );
  
  ptr_iter_dest = ptr_dest->CreateIterator();  
  for( index_1 = 0; index_1 < x1_dim; index_1++ )
    for( index_0 = 0; index_0 < x0_dim; index_0++ )
      {**ptr_iter_dest = data_real( index_0 )( index_1 );
      (*ptr_iter_dest)++;}  
  delete ptr_iter_dest;
};


template< class PixelType >
void bicotiTransformationFourier2D< PixelType > :: Antitransform()
{
  unsigned x0_dim, x1_dim;

  ptr_src = (bicotiImageImplementation2DSimple< PixelType > *)ptr_real_image;
  ptr_dest = (bicotiImageImplementation2DSimple< bicotiPixelComplex< PixelType > > *)ptr_transformed_image;
    
  for( x0_dim = 1 ; x0_dim < ptr_dest->GetX0Size(); x0_dim *= 2 );
  for( x1_dim = 1 ; x1_dim < ptr_dest->GetX1Size(); x1_dim *= 2 );
  
  bicotiDataContainer< bicotiDataContainer< bicotiPixelComplex< PixelType >, PixelType >, PixelType > data_real( x0_dim );
  bicotiDataContainer< bicotiDataContainer< bicotiPixelComplex< PixelType >, PixelType >, PixelType > data_transformed( x0_dim );
  
  bicotiImageIterator< PixelType > * ptr_iter_src;
  bicotiImageIterator< bicotiPixelComplex< PixelType > > * ptr_iter_dest;
  unsigned index_0, index_1;
  PixelType zero_pix;
  zero_pix *= 0;
  
  for( index_0 = 0; index_0 < x0_dim; index_0++ )
    data_transformed( index_0 ) = bicotiDataContainer< bicotiPixelComplex< PixelType >, PixelType >( x1_dim );
  if( ( x0_dim != ptr_dest->GetX0Size() ) || ( x1_dim != ptr_dest->GetX1Size() ) )
    {if( ptr_extrapol_strat_dest == NULL )
      throw bicotiException( "bicotiTransformationFourier2D", "Antitransform", 6 );
    ptr_extrapol_strat_dest->SetImplementation( ptr_dest );
    for( index_1 = 0; index_1 < x1_dim; index_1++ )
      for( index_0 = 0; index_0 < x0_dim; index_0++ )
	data_transformed( index_0 )( index_1 ) = (*ptr_extrapol_strat_dest)( index_0, index_1 );}
  else
    {ptr_iter_dest = ptr_dest->CreateIterator();
    ptr_iter_dest->First();
    for( index_1 = 0; index_1 < x1_dim; index_1++ )
      for( index_0 = 0; index_0 < x0_dim; index_0++ )
	{data_transformed( index_0 )( index_1 ) = **ptr_iter_dest;
	(*ptr_iter_dest)++;}
    delete ptr_iter_dest;}
  
  for( index_0 = 0; index_0 < x0_dim; index_0++ )
    FFT( data_real( index_0 ), data_transformed( index_0 ), false );
  
  FFT( data_transformed, data_real, false );
  
  ptr_src->Resize( x0_dim, x1_dim );

  ptr_iter_src = ptr_src->CreateIterator();  
  for( index_1 = 0; index_1 < x1_dim; index_1++ )
    for( index_0 = 0; index_0 < x0_dim; index_0++ )
      {**ptr_iter_src = data_transformed( index_0 )( index_1 ).Real();
      (*ptr_iter_src)++;}
  delete ptr_iter_src;
};


template< class PixelType >
bicotiImageImplementation2D< PixelType > * bicotiTransformationFourier2D< PixelType > :: GetTransformedModule()
{
  bicotiImageDescriptor * ptr_desc = ptr_dest->CreateImageDescriptor();
  bicotiImageImplementation2D< PixelType > * ptr_result = (bicotiImageImplementation2D< PixelType > *)(ptr_src->CreateImageImplementation( ptr_desc ));
  delete ptr_desc;

  bicotiImageIterator< PixelType > * ptr_iter_result = ptr_result->CreateIterator();
  bicotiImageIterator< bicotiPixelComplex < PixelType > > * ptr_iter_dest = ptr_dest->CreateIterator();
  
  for( ptr_iter_result->First(), ptr_iter_dest->First(); !ptr_iter_dest->IsDone(); (*ptr_iter_result)++, (*ptr_iter_dest)++ )
    **ptr_iter_result = (**ptr_iter_dest).Module();
  
  delete ptr_iter_result;
  delete ptr_iter_dest;
  
  return( ptr_result );
};


template< class PixelType >
bicotiImageImplementation2D< PixelType > * bicotiTransformationFourier2D< PixelType > :: GetTransformedArgument()
{
  bicotiImageDescriptor * ptr_desc = ptr_dest->CreateImageDescriptor();
  bicotiImageImplementation2D< PixelType > * ptr_result = (bicotiImageImplementation2D< PixelType > *)(ptr_src->CreateImageImplementation( ptr_desc ));
  delete ptr_desc;

  bicotiImageIterator< PixelType > * ptr_iter_result = ptr_result->CreateIterator();
  bicotiImageIterator< bicotiPixelComplex < PixelType > > * ptr_iter_dest = ptr_dest->CreateIterator();
  
  for( ptr_iter_result->First(), ptr_iter_dest->First(); !ptr_iter_dest->IsDone(); (*ptr_iter_result)++, (*ptr_iter_dest)++ )
    **ptr_iter_result = (**ptr_iter_dest).Argument();
  
  delete ptr_iter_result;
  delete ptr_iter_dest;
  
  return( ptr_result );
};        



#endif
